@@ -17,14 +17,11 @@ from typing import (
     Dict,
     Generic,
     Hashable,
-    Iterable,
     Iterator,
-    List,
     MutableSequence,
     NoReturn,
     Optional,
     Sequence,
-    Tuple,
     TypedDict,
     TypeVar,
     Union,
@@ -34,7 +31,7 @@ from typing import (
 from google.protobuf.message import Message
 from sortedcontainers import SortedDict, SortedList
 
-from cognite.client.data_classes.datapoints import NUMPY_IS_AVAILABLE, Datapoints, DatapointsArray
+from cognite.client.data_classes.datapoints import NUMPY_IS_AVAILABLE, Aggregate, Datapoints, DatapointsArray
 from cognite.client.utils._auxiliary import import_legacy_protobuf, is_unlimited
 from cognite.client.utils._identifier import Identifier
 from cognite.client.utils._text import convert_all_keys_to_snake_case, to_camel_case, to_snake_case
@@ -50,8 +47,8 @@ if not import_legacy_protobuf():
     from cognite.client._proto.data_point_list_response_pb2 import DataPointListItem
     from cognite.client._proto.data_points_pb2 import AggregateDatapoint, NumericDatapoint, StringDatapoint
 else:
-    from cognite.client._proto_legacy.data_point_list_response_pb2 import DataPointListItem  # type: ignore [misc]
-    from cognite.client._proto_legacy.data_points_pb2 import (  # type: ignore [misc]
+    from cognite.client._proto_legacy.data_point_list_response_pb2 import DataPointListItem  # type: ignore [assignment]
+    from cognite.client._proto_legacy.data_points_pb2 import (  # type: ignore [assignment]
         AggregateDatapoint,
         NumericDatapoint,
         StringDatapoint,
@@ -80,13 +77,13 @@ DatapointsExternalId = Union[None, str, Dict[str, Any], Sequence[Union[str, Dict
 
 class CustomDatapointsQuery(TypedDict, total=False):
     # No field required
-    start: Union[int, str, datetime, None]
-    end: Union[int, str, datetime, None]
-    aggregates: Optional[List[str]]
-    granularity: Optional[str]
-    limit: Optional[int]
-    include_outside_points: Optional[bool]
-    ignore_unknown_ids: Optional[bool]
+    start: int | str | datetime | None
+    end: int | str | datetime | None
+    aggregates: list[str] | None
+    granularity: str | None
+    limit: int | None
+    include_outside_points: bool | None
+    ignore_unknown_ids: bool | None
 
 
 class DatapointsQueryId(CustomDatapointsQuery):
@@ -101,27 +98,27 @@ class CustomDatapoints(TypedDict, total=False):
     # No field required
     start: int
     end: int
-    aggregates: Optional[List[str]]
-    granularity: Optional[str]
+    aggregates: list[str] | None
+    granularity: str | None
     limit: int
     include_outside_points: bool
 
 
 class DatapointsPayload(CustomDatapoints):
-    items: List[CustomDatapoints]
+    items: list[CustomDatapoints]
 
 
 @dataclass
 class _DatapointsQuery:
     """Internal representation of a user request for datapoints, previously public (before v5)"""
 
-    start: Union[int, str, datetime, None] = None
-    end: Union[int, str, datetime, None] = None
-    id: Optional[DatapointsId] = None
-    external_id: Optional[DatapointsExternalId] = None
-    aggregates: Union[str, List[str], None] = None
-    granularity: Optional[str] = None
-    limit: Optional[int] = None
+    start: int | str | datetime | None = None
+    end: int | str | datetime | None = None
+    id: DatapointsId | None = None
+    external_id: DatapointsExternalId | None = None
+    aggregates: Aggregate | str | list[Aggregate | str] | None = None
+    granularity: str | None = None
+    limit: int | None = None
     include_outside_points: bool = False
     ignore_unknown_ids: bool = False
 
@@ -141,7 +138,7 @@ class _SingleTSQueryValidator:
         self.user_query = user_query
         self.dps_limit_raw = dps_limit_raw
         self.dps_limit_agg = dps_limit_agg
-        self.defaults: Dict[str, Any] = dict(
+        self.defaults: dict[str, Any] = dict(
             start=user_query.start,
             end=user_query.end,
             limit=user_query.limit,
@@ -155,7 +152,7 @@ class _SingleTSQueryValidator:
         # exception 'end not after start' if both are set to the same value.
         self.__time_now = timestamp_to_ms("now")
 
-    def _ts_to_ms_frozen_now(self, ts: Union[int, str, datetime, None], default: int) -> int:
+    def _ts_to_ms_frozen_now(self, ts: int | str | datetime | None, default: int) -> int:
         # Time 'now' is frozen for all queries in a single call from the user, leading to identical
         # results e.g. "4d-ago" and "now"
         if ts is None:
@@ -165,7 +162,7 @@ class _SingleTSQueryValidator:
         else:
             return timestamp_to_ms(ts)
 
-    def validate_and_create_single_queries(self) -> List[_SingleTSQueryBase]:
+    def validate_and_create_single_queries(self) -> list[_SingleTSQueryBase]:
         queries = []
         if self.user_query.id is not None:
             id_queries = self._validate_multiple_id(self.user_query.id)
@@ -177,16 +174,16 @@ class _SingleTSQueryValidator:
             return queries
         raise ValueError("Pass at least one time series `id` or `external_id`!")
 
-    def _validate_multiple_id(self, id: DatapointsId) -> List[_SingleTSQueryBase]:
+    def _validate_multiple_id(self, id: DatapointsId) -> list[_SingleTSQueryBase]:
         return self._validate_id_or_xid(id, "id", numbers.Integral)
 
-    def _validate_multiple_xid(self, external_id: DatapointsExternalId) -> List[_SingleTSQueryBase]:
+    def _validate_multiple_xid(self, external_id: DatapointsExternalId) -> list[_SingleTSQueryBase]:
         return self._validate_id_or_xid(external_id, "external_id", str)
 
     def _validate_id_or_xid(
-        self, id_or_xid: Union[DatapointsId, DatapointsExternalId], arg_name: str, exp_type: type
-    ) -> List[_SingleTSQueryBase]:
-        id_or_xid_seq: Sequence[Union[int, str, Dict[str, Any]]]
+        self, id_or_xid: DatapointsId | DatapointsExternalId, arg_name: str, exp_type: type
+    ) -> list[_SingleTSQueryBase]:
+        id_or_xid_seq: Sequence[int | str | dict[str, Any]]
         if isinstance(id_or_xid, (dict, exp_type)):
             # Lazy - we postpone evaluation:
             id_or_xid_seq = [cast(Union[int, str, Dict[str, Any]], id_or_xid)]
@@ -206,7 +203,7 @@ class _SingleTSQueryValidator:
 
             elif isinstance(ts, dict):
                 ts_validated = self._validate_user_supplied_dict_keys(ts, arg_name)
-                if not isinstance(identifier := ts_validated[arg_name], exp_type):  # type: ignore [literal-required]
+                if not isinstance(identifier := ts_validated[arg_name], exp_type):
                     self._raise_on_wrong_ts_identifier_type(identifier, arg_name, exp_type)
                 # We merge 'defaults' and given ts-dict, ts-dict takes precedence:
                 ts_dct = {**self.defaults, **ts_validated}
@@ -230,7 +227,7 @@ class _SingleTSQueryValidator:
         )
 
     @staticmethod
-    def _validate_user_supplied_dict_keys(dct: Dict[str, Any], arg_name: str) -> Dict[str, Any]:
+    def _validate_user_supplied_dict_keys(dct: dict[str, Any], arg_name: str) -> dict[str, Any]:
         if arg_name not in dct:
             if (arg_name_cc := to_camel_case(arg_name)) not in dct:
                 raise KeyError(f"Missing required key `{arg_name}` in dict: {dct}.")
@@ -254,9 +251,7 @@ class _SingleTSQueryValidator:
             )
         return dct
 
-    def _validate_and_create_query(
-        self, dct: Union[DatapointsQueryId, DatapointsQueryExternalId]
-    ) -> _SingleTSQueryBase:
+    def _validate_and_create_query(self, dct: DatapointsQueryId | DatapointsQueryExternalId) -> _SingleTSQueryBase:
         limit = self._verify_limit(dct["limit"])
         granularity, aggregates = dct["granularity"], dct["aggregates"]
 
@@ -294,10 +289,10 @@ class _SingleTSQueryValidator:
 
     def _convert_parameters(
         self,
-        dct: Union[DatapointsQueryId, DatapointsQueryExternalId],
-        limit: Optional[int],
+        dct: DatapointsQueryId | DatapointsQueryExternalId,
+        limit: int | None,
         is_raw: bool,
-    ) -> Dict[str, Any]:
+    ) -> dict[str, Any]:
         identifier = Identifier.of_either(
             cast(Optional[int], dct.get("id")), cast(Optional[str], dct.get("external_id"))
         )
@@ -318,7 +313,7 @@ class _SingleTSQueryValidator:
             converted["granularity"] = dct["granularity"]
         return converted
 
-    def _verify_limit(self, limit: Optional[int]) -> Optional[int]:
+    def _verify_limit(self, limit: int | None) -> int | None:
         if is_unlimited(limit):
             return None
         elif isinstance(limit, numbers.Integral) and limit >= 0:  # limit=0 is accepted by the API
@@ -334,12 +329,12 @@ class _SingleTSQueryValidator:
 
     def _verify_time_range(
         self,
-        start: Union[int, str, datetime, None],
-        end: Union[int, str, datetime, None],
-        granularity: Optional[str],
+        start: int | str | datetime | None,
+        end: int | str | datetime | None,
+        granularity: str | None,
         is_raw: bool,
         identifier: Identifier,
-    ) -> Tuple[int, int]:
+    ) -> tuple[int, int]:
         start = self._ts_to_ms_frozen_now(start, default=0)  # 1970-01-01
         end = self._ts_to_ms_frozen_now(end, default=self.__time_now)
 
@@ -361,7 +356,7 @@ class _SingleTSQueryBase:
         start: int,
         end: int,
         max_query_limit: int,
-        limit: Optional[int],
+        limit: int | None,
         include_outside_points: bool,
         ignore_unknown_ids: bool,
     ) -> None:
@@ -373,8 +368,8 @@ class _SingleTSQueryBase:
         self.include_outside_points = include_outside_points
         self.ignore_unknown_ids = ignore_unknown_ids
 
-        self.granularity: Optional[str] = None
-        self._is_missing: Optional[bool] = None
+        self.granularity: str | None = None
+        self._is_missing: bool | None = None
 
         if self.include_outside_points and self.limit is not None:
             warnings.warn(
@@ -406,7 +401,7 @@ class _SingleTSQueryBase:
         ...
 
     @abstractmethod
-    def to_payload(self) -> Dict[str, Any]:
+    def to_payload(self) -> dict[str, Any]:
         raise NotImplementedError
 
     @property
@@ -431,7 +426,7 @@ class _SingleTSQueryRaw(_SingleTSQueryBase):
     def is_raw_query(self) -> bool:
         return True
 
-    def to_payload(self) -> Dict[str, Any]:
+    def to_payload(self) -> dict[str, Any]:
         return {
             **self.identifier.as_dict(),
             "start": self.start,
@@ -461,7 +456,7 @@ class _SingleTSQueryRawUnlimited(_SingleTSQueryRaw):
 
 
 class _SingleTSQueryAgg(_SingleTSQueryBase):
-    def __init__(self, *, aggregates: List[str], granularity: str, **kwargs: Any) -> None:
+    def __init__(self, *, aggregates: list[str], granularity: str, **kwargs: Any) -> None:
         super().__init__(**kwargs, include_outside_points=False)
         self.aggregates = aggregates
         self.granularity = granularity
@@ -471,10 +466,10 @@ class _SingleTSQueryAgg(_SingleTSQueryBase):
         return False
 
     @cached_property
-    def aggregates_cc(self) -> List[str]:
+    def aggregates_cc(self) -> list[str]:
         return list(map(to_camel_case, self.aggregates))
 
-    def to_payload(self) -> Dict[str, Any]:
+    def to_payload(self) -> dict[str, Any]:
         return {
             **self.identifier.as_dict(),
             "start": self.start,
@@ -508,11 +503,11 @@ class _SingleTSQueryAggUnlimited(_SingleTSQueryAgg):
 class DpsUnpackFns:
     ts: Callable[[Message], int] = op.attrgetter("timestamp")
     raw_dp: Callable[[Message], RawDatapointValue] = op.attrgetter("value")
-    ts_dp_tpl: Callable[[Message], Tuple[int, RawDatapointValue]] = op.attrgetter("timestamp", "value")
+    ts_dp_tpl: Callable[[Message], tuple[int, RawDatapointValue]] = op.attrgetter("timestamp", "value")
     count: Callable[[Message], int] = op.attrgetter("count")
 
     @staticmethod
-    def custom_from_aggregates(lst: List[str]) -> Callable[[DatapointsAgg], Tuple[float, ...]]:
+    def custom_from_aggregates(lst: list[str]) -> Callable[[DatapointsAgg], tuple[float, ...]]:
         return op.attrgetter(*lst)
 
 
@@ -521,7 +516,7 @@ FIRST_IDX = (0,)
 
 
 class DefaultSortedDict(SortedDict, Generic[T]):
-    def __init__(self, default_factory: Callable[[], T], /, **kw: Any):
+    def __init__(self, default_factory: Callable[[], T], /, **kw: Any) -> None:
         self.default_factory = default_factory
         super().__init__(**kw)
 
@@ -553,7 +548,7 @@ def get_datapoints_from_proto(res: DataPointListItem) -> DatapointsAny:
     return cast(MutableSequence[Any], [])
 
 
-def get_ts_info_from_proto(res: DataPointListItem) -> Dict[str, Union[int, str, bool]]:
+def get_ts_info_from_proto(res: DataPointListItem) -> dict[str, int | str | bool]:
     return {
         "id": res.id,
         "external_id": res.externalId,
@@ -567,16 +562,16 @@ def create_array_from_dps_container(container: DefaultSortedDict) -> npt.NDArray
     return np.hstack(list(chain.from_iterable(container.values())))
 
 
-def create_aggregates_arrays_from_dps_container(container: DefaultSortedDict, n_aggs: int) -> List[npt.NDArray]:
+def create_aggregates_arrays_from_dps_container(container: DefaultSortedDict, n_aggs: int) -> list[npt.NDArray]:
     all_aggs_arr = np.vstack(list(chain.from_iterable(container.values())))
     return list(map(np.ravel, np.hsplit(all_aggs_arr, n_aggs)))
 
 
-def create_list_from_dps_container(container: DefaultSortedDict) -> List:
+def create_list_from_dps_container(container: DefaultSortedDict) -> list:
     return list(chain.from_iterable(chain.from_iterable(container.values())))
 
 
-def create_aggregates_list_from_dps_container(container: DefaultSortedDict) -> Iterator[List[List]]:
+def create_aggregates_list_from_dps_container(container: DefaultSortedDict) -> Iterator[list[list]]:
     concatenated = chain.from_iterable(chain.from_iterable(container.values()))
     return map(list, zip(*concatenated))  # rows to columns
 
@@ -602,15 +597,15 @@ class BaseDpsFetchSubtask:
 
         self.is_done = False
 
-    def get_remaining_limit(self) -> Optional[int]:
+    def get_remaining_limit(self) -> int | None:
         return self.parent.get_remaining_limit(self)
 
     @abstractmethod
-    def get_next_payload(self) -> Optional[CustomDatapoints]:
+    def get_next_payload(self) -> CustomDatapoints | None:
         ...
 
     @abstractmethod
-    def store_partial_result(self, res: DataPointListItem) -> Optional[List[SplittingFetchSubtask]]:
+    def store_partial_result(self, res: DataPointListItem) -> list[SplittingFetchSubtask] | None:
         ...
 
 
@@ -618,10 +613,10 @@ class OutsideDpsFetchSubtask(BaseDpsFetchSubtask):
     """Fetches outside points and stores in parent"""
 
     def __init__(self, **kwargs: Any) -> None:
-        outside_dps_settings: Dict[str, Any] = dict(priority=0, is_raw_query=True, max_query_limit=0)
+        outside_dps_settings: dict[str, Any] = dict(priority=0, is_raw_query=True, max_query_limit=0)
         super().__init__(**kwargs, **outside_dps_settings)
 
-    def get_next_payload(self) -> Optional[CustomDatapoints]:
+    def get_next_payload(self) -> CustomDatapoints | None:
         if self.is_done:
             return None
         return self._create_payload_item()
@@ -629,7 +624,7 @@ class OutsideDpsFetchSubtask(BaseDpsFetchSubtask):
     def _create_payload_item(self) -> CustomDatapoints:
         return CustomDatapoints(
             {
-                **self.identifier.as_dict(),  # type: ignore [misc]
+                **self.identifier.as_dict(),  # type: ignore [typeddict-item]
                 "start": self.start,
                 "end": self.end,
                 "limit": 0,  # Not a bug; it just returns the outside points
@@ -652,10 +647,10 @@ class SerialFetchSubtask(BaseDpsFetchSubtask):
     def __init__(
         self,
         *,
-        limit: Optional[int],
-        aggregates: Optional[List[str]],
-        granularity: Optional[str],
-        subtask_idx: Tuple[float, ...],
+        limit: int | None,
+        aggregates: list[str] | None,
+        granularity: str | None,
+        subtask_idx: tuple[float, ...],
         **kwargs: Any,
     ) -> None:
         super().__init__(**kwargs)
@@ -670,7 +665,7 @@ class SerialFetchSubtask(BaseDpsFetchSubtask):
         if not self.is_raw_query:
             self.agg_kwargs = {"aggregates": self.aggregates, "granularity": self.granularity}
 
-    def get_next_payload(self) -> Optional[CustomDatapoints]:
+    def get_next_payload(self) -> CustomDatapoints | None:
         if self.is_done:
             return None
         remaining = self.get_remaining_limit()
@@ -690,7 +685,7 @@ class SerialFetchSubtask(BaseDpsFetchSubtask):
     def _create_payload_item(self, remaining_limit: float) -> CustomDatapoints:
         return CustomDatapoints(
             {
-                **self.identifier.as_dict(),  # type: ignore [misc]
+                **self.identifier.as_dict(),  # type: ignore [typeddict-item]
                 "start": self.next_start,
                 "end": self.end,
                 "limit": min(remaining_limit, self.max_query_limit),
@@ -698,7 +693,7 @@ class SerialFetchSubtask(BaseDpsFetchSubtask):
             }
         )
 
-    def store_partial_result(self, res: DataPointListItem) -> Optional[List[SplittingFetchSubtask]]:
+    def store_partial_result(self, res: DataPointListItem) -> list[SplittingFetchSubtask] | None:
         if self.parent.ts_info is None:
             # In eager mode, first task to complete gets the honor to store ts info:
             self.parent._store_ts_info(res)
@@ -724,14 +719,15 @@ class SerialFetchSubtask(BaseDpsFetchSubtask):
 
 class SplittingFetchSubtask(SerialFetchSubtask):
     """Fetches data serially, but splits its time domain ("divide and conquer") based on the density
-    of returned datapoints. Stores data in parent"""
+    of returned datapoints. Stores data in parent.
+    """
 
     def __init__(self, *, max_splitting_factor: int = 10, **kwargs: Any) -> None:
         super().__init__(**kwargs)
         self.max_splitting_factor = max_splitting_factor
         self.split_subidx: int = 0  # Actual value doesnt matter (any int will do)
 
-    def store_partial_result(self, res: DataPointListItem) -> Optional[List[SplittingFetchSubtask]]:
+    def store_partial_result(self, res: DataPointListItem) -> list[SplittingFetchSubtask] | None:
         self.prev_start = self.next_start
         super().store_partial_result(res)
         if not self.is_done:
@@ -739,16 +735,16 @@ class SplittingFetchSubtask(SerialFetchSubtask):
             return self._split_self_into_new_subtasks_if_needed(last_ts)
         return None
 
-    def _create_subtasks_idxs(self, n_new_tasks: int) -> Iterable[Tuple[float, ...]]:
-        """Since this task may decide to split itself multiple times, we count backwards to keep order
-        (we rely on tuple sorting logic). Example using `self.subtask_idx=(4,)`:
-        - First split into e.g. 3 parts: (4,-3), (4,-2), (4,-1)
-        - Next, split into 2: (4, -5) and (4, -4). These now sort before the first split."""
+    def _create_subtasks_idxs(self, n_new_tasks: int) -> Iterator[tuple[float, ...]]:
+        # Since this task may decide to split itself multiple times, we count backwards to keep order
+        # (we rely on tuple sorting logic). Example using `self.subtask_idx=(4,)`:
+        # - First split into e.g. 3 parts: (4,-3), (4,-2), (4,-1)
+        # - Next, split into 2: (4, -5) and (4, -4). These now sort before the first split.
         end = self.split_subidx
         self.split_subidx -= n_new_tasks
         yield from ((*self.subtask_idx, i) for i in range(self.split_subidx, end))
 
-    def _split_self_into_new_subtasks_if_needed(self, last_ts: int) -> Optional[List[SplittingFetchSubtask]]:
+    def _split_self_into_new_subtasks_if_needed(self, last_ts: int) -> list[SplittingFetchSubtask] | None:
         # How many new tasks because of % of time range was fetched?
         tot_ms = self.end - (start := self.prev_start)
         part_ms = last_ts - start
@@ -765,7 +761,7 @@ class SplittingFetchSubtask(SerialFetchSubtask):
         # Find a `delta_ms` thats a multiple of granularity in ms (trivial for raw queries):
         boundaries = split_time_range(last_ts, self.end, n_new_tasks, self.parent.offset_next)
         self.end = boundaries[1]  # We shift end of 'self' backwards
-        static_params: Dict[str, Any] = {
+        static_params: dict[str, Any] = {
             "parent": self.parent,
             "priority": self.priority,
             "identifier": self.identifier,
@@ -789,18 +785,18 @@ class BaseConcurrentTask:
         query: Any,  # subclasses assert correct type
         eager_mode: bool,
         use_numpy: bool,
-        first_dps_batch: Optional[DataPointListItem] = None,
-        first_limit: Optional[int] = None,
+        first_dps_batch: DataPointListItem | None = None,
+        first_limit: int | None = None,
     ) -> None:
         self.query = query
         self.eager_mode = eager_mode
         self.use_numpy = use_numpy
-        self.ts_info: Optional[Dict] = None
+        self.ts_info: dict | None = None
         self.ts_data = create_dps_container()
         self.dps_data = create_dps_container()
         self.subtasks = create_subtask_lst()
-        self.subtask_outside_points: Optional[OutsideDpsFetchSubtask] = None
-        self.raw_dtype: Optional[type] = None
+        self.subtask_outside_points: OutsideDpsFetchSubtask | None = None
+        self.raw_dtype: type | None = None
         self._is_done = False
         self.lock = Lock()
 
@@ -817,7 +813,7 @@ class BaseConcurrentTask:
             self._store_first_batch(dps, first_limit)
 
     @property
-    def ts_info_dct(self) -> Dict[str, Any]:
+    def ts_info_dct(self) -> dict[str, Any]:
         # This is mostly for mypy to avoid 'cast' x 10000, but also a nice check to make sure
         # we have the required ts info before returning a result dps object.
         assert self.ts_info is not None
@@ -861,11 +857,11 @@ class BaseConcurrentTask:
         ...
 
     @abstractmethod
-    def get_result(self) -> Union[Datapoints, DatapointsArray]:
+    def get_result(self) -> Datapoints | DatapointsArray:
         ...
 
     @abstractmethod
-    def _unpack_and_store(self, idx: Tuple[float, ...], dps: DatapointsAny) -> None:
+    def _unpack_and_store(self, idx: tuple[float, ...], dps: DatapointsAny) -> None:
         ...
 
     @abstractmethod
@@ -876,14 +872,14 @@ class BaseConcurrentTask:
     def _find_number_of_subtasks_uniform_split(self, tot_ms: int, n_workers_per_queries: int) -> int:
         ...
 
-    def split_into_subtasks(self, max_workers: int, n_tot_queries: int) -> List[BaseDpsFetchSubtask]:
+    def split_into_subtasks(self, max_workers: int, n_tot_queries: int) -> list[BaseDpsFetchSubtask]:
         # Given e.g. a single time series, we want to put all our workers to work by splitting into lots of pieces!
         # As the number grows - or we start combining multiple into the same query - we want to split less:
         # we hold back to not create too many subtasks:
         if self.is_done:
             return []
         n_workers_per_queries = max(1, round(max_workers / n_tot_queries))
-        subtasks: List[BaseDpsFetchSubtask] = self._create_uniformly_split_subtasks(n_workers_per_queries)
+        subtasks: list[BaseDpsFetchSubtask] = self._create_uniformly_split_subtasks(n_workers_per_queries)
         self.subtasks.update(subtasks)
         if self.eager_mode and self.query.include_outside_points:
             # In eager mode we do not get the "first dps batch" to extract outside points from:
@@ -897,7 +893,7 @@ class BaseConcurrentTask:
             subtasks.append(self.subtask_outside_points)
         return subtasks
 
-    def _create_uniformly_split_subtasks(self, n_workers_per_queries: int) -> List[BaseDpsFetchSubtask]:
+    def _create_uniformly_split_subtasks(self, n_workers_per_queries: int) -> list[BaseDpsFetchSubtask]:
         start = self.query.start if self.eager_mode else self.first_start
         tot_ms = (end := self.query.end) - start
         n_periods = self._find_number_of_subtasks_uniform_split(tot_ms, n_workers_per_queries)
@@ -942,7 +938,7 @@ class BaseConcurrentTask:
         elif len(dps) < first_limit:
             self._is_done = True
 
-    def get_remaining_limit(self, subtask: BaseDpsFetchSubtask) -> Optional[int]:
+    def get_remaining_limit(self, subtask: BaseDpsFetchSubtask) -> int | None:
         if not self.has_limit:
             return None
         # For limited queries: if the sum of fetched points of earlier tasks have already hit/surpassed
@@ -1003,15 +999,15 @@ class ConcurrentLimitedMixin(BaseConcurrentTask):
 
 class BaseConcurrentRawTask(BaseConcurrentTask):
     def __init__(self, **kwargs: Any) -> None:
-        self.dp_outside_start: Optional[Tuple[int, RawDatapointValue]] = None
-        self.dp_outside_end: Optional[Tuple[int, RawDatapointValue]] = None
+        self.dp_outside_start: tuple[int, RawDatapointValue] | None = None
+        self.dp_outside_end: tuple[int, RawDatapointValue] | None = None
         super().__init__(**kwargs)
 
     @property
     def offset_next(self) -> int:
         return 1  # 1 ms
 
-    def _create_empty_result(self) -> Union[Datapoints, DatapointsArray]:
+    def _create_empty_result(self) -> Datapoints | DatapointsArray:
         if not self.use_numpy:
             return Datapoints(**self.ts_info_dct, timestamp=[], value=[])
         return DatapointsArray._load(
@@ -1025,7 +1021,7 @@ class BaseConcurrentRawTask(BaseConcurrentTask):
     def _was_any_data_fetched(self) -> bool:
         return any((self.ts_data, self.dp_outside_start, self.dp_outside_end))
 
-    def get_result(self) -> Union[Datapoints, DatapointsArray]:
+    def get_result(self) -> Datapoints | DatapointsArray:
         if not self._was_any_data_fetched():
             return self._create_empty_result()
         if self.has_limit:
@@ -1077,15 +1073,15 @@ class BaseConcurrentRawTask(BaseConcurrentTask):
     def _include_outside_points_in_result(self) -> None:
         for dp, idx in zip((self.dp_outside_start, self.dp_outside_end), (-math.inf, math.inf)):
             if dp:
-                ts: Union[List[int], NumpyInt64Array] = [dp[0]]
-                value: Union[List[Union[float, str]], NumpyFloat64Array, NumpyObjArray] = [dp[1]]
+                ts: list[int] | NumpyInt64Array = [dp[0]]
+                value: list[float | str] | NumpyFloat64Array | NumpyObjArray = [dp[1]]
                 if self.use_numpy:
                     ts = np.array(ts, dtype=np.int64)
                     value = np.array(value, dtype=self.raw_dtype)
                 self.ts_data[(idx,)].append(ts)
                 self.dps_data[(idx,)].append(value)
 
-    def _unpack_and_store(self, idx: Tuple[float, ...], dps: DatapointsRaw) -> None:  # type: ignore [override]
+    def _unpack_and_store(self, idx: tuple[float, ...], dps: DatapointsRaw) -> None:  # type: ignore [override]
         if self.use_numpy:  # Faster than feeding listcomp to np.array:
             self.ts_data[idx].append(np.fromiter(map(DpsUnpackFns.ts, dps), dtype=np.int64, count=len(dps)))
             self.dps_data[idx].append(np.fromiter(map(DpsUnpackFns.raw_dp, dps), dtype=self.raw_dtype, count=len(dps)))
@@ -1144,7 +1140,7 @@ class BaseConcurrentAggTask(BaseConcurrentTask):
     def offset_next(self) -> int:
         return granularity_to_ms(self.query.granularity)
 
-    def _set_aggregate_vars(self, aggregates_cc: List[str], use_numpy: bool) -> None:
+    def _set_aggregate_vars(self, aggregates_cc: list[str], use_numpy: bool) -> None:
         # Developer note here: If you ask for datapoints to be returned in JSON, you get `count` as an integer.
         # Nice. However, when using protobuf, you get `double` xD ...so when this code was pivoted from
         # JSON -> protobuf, the special handling of `count` was kept in the hopes that one day protobuf
@@ -1172,7 +1168,7 @@ class BaseConcurrentAggTask(BaseConcurrentTask):
         n_max_dps = tot_ms // self.offset_next  # evenly divides
         return min(n_workers_per_queries, math.ceil(n_max_dps / self.query.max_query_limit))
 
-    def _create_empty_result(self) -> Union[Datapoints, DatapointsArray]:
+    def _create_empty_result(self) -> Datapoints | DatapointsArray:
         if self.use_numpy:
             arr_dct = {"timestamp": np.array([], dtype=np.int64)}
             if self.is_count_query:
@@ -1181,14 +1177,14 @@ class BaseConcurrentAggTask(BaseConcurrentTask):
                 arr_dct.update({agg: np.array([], dtype=np.float64) for agg in self.float_aggs})
             return DatapointsArray._load({**self.ts_info_dct, **arr_dct})
 
-        lst_dct: Dict[str, List] = {"timestamp": []}
+        lst_dct: dict[str, list] = {"timestamp": []}
         if self.is_count_query:
             lst_dct["count"] = []
         if self.has_non_count_aggs:
             lst_dct.update({agg: [] for agg in self.float_aggs})
         return Datapoints(**self.ts_info_dct, **convert_all_keys_to_snake_case(lst_dct))
 
-    def get_result(self) -> Union[Datapoints, DatapointsArray]:
+    def get_result(self) -> Datapoints | DatapointsArray:
         if not self.ts_data or self.query.limit == 0:
             return self._create_empty_result()
         if self.has_limit:
@@ -1235,13 +1231,13 @@ class BaseConcurrentAggTask(BaseConcurrentTask):
                     setattr(self, attr, dict(data.items()[: i + 1]))  # regular dict (no further inserts)
                 return None
 
-    def _unpack_and_store(self, idx: Tuple[float, ...], dps: DatapointsAgg) -> None:  # type: ignore [override]
+    def _unpack_and_store(self, idx: tuple[float, ...], dps: DatapointsAgg) -> None:  # type: ignore [override]
         if self.use_numpy:
             self._unpack_and_store_numpy(idx, dps)
         else:
             self._unpack_and_store_basic(idx, dps)
 
-    def _unpack_and_store_numpy(self, idx: Tuple[float, ...], dps: DatapointsAgg) -> None:
+    def _unpack_and_store_numpy(self, idx: tuple[float, ...], dps: DatapointsAgg) -> None:
         n = len(dps)
         self.ts_data[idx].append(np.fromiter(map(DpsUnpackFns.ts, dps), dtype=np.int64, count=n))
 
@@ -1263,7 +1259,7 @@ class BaseConcurrentAggTask(BaseConcurrentTask):
                 )
             self.dps_data[idx].append(arr.reshape(n, len(self.float_aggs)))
 
-    def _unpack_and_store_basic(self, idx: Tuple[float, ...], dps: DatapointsAgg) -> None:
+    def _unpack_and_store_basic(self, idx: tuple[float, ...], dps: DatapointsAgg) -> None:
         self.ts_data[idx].append(list(map(DpsUnpackFns.ts, dps)))
 
         if self.is_count_query:
@@ -1272,7 +1268,7 @@ class BaseConcurrentAggTask(BaseConcurrentTask):
 
         if self.has_non_count_aggs:
             try:
-                lst: List[Any] = list(map(self.agg_unpack_fn, dps))  # type: ignore [arg-type]
+                lst: list[Any] = list(map(self.agg_unpack_fn, dps))  # type: ignore [arg-type]
             except AttributeError:
                 if self.single_non_count_agg:
                     lst = [getattr(dp, self.first_non_count_agg, None) for dp in dps]
