@@ -1,10 +1,10 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Dict, Optional, Sequence, Union, overload
+from typing import TYPE_CHECKING, Any, Literal, Sequence, overload
 
 from cognite.client import utils
 from cognite.client._api_client import APIClient
-from cognite.client._constants import LIST_LIMIT_DEFAULT
+from cognite.client._constants import DEFAULT_LIMIT_READ
 from cognite.client.data_classes import (
     ExtractionPipeline,
     ExtractionPipelineConfig,
@@ -27,20 +27,20 @@ if TYPE_CHECKING:
 class ExtractionPipelinesAPI(APIClient):
     _RESOURCE_PATH = "/extpipes"
 
-    def __init__(self, config: ClientConfig, api_version: Optional[str], cognite_client: CogniteClient) -> None:
+    def __init__(self, config: ClientConfig, api_version: str | None, cognite_client: CogniteClient) -> None:
         super().__init__(config, api_version, cognite_client)
         self.runs = ExtractionPipelineRunsAPI(config, api_version, cognite_client)
         self.config = ExtractionPipelineConfigsAPI(config, api_version, cognite_client)
 
-    def retrieve(self, id: Optional[int] = None, external_id: Optional[str] = None) -> Optional[ExtractionPipeline]:
-        """`Retrieve a single extraction pipeline by id. <https://docs.cognite.com/api/v1/#operation/showExtPipe>`_
+    def retrieve(self, id: int | None = None, external_id: str | None = None) -> ExtractionPipeline | None:
+        """`Retrieve a single extraction pipeline by id. <https://developer.cognite.com/api#tag/Extraction-Pipelines/operation/showExtPipe>`_
 
         Args:
-            id (int, optional): ID
-            external_id (str, optional): External ID
+            id (int | None): ID
+            external_id (str | None): External ID
 
         Returns:
-            Optional[ExtractionPipeline]: Requested extraction pipeline or None if it does not exist.
+            ExtractionPipeline | None: Requested extraction pipeline or None if it does not exist.
 
         Examples:
 
@@ -64,15 +64,15 @@ class ExtractionPipelinesAPI(APIClient):
 
     def retrieve_multiple(
         self,
-        ids: Optional[Sequence[int]] = None,
-        external_ids: Optional[Sequence[str]] = None,
+        ids: Sequence[int] | None = None,
+        external_ids: Sequence[str] | None = None,
         ignore_unknown_ids: bool = False,
     ) -> ExtractionPipelineList:
-        """`Retrieve multiple extraction pipelines by ids and external ids. <https://docs.cognite.com/api/v1/#operation/byidsExtPipes>`_
+        """`Retrieve multiple extraction pipelines by ids and external ids. <https://developer.cognite.com/api#tag/Extraction-Pipelines/operation/byidsExtPipes>`_
 
         Args:
-            ids (Sequence[int], optional): IDs
-            external_ids (Sequence[str], optional): External IDs
+            ids (Sequence[int] | None): IDs
+            external_ids (Sequence[str] | None): External IDs
             ignore_unknown_ids (bool): Ignore IDs and external IDs that are not found rather than throw an exception.
 
         Returns:
@@ -100,12 +100,11 @@ class ExtractionPipelinesAPI(APIClient):
             ignore_unknown_ids=ignore_unknown_ids,
         )
 
-    def list(self, limit: int = LIST_LIMIT_DEFAULT) -> ExtractionPipelineList:
-        """`List extraction pipelines <https://docs.cognite.com/api/v1/#operation/listExtPipes>`_
+    def list(self, limit: int | None = DEFAULT_LIMIT_READ) -> ExtractionPipelineList:
+        """`List extraction pipelines <https://developer.cognite.com/api#tag/Extraction-Pipelines/operation/listExtPipes>`_
 
         Args:
-            limit (int, optional): Maximum number of ExtractionPipelines to return. Defaults to 25. Set to -1, float("inf") or None
-                to return all items.
+            limit (int | None): Maximum number of ExtractionPipelines to return. Defaults to 25. Set to -1, float("inf") or None to return all items.
 
         Returns:
             ExtractionPipelineList: List of requested ExtractionPipelines
@@ -130,17 +129,17 @@ class ExtractionPipelinesAPI(APIClient):
         ...
 
     def create(
-        self, extraction_pipeline: Union[ExtractionPipeline, Sequence[ExtractionPipeline]]
-    ) -> Union[ExtractionPipeline, ExtractionPipelineList]:
-        """`Create one or more extraction pipelines. <https://docs.cognite.com/api/v1/#operation/createExtPipes>`_
+        self, extraction_pipeline: ExtractionPipeline | Sequence[ExtractionPipeline]
+    ) -> ExtractionPipeline | ExtractionPipelineList:
+        """`Create one or more extraction pipelines. <https://developer.cognite.com/api#tag/Extraction-Pipelines/operation/createExtPipes>`_
 
         You can create an arbitrary number of extraction pipelines, and the SDK will split the request into multiple requests if necessary.
 
         Args:
-            extraction_pipeline (Union[ExtractionPipeline, List[ExtractionPipeline]]): Extraction pipeline or list of extraction pipelines to create.
+            extraction_pipeline (ExtractionPipeline | Sequence[ExtractionPipeline]): Extraction pipeline or list of extraction pipelines to create.
 
         Returns:
-            Union[ExtractionPipeline, ExtractionPipelineList]: Created extraction pipeline(s)
+            ExtractionPipeline | ExtractionPipelineList: Created extraction pipeline(s)
 
         Examples:
 
@@ -157,15 +156,12 @@ class ExtractionPipelinesAPI(APIClient):
             list_cls=ExtractionPipelineList, resource_cls=ExtractionPipeline, items=extraction_pipeline
         )
 
-    def delete(self, id: Union[int, Sequence[int]] = None, external_id: Union[str, Sequence[str]] = None) -> None:
-        """`Delete one or more extraction pipelines <https://docs.cognite.com/api/v1/#operation/deleteExtPipes>`_
+    def delete(self, id: int | Sequence[int] | None = None, external_id: str | Sequence[str] | None = None) -> None:
+        """`Delete one or more extraction pipelines <https://developer.cognite.com/api#tag/Extraction-Pipelines/operation/deleteExtPipes>`_
 
         Args:
-            id (Union[int, Sequence[int]): Id or list of ids
-            external_id (Union[str, Sequence[str]]): External ID or list of external ids
-
-        Returns:
-            None
+            id (int | Sequence[int] | None): Id or list of ids
+            external_id (str | Sequence[str] | None): External ID or list of external ids
 
         Examples:
 
@@ -178,26 +174,24 @@ class ExtractionPipelinesAPI(APIClient):
         self._delete_multiple(identifiers=IdentifierSequence.load(id, external_id), wrap_ids=True, extra_body_fields={})
 
     @overload
-    def update(self, item: Union[ExtractionPipeline, ExtractionPipelineUpdate]) -> ExtractionPipeline:
+    def update(self, item: ExtractionPipeline | ExtractionPipelineUpdate) -> ExtractionPipeline:
         ...
 
     @overload
-    def update(self, item: Sequence[Union[ExtractionPipeline, ExtractionPipelineUpdate]]) -> ExtractionPipelineList:
+    def update(self, item: Sequence[ExtractionPipeline | ExtractionPipelineUpdate]) -> ExtractionPipelineList:
         ...
 
     def update(
         self,
-        item: Union[
-            ExtractionPipeline, ExtractionPipelineUpdate, Sequence[Union[ExtractionPipeline, ExtractionPipelineUpdate]]
-        ],
-    ) -> Union[ExtractionPipeline, ExtractionPipelineList]:
-        """`Update one or more extraction pipelines <https://docs.cognite.com/api/v1/#operation/updateExtPipes>`_
+        item: ExtractionPipeline | ExtractionPipelineUpdate | Sequence[ExtractionPipeline | ExtractionPipelineUpdate],
+    ) -> ExtractionPipeline | ExtractionPipelineList:
+        """`Update one or more extraction pipelines <https://developer.cognite.com/api#tag/Extraction-Pipelines/operation/updateExtPipes>`_
 
         Args:
-            item (Union[ExtractionPipeline, ExtractionPipelineUpdate, Sequence[Union[ExtractionPipeline, ExtractionPipelineUpdate]]]): Extraction pipeline(s) to update
+            item (ExtractionPipeline | ExtractionPipelineUpdate | Sequence[ExtractionPipeline | ExtractionPipelineUpdate]): Extraction pipeline(s) to update
 
         Returns:
-            Union[ExtractionPipeline, ExtractionPipelineList]: Updated extraction pipeline(s)
+            ExtractionPipeline | ExtractionPipelineList: Updated extraction pipeline(s)
 
         Examples:
 
@@ -219,25 +213,23 @@ class ExtractionPipelinesAPI(APIClient):
 
 class ExtractionPipelineRunsAPI(APIClient):
     _RESOURCE_PATH = "/extpipes/runs"
-    _LIST_CLASS = ExtractionPipelineRunList
 
     def list(
         self,
         external_id: str,
-        statuses: Sequence[str] = None,
-        message_substring: str = None,
-        created_time: Union[Dict[str, Any], TimestampRange] = None,
-        limit: int = LIST_LIMIT_DEFAULT,
+        statuses: Sequence[str] | None = None,
+        message_substring: str | None = None,
+        created_time: dict[str, Any] | TimestampRange | None = None,
+        limit: int | None = DEFAULT_LIMIT_READ,
     ) -> ExtractionPipelineRunList:
-        """`List runs for an extraction pipeline with given external_id <https://docs.cognite.com/api/v1/#operation/filterRuns>`_
+        """`List runs for an extraction pipeline with given external_id <https://developer.cognite.com/api#tag/Extraction-Pipelines/operation/filterRuns>`_
 
         Args:
             external_id (str): Extraction pipeline external Id.
-            statuses (Sequence[str]): One or more among "success" / "failure" / "seen".
-            message_substring (str): Failure message part.
-            created_time (int): The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.
-            limit (int, optional): Maximum number of ExtractionPipelines to return. Defaults to 25. Set to -1, float("inf") or None
-                to return all items.
+            statuses (Sequence[str] | None): One or more among "success" / "failure" / "seen".
+            message_substring (str | None): Failure message part.
+            created_time (dict[str, Any] | TimestampRange | None): The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.
+            limit (int | None): Maximum number of ExtractionPipelines to return. Defaults to 25. Set to -1, float("inf") or None to return all items.
 
         Returns:
             ExtractionPipelineRunList: List of requested extraction pipeline runs
@@ -264,21 +256,21 @@ class ExtractionPipelineRunsAPI(APIClient):
                 message=StringFilter(substring=message_substring),
                 created_time=created_time,
             ).dump(camel_case=True)
-            return self._list(
-                list_cls=ExtractionPipelineRunList,
-                resource_cls=ExtractionPipelineRun,
-                method="POST",
-                limit=limit,
-                filter=filter,
-            )
+            method: Literal["POST", "GET"] = "POST"
+        else:
+            filter = {"externalId": external_id}
+            method = "GET"
 
-        return self._list(
+        res = self._list(
             list_cls=ExtractionPipelineRunList,
             resource_cls=ExtractionPipelineRun,
-            method="GET",
+            method=method,
             limit=limit,
-            filter={"externalId": external_id},
+            filter=filter,
         )
+        for run in res:
+            run.extpipe_external_id = external_id
+        return res
 
     @overload
     def create(self, run: ExtractionPipelineRun) -> ExtractionPipelineRun:
@@ -289,17 +281,17 @@ class ExtractionPipelineRunsAPI(APIClient):
         ...
 
     def create(
-        self, run: Union[ExtractionPipelineRun, Sequence[ExtractionPipelineRun]]
-    ) -> Union[ExtractionPipelineRun, ExtractionPipelineRunList]:
-        """`Create one or more extraction pipeline runs. <https://docs.cognite.com/api/v1/#operation/createRuns>`_
+        self, run: ExtractionPipelineRun | Sequence[ExtractionPipelineRun]
+    ) -> ExtractionPipelineRun | ExtractionPipelineRunList:
+        """`Create one or more extraction pipeline runs. <https://developer.cognite.com/api#tag/Extraction-Pipelines/operation/createRuns>`_
 
         You can create an arbitrary number of extraction pipeline runs, and the SDK will split the request into multiple requests.
 
         Args:
-            run (Union[ExtractionPipelineRun, Sequence[ExtractionPipelineRun]]): Extraction pipeline or list of extraction pipeline runs to create.
+            run (ExtractionPipelineRun | Sequence[ExtractionPipelineRun]): Extraction pipeline or list of extraction pipeline runs to create.
 
         Returns:
-            Union[ExtractionPipelineRun, ExtractionPipelineRunList]: Created extraction pipeline run(s)
+            ExtractionPipelineRun | ExtractionPipelineRunList: Created extraction pipeline run(s)
 
         Examples:
 
@@ -317,19 +309,18 @@ class ExtractionPipelineRunsAPI(APIClient):
 
 class ExtractionPipelineConfigsAPI(APIClient):
     _RESOURCE_PATH = "/extpipes/config"
-    _LIST_CLASS = ExtractionPipelineConfigRevisionList
 
     def retrieve(
-        self, external_id: str, revision: Optional[int] = None, active_at_time: Optional[int] = None
+        self, external_id: str, revision: int | None = None, active_at_time: int | None = None
     ) -> ExtractionPipelineConfig:
-        """`Retrieve a specific configuration revision, or the latest by default <https://docs.cognite.com/api/v1/#tag/Extraction-Pipelines-Config/operation/getExtPipeConfigRevision>`
+        """`Retrieve a specific configuration revision, or the latest by default <https://developer.cognite.com/api#tag/Extraction-Pipelines/operation/getExtPipeConfigRevision>`
 
         By default the latest configuration revision is retrieved, or you can specify a timestamp or a revision number.
 
         Args:
             external_id (str): External id of the extraction pipeline to retrieve config from.
-            revision (Optional[int]): Optionally specify a revision number to retrieve.
-            active_at_time (Optional[int]): Optionally specify a timestamp the configuration revision should be active.
+            revision (int | None): Optionally specify a revision number to retrieve.
+            active_at_time (int | None): Optionally specify a timestamp the configuration revision should be active.
 
         Returns:
             ExtractionPipelineConfig: Retrieved extraction pipeline configuration revision
@@ -348,7 +339,7 @@ class ExtractionPipelineConfigsAPI(APIClient):
         return ExtractionPipelineConfig._load(response.json(), cognite_client=self._cognite_client)
 
     def list(self, external_id: str) -> ExtractionPipelineConfigRevisionList:
-        """`Retrieve all configuration revisions from an extraction pipeline <https://docs.cognite.com/api/v1/#tag/Extraction-Pipelines-Config/operation/listExtPipeConfigRevisions>`
+        """`Retrieve all configuration revisions from an extraction pipeline <https://developer.cognite.com/api#tag/Extraction-Pipelines/operation/listExtPipeConfigRevisions>`
 
         Args:
             external_id (str): External id of the extraction pipeline to retrieve config from.
@@ -368,7 +359,7 @@ class ExtractionPipelineConfigsAPI(APIClient):
         return ExtractionPipelineConfigRevisionList._load(response.json()["items"], cognite_client=self._cognite_client)
 
     def create(self, config: ExtractionPipelineConfig) -> ExtractionPipelineConfig:
-        """`Create a new configuration revision <https://docs.cognite.com/api/v1/#tag/Extraction-Pipelines-Config/operation/createExtPipeConfig>`
+        """`Create a new configuration revision <https://developer.cognite.com/api#tag/Extraction-Pipelines/operation/createExtPipeConfig>`
 
         Args:
             config (ExtractionPipelineConfig): Configuration revision to create.
@@ -381,7 +372,7 @@ class ExtractionPipelineConfigsAPI(APIClient):
             Create a config revision::
 
                 >>> from cognite.client import CogniteClient
-                >>> from cognite.client.config import ExtractionPipelineConfig
+                >>> from cognite.client.data_classes import ExtractionPipelineConfig
                 >>> c = CogniteClient()
                 >>> res = c.extraction_pipelines.config.create(ExtractionPipelineConfig(external_id="extId", config="my config contents"))
         """
@@ -389,7 +380,7 @@ class ExtractionPipelineConfigsAPI(APIClient):
         return ExtractionPipelineConfig._load(response.json(), cognite_client=self._cognite_client)
 
     def revert(self, external_id: str, revision: int) -> ExtractionPipelineConfig:
-        """`Revert to a previous configuration revision <https://docs.cognite.com/api/v1/#tag/Extraction-Pipelines-Config/operation/createExtPipeConfig>`
+        """`Revert to a previous configuration revision <https://developer.cognite.com/api#tag/Extraction-Pipelines/operation/createExtPipeConfig>`
 
         Args:
             external_id (str): External id of the extraction pipeline to revert revision for.
