@@ -4,11 +4,12 @@ import json as complexjson
 import numbers
 import urllib.parse
 import warnings
-from typing import Any, Dict, Generator, List, Optional, Sequence, Union, cast, overload
+from typing import Any, Iterator, Sequence, cast, overload
 
 from requests.exceptions import ChunkedEncodingError
 
 from cognite.client._api_client import APIClient
+from cognite.client._constants import DEFAULT_LIMIT_READ
 from cognite.client.data_classes.geospatial import (
     CoordinateReferenceSystem,
     CoordinateReferenceSystemList,
@@ -58,17 +59,15 @@ class GeospatialAPI(APIClient):
     def create_feature_types(self, feature_type: Sequence[FeatureType]) -> FeatureTypeList:
         ...
 
-    def create_feature_types(
-        self, feature_type: Union[FeatureType, Sequence[FeatureType]]
-    ) -> Union[FeatureType, FeatureTypeList]:
+    def create_feature_types(self, feature_type: FeatureType | Sequence[FeatureType]) -> FeatureType | FeatureTypeList:
         """`Creates feature types`
-        <https://docs.cognite.com/api/v1/#operation/createFeatureTypes>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/createFeatureTypes>
 
         Args:
-            feature_type (Union[FeatureType, Sequence[FeatureType]]): feature type definition or list of feature type definitions to create.
+            feature_type (FeatureType | Sequence[FeatureType]): feature type definition or list of feature type definitions to create.
 
         Returns:
-            Union[FeatureType, FeatureTypeList]: Created feature type definition(s)
+            FeatureType | FeatureTypeList: Created feature type definition(s)
 
         Examples:
 
@@ -94,17 +93,14 @@ class GeospatialAPI(APIClient):
             resource_path=f"{self._RESOURCE_PATH}/featuretypes",
         )
 
-    def delete_feature_types(self, external_id: Union[str, Sequence[str]], recursive: bool = False) -> None:
+    def delete_feature_types(self, external_id: str | Sequence[str], recursive: bool = False) -> None:
         """`Delete one or more feature type`
-        <https://docs.cognite.com/api/v1/#operation/GeospatialDeleteFeatureTypes>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/GeospatialDeleteFeatureTypes>
 
         Args:
-            external_id (Union[str, Sequence[str]]): External ID or list of external ids
+            external_id (str | Sequence[str]): External ID or list of external ids
             recursive (bool): if `true` the features will also be dropped
 
-        Returns:
-            None
-
         Examples:
 
             Delete feature type definitions external id:
@@ -114,7 +110,7 @@ class GeospatialAPI(APIClient):
                 >>> c.geospatial.delete_feature_types(external_id=["wells", "cities"])
         """
         extra_body_fields = {"recursive": True} if recursive else {}
-        return self._delete_multiple(
+        self._delete_multiple(
             identifiers=IdentifierSequence.load(external_ids=external_id),
             wrap_ids=True,
             resource_path=f"{self._RESOURCE_PATH}/featuretypes",
@@ -123,7 +119,7 @@ class GeospatialAPI(APIClient):
 
     def list_feature_types(self) -> FeatureTypeList:
         """`List feature types`
-        <https://docs.cognite.com/api/v1/#operation/listFeatureTypes>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/listFeatureTypes>
 
         Returns:
             FeatureTypeList: List of feature types
@@ -149,18 +145,18 @@ class GeospatialAPI(APIClient):
         ...
 
     @overload
-    def retrieve_feature_types(self, external_id: List[str]) -> FeatureTypeList:
+    def retrieve_feature_types(self, external_id: list[str]) -> FeatureTypeList:
         ...
 
-    def retrieve_feature_types(self, external_id: Union[str, List[str]]) -> Union[FeatureType, FeatureTypeList]:
+    def retrieve_feature_types(self, external_id: str | list[str]) -> FeatureType | FeatureTypeList:
         """`Retrieve feature types`
-        <https://docs.cognite.com/api/v1/#operation/getFeatureTypesByIds>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/getFeatureTypesByIds>
 
         Args:
-            external_id (Union[str, List[str]]): External ID
+            external_id (str | list[str]): External ID
 
         Returns:
-            FeatureTypeList: Requested Type or None if it does not exist.
+            FeatureType | FeatureTypeList: Requested Type or None if it does not exist.
 
         Examples:
 
@@ -178,12 +174,12 @@ class GeospatialAPI(APIClient):
             resource_path=f"{self._RESOURCE_PATH}/featuretypes",
         )
 
-    def update_feature_types(self, update: Union[FeatureTypeUpdate, Sequence[FeatureTypeUpdate]]) -> FeatureTypeList:
+    def update_feature_types(self, update: FeatureTypeUpdate | Sequence[FeatureTypeUpdate]) -> FeatureTypeList:
         """`Update feature types (Deprecated)`
-        <https://docs.cognite.com/api/v1/#operation/updateFeatureTypes>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/updateFeatureTypes>
 
         Args:
-            update (Union[FeatureTypeUpdate, Sequence[FeatureTypeUpdate]]): the update to apply
+            update (FeatureTypeUpdate | Sequence[FeatureTypeUpdate]): the update to apply
 
         Returns:
             FeatureTypeList: The updated feature types.
@@ -221,7 +217,7 @@ class GeospatialAPI(APIClient):
         if isinstance(update, FeatureTypeUpdate):
             update = [update]
 
-        def mapper(it: FeatureTypeUpdate) -> Dict[str, Any]:
+        def mapper(it: FeatureTypeUpdate) -> dict[str, Any]:
             add_properties = it.add.properties if hasattr(it, "add") else None
             remove_properties = it.remove.properties if hasattr(it, "remove") else None
             add_search_spec = it.add.search_spec if hasattr(it, "add") else None
@@ -234,12 +230,12 @@ class GeospatialAPI(APIClient):
         res = self._post(url_path=f"{self._RESOURCE_PATH}/featuretypes/update", json=json)
         return FeatureTypeList._load(res.json()["items"], cognite_client=self._cognite_client)
 
-    def patch_feature_types(self, patch: Union[FeatureTypePatch, Sequence[FeatureTypePatch]]) -> FeatureTypeList:
+    def patch_feature_types(self, patch: FeatureTypePatch | Sequence[FeatureTypePatch]) -> FeatureTypeList:
         """`Patch feature types`
-        <https://docs.cognite.com/api/v1/#operation/updateFeatureTypes>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/updateFeatureTypes>
 
         Args:
-            patch (Union[FeatureTypePatch, Sequence[FeatureTypePatch]]): the patch to apply
+            patch (FeatureTypePatch | Sequence[FeatureTypePatch]): the patch to apply
 
         Returns:
             FeatureTypeList: The patched feature types.
@@ -296,7 +292,7 @@ class GeospatialAPI(APIClient):
         feature_type_external_id: str,
         feature: Feature,
         allow_crs_transformation: bool = False,
-        chunk_size: int = None,
+        chunk_size: int | None = None,
     ) -> Feature:
         ...
 
@@ -304,33 +300,30 @@ class GeospatialAPI(APIClient):
     def create_features(
         self,
         feature_type_external_id: str,
-        feature: Union[Sequence[Feature], FeatureList],
+        feature: Sequence[Feature] | FeatureList,
         allow_crs_transformation: bool = False,
-        chunk_size: int = None,
+        chunk_size: int | None = None,
     ) -> FeatureList:
         ...
 
     def create_features(
         self,
         feature_type_external_id: str,
-        feature: Union[Feature, Sequence[Feature], FeatureList],
+        feature: Feature | Sequence[Feature] | FeatureList,
         allow_crs_transformation: bool = False,
-        chunk_size: int = None,
-    ) -> Union[Feature, FeatureList]:
+        chunk_size: int | None = None,
+    ) -> Feature | FeatureList:
         """`Creates features`
-        <https://docs.cognite.com/api/v1/#operation/createFeatures>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/createFeatures>
 
         Args:
-            feature_type_external_id: Feature type definition for the features to create.
-            feature: one feature or a list of features to create or a FeatureList object
-            allow_crs_transformation: If true, then input geometries will be transformed into the Coordinate Reference
-                System defined in the feature type specification. When it is false, then requests with geometries in
-                Coordinate Reference System different from the ones defined in the feature type will result in
-                CogniteAPIError exception.
-            chunk_size: maximum number of items in a single request to the api
+            feature_type_external_id (str): Feature type definition for the features to create.
+            feature (Feature | Sequence[Feature] | FeatureList): one feature or a list of features to create or a FeatureList object
+            allow_crs_transformation (bool): If true, then input geometries will be transformed into the Coordinate Reference System defined in the feature type specification. When it is false, then requests with geometries in Coordinate Reference System different from the ones defined in the feature type will result in CogniteAPIError exception.
+            chunk_size (int | None): maximum number of items in a single request to the api
 
         Returns:
-            Union[Feature, FeatureList]: Created features
+            Feature | FeatureList: Created features
 
         Examples:
 
@@ -372,16 +365,13 @@ class GeospatialAPI(APIClient):
             limit=chunk_size,
         )
 
-    def delete_features(self, feature_type_external_id: str, external_id: Union[str, Sequence[str]] = None) -> None:
+    def delete_features(self, feature_type_external_id: str, external_id: str | Sequence[str] | None = None) -> None:
         """`Delete one or more feature`
-        <https://docs.cognite.com/api/v1/#operation/deleteFeatures>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/deleteFeatures>
 
         Args:
-            feature_type_external_id : Feature type external id for the features to delete.
-            external_id (Union[str, Sequence[str]]): External ID or list of external ids
-
-        Returns:
-            None
+            feature_type_external_id (str): No description.
+            external_id (str | Sequence[str] | None): External ID or list of external ids
 
         Examples:
 
@@ -404,7 +394,7 @@ class GeospatialAPI(APIClient):
         self,
         feature_type_external_id: str,
         external_id: str,
-        properties: Dict[str, Any] = None,
+        properties: dict[str, Any] | None = None,
     ) -> Feature:
         ...
 
@@ -412,27 +402,27 @@ class GeospatialAPI(APIClient):
     def retrieve_features(
         self,
         feature_type_external_id: str,
-        external_id: List[str],
-        properties: Dict[str, Any] = None,
+        external_id: list[str],
+        properties: dict[str, Any] | None = None,
     ) -> FeatureList:
         ...
 
     def retrieve_features(
         self,
         feature_type_external_id: str,
-        external_id: Union[str, List[str]],
-        properties: Dict[str, Any] = None,
-    ) -> Union[FeatureList, Feature]:
+        external_id: str | list[str],
+        properties: dict[str, Any] | None = None,
+    ) -> FeatureList | Feature:
         """`Retrieve features`
-        <https://docs.cognite.com/api/v1/#operation/getFeaturesByIds>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/getFeaturesByIds>
 
         Args:
-            feature_type_external_id : Feature type external id for the features to retrieve.
-            external_id (Union[str, List[str]]): External ID or list of external ids
-            properties (Dict[str, Any]): the output property selection
+            feature_type_external_id (str): No description.
+            external_id (str | list[str]): External ID or list of external ids
+            properties (dict[str, Any] | None): the output property selection
 
         Returns:
-            FeatureList: Requested features or None if it does not exist.
+            FeatureList | Feature: Requested features or None if it does not exist.
 
         Examples:
 
@@ -458,21 +448,18 @@ class GeospatialAPI(APIClient):
     def update_features(
         self,
         feature_type_external_id: str,
-        feature: Union[Feature, Sequence[Feature]],
+        feature: Feature | Sequence[Feature],
         allow_crs_transformation: bool = False,
-        chunk_size: int = None,
+        chunk_size: int | None = None,
     ) -> FeatureList:
         """`Update features`
-        <https://docs.cognite.com/api/v1/#operation/updateFeatures>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/updateFeatures>
 
         Args:
-            feature_type_external_id : Feature type definition for the features to update.
-            feature (Union[Feature, Sequence[Feature]]): feature or list of features.
-            allow_crs_transformation: If true, then input geometries will be transformed into the Coordinate Reference
-                System defined in the feature type specification. When it is false, then requests with geometries in
-                Coordinate Reference System different from the ones defined in the feature type will result in
-                CogniteAPIError exception.
-            chunk_size: maximum number of items in a single request to the api
+            feature_type_external_id (str): No description.
+            feature (Feature | Sequence[Feature]): feature or list of features.
+            allow_crs_transformation (bool): If true, then input geometries will be transformed into the Coordinate Reference System defined in the feature type specification. When it is false, then requests with geometries in Coordinate Reference System different from the ones defined in the feature type will result in CogniteAPIError exception.
+            chunk_size (int | None): maximum number of items in a single request to the api
 
         Returns:
             FeatureList: Updated features
@@ -515,26 +502,22 @@ class GeospatialAPI(APIClient):
     def list_features(
         self,
         feature_type_external_id: str,
-        filter: Optional[Dict[str, Any]] = None,
-        properties: Dict[str, Any] = None,
-        limit: int = 100,
+        filter: dict[str, Any] | None = None,
+        properties: dict[str, Any] | None = None,
+        limit: int | None = DEFAULT_LIMIT_READ,
         allow_crs_transformation: bool = False,
     ) -> FeatureList:
         """`List features`
-        <https://docs.cognite.com/api/v1/#operation/listFeatures>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/listFeatures>
 
         This method allows to filter all features.
 
         Args:
-            feature_type_external_id: the feature type to list features for
-            filter (Dict[str, Any]): the list filter
-            limit (int, optional): Maximum number of features to return. Defaults to 25. Set to -1, float("inf") or None
-                to return all features.
-            properties (Dict[str, Any]): the output property selection
-            allow_crs_transformation: If true, then input geometries if existing in the filter will be transformed into
-                the Coordinate Reference System defined in the feature type specification. When it is false, then
-                requests with geometries in Coordinate Reference System different from the ones defined in the feature
-                type will result in CogniteAPIError exception.
+            feature_type_external_id (str): the feature type to list features for
+            filter (dict[str, Any] | None): the list filter
+            properties (dict[str, Any] | None): the output property selection
+            limit (int | None): Maximum number of features to return. Defaults to 25. Set to -1, float("inf") or None to return all features.
+            allow_crs_transformation (bool): If true, then input geometries if existing in the filter will be transformed into the Coordinate Reference System defined in the feature type specification. When it is false, then requests with geometries in Coordinate Reference System different from the ones defined in the feature type will result in CogniteAPIError exception.
 
         Returns:
             FeatureList: The filtered features
@@ -597,29 +580,26 @@ class GeospatialAPI(APIClient):
     def search_features(
         self,
         feature_type_external_id: str,
-        filter: Optional[Dict[str, Any]] = None,
-        properties: Dict[str, Any] = None,
-        limit: int = 100,
-        order_by: Sequence[OrderSpec] = None,
+        filter: dict[str, Any] | None = None,
+        properties: dict[str, Any] | None = None,
+        limit: int = DEFAULT_LIMIT_READ,
+        order_by: Sequence[OrderSpec] | None = None,
         allow_crs_transformation: bool = False,
     ) -> FeatureList:
         """`Search for features`
-        <https://docs.cognite.com/api/v1/#operation/searchFeatures>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/searchFeatures>
 
         This method allows to order the result by one or more of the properties of the feature type.
-        However, the number of items returned is  limited to 1000 and there is no support for cursors yet.
+        However, the number of items returned is limited to 1000 and there is no support for cursors yet.
         If you need to return more than 1000 items, use the `stream_features(...)` method instead.
 
         Args:
-            feature_type_external_id: the feature type to search for
-            filter (Dict[str, Any]): the search filter
-            limit (int): maximum number of results
-            properties (Dict[str, Any]): the output property selection
-            order_by (Sequence[OrderSpec]): the order specification
-            allow_crs_transformation: If true, then input geometries will be transformed into the Coordinate Reference
-                System defined in the feature type specification. When it is false, then requests with geometries in
-                Coordinate Reference System different from the ones defined in the feature type will result in
-                CogniteAPIError exception.
+            feature_type_external_id (str): The feature type to search for
+            filter (dict[str, Any] | None): The search filter
+            properties (dict[str, Any] | None): The output property selection
+            limit (int): Maximum number of results
+            order_by (Sequence[OrderSpec] | None): The order specification
+            allow_crs_transformation (bool): If true, then input geometries will be transformed into the Coordinate Reference System defined in the feature type specification. When it is false, then requests with geometries in Coordinate Reference System different from the ones defined in the feature type will result in CogniteAPIError exception.
 
         Returns:
             FeatureList: the filtered features
@@ -718,28 +698,25 @@ class GeospatialAPI(APIClient):
     def stream_features(
         self,
         feature_type_external_id: str,
-        filter: Optional[Dict[str, Any]] = None,
-        properties: Dict[str, Any] = None,
+        filter: dict[str, Any] | None = None,
+        properties: dict[str, Any] | None = None,
         allow_crs_transformation: bool = False,
-    ) -> Generator[Feature, None, None]:
+    ) -> Iterator[Feature]:
         """`Stream features`
-        <https://docs.cognite.com/api/v1/#operation/searchFeaturesStreaming>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/searchFeaturesStreaming>
 
         This method allows to return any number of items until the underlying
         api calls times out. The order of the result items is not deterministic.
         If you need to order the results, use the `search_features(...)` method instead.
 
         Args:
-            feature_type_external_id: the feature type to search for
-            filter (Dict[str, Any]): the search filter
-            properties (Dict[str, Any]): the output property selection
-            allow_crs_transformation: If true, then input geometries will be transformed into the Coordinate Reference
-                System defined in the feature type specification. When it is false, then requests with geometries in
-                Coordinate Reference System different from the ones defined in the feature type will result in
-                CogniteAPIError exception.
+            feature_type_external_id (str): the feature type to search for
+            filter (dict[str, Any] | None): the search filter
+            properties (dict[str, Any] | None): the output property selection
+            allow_crs_transformation (bool): If true, then input geometries will be transformed into the Coordinate Reference System defined in the feature type specification. When it is false, then requests with geometries in Coordinate Reference System different from the ones defined in the feature type will result in CogniteAPIError exception.
 
-        Returns:
-            Generator[Feature]: a generator for the filtered features
+        Yields:
+            Feature: a generator for the filtered features
 
         Examples:
 
@@ -767,7 +744,6 @@ class GeospatialAPI(APIClient):
                 ... )
                 >>> for f in features:
                 ...     # do something with the features
-
         """
         resource_path = self._feature_resource_path(feature_type_external_id) + "/search-streaming"
         json = {"filter": filter or {}, "output": {"properties": properties, "jsonStreamFormat": "NEW_LINE_DELIMITED"}}
@@ -785,24 +761,24 @@ class GeospatialAPI(APIClient):
     def aggregate_features(
         self,
         feature_type_external_id: str,
-        property: str = None,
-        aggregates: Sequence[str] = None,
-        filter: Optional[Dict[str, Any]] = None,
-        group_by: Sequence[str] = None,
-        order_by: Sequence[OrderSpec] = None,
-        output: Dict[str, Any] = None,
+        property: str | None = None,
+        aggregates: Sequence[str] | None = None,
+        filter: dict[str, Any] | None = None,
+        group_by: Sequence[str] | None = None,
+        order_by: Sequence[OrderSpec] | None = None,
+        output: dict[str, Any] | None = None,
     ) -> FeatureAggregateList:
         """`Aggregate filtered features`
-        <https://docs.cognite.com/api/v1/#operation/aggregateFeatures>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/aggregateFeatures>
 
         Args:
-            feature_type_external_id: the feature type to filter features from
-            filter (Dict[str, Any]): the search filter
-            property (str): the property for which aggregates should be calculated
-            aggregates (Sequence[str]): list of aggregates to be calculated
-            group_by (Sequence[str]): list of properties to group by with
-            order_by (Sequence[OrderSpec]): the order specification
-            output (Dict[str, Any]): the aggregate output
+            feature_type_external_id (str): the feature type to filter features from
+            property (str | None): the property for which aggregates should be calculated
+            aggregates (Sequence[str] | None): list of aggregates to be calculated
+            filter (dict[str, Any] | None): the search filter
+            group_by (Sequence[str] | None): list of properties to group by with
+            order_by (Sequence[OrderSpec] | None): the order specification
+            output (dict[str, Any] | None): the aggregate output
 
         Returns:
             FeatureAggregateList: the filtered features
@@ -855,12 +831,12 @@ class GeospatialAPI(APIClient):
         )
         return cls._load(res.json()["items"], cognite_client=self._cognite_client)
 
-    def get_coordinate_reference_systems(self, srids: Union[int, Sequence[int]]) -> CoordinateReferenceSystemList:
+    def get_coordinate_reference_systems(self, srids: int | Sequence[int]) -> CoordinateReferenceSystemList:
         """`Get Coordinate Reference Systems`
-        <https://docs.cognite.com/api/v1/#operation/getCoordinateReferenceSystem>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/getCoordinateReferenceSystem>
 
         Args:
-            srids: (Union[int, Sequence[int]]): SRID or list of SRIDs
+            srids (int | Sequence[int]): (Union[int, Sequence[int]]): SRID or list of SRIDs
 
         Returns:
             CoordinateReferenceSystemList: Requested CRSs.
@@ -874,7 +850,7 @@ class GeospatialAPI(APIClient):
                 >>> crs = c.geospatial.get_coordinate_reference_systems(srids=[4326, 4327])
         """
         if isinstance(srids, (int, numbers.Integral)):
-            srids_processed: Sequence[Union[numbers.Integral, int]] = [srids]
+            srids_processed: Sequence[numbers.Integral | int] = [srids]
         else:
             srids_processed = srids
 
@@ -885,7 +861,7 @@ class GeospatialAPI(APIClient):
 
     def list_coordinate_reference_systems(self, only_custom: bool = False) -> CoordinateReferenceSystemList:
         """`List Coordinate Reference Systems`
-        <https://docs.cognite.com/api/v1/#operation/listGeospatialCoordinateReferenceSystems>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/listGeospatialCoordinateReferenceSystems>
 
         Args:
             only_custom (bool): list only custom CRSs or not
@@ -905,13 +881,13 @@ class GeospatialAPI(APIClient):
         return CoordinateReferenceSystemList._load(res.json()["items"], cognite_client=self._cognite_client)
 
     def create_coordinate_reference_systems(
-        self, crs: Union[CoordinateReferenceSystem, Sequence[CoordinateReferenceSystem]]
+        self, crs: CoordinateReferenceSystem | Sequence[CoordinateReferenceSystem]
     ) -> CoordinateReferenceSystemList:
         """`Create Coordinate Reference System`
-        <https://docs.cognite.com/api/v1/#operation/createGeospatialCoordinateReferenceSystems>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/createGeospatialCoordinateReferenceSystems>
 
         Args:
-            crs: a CoordinateReferenceSystem or a list of CoordinateReferenceSystem
+            crs (CoordinateReferenceSystem | Sequence[CoordinateReferenceSystem]): a CoordinateReferenceSystem or a list of CoordinateReferenceSystem
 
         Returns:
             CoordinateReferenceSystemList: list of CRSs.
@@ -965,15 +941,12 @@ class GeospatialAPI(APIClient):
         )
         return CoordinateReferenceSystemList._load(res.json()["items"], cognite_client=self._cognite_client)
 
-    def delete_coordinate_reference_systems(self, srids: Union[int, Sequence[int]]) -> None:
+    def delete_coordinate_reference_systems(self, srids: int | Sequence[int]) -> None:
         """`Delete Coordinate Reference System`
-        <https://docs.cognite.com/api/v1/#operation/deleteGeospatialCoordinateReferenceSystems>
+        <https://developer.cognite.com/api#tag/Geospatial/operation/deleteGeospatialCoordinateReferenceSystems>
 
         Args:
-            srids: (Union[int, Sequence[int]]): SRID or list of SRIDs
-
-        Returns:
-            None
+            srids (int | Sequence[int]): (Union[int, Sequence[int]]): SRID or list of SRIDs
 
         Examples:
 
@@ -984,7 +957,7 @@ class GeospatialAPI(APIClient):
                 >>> crs = c.geospatial.delete_coordinate_reference_systems(srids=[121111])
         """
         if isinstance(srids, (int, numbers.Integral)):
-            srids_processed: Sequence[Union[numbers.Integral, int]] = [srids]
+            srids_processed: Sequence[numbers.Integral | int] = [srids]
         else:
             srids_processed = srids
 
@@ -1001,23 +974,21 @@ class GeospatialAPI(APIClient):
         raster_srid: int,
         file: str,
         allow_crs_transformation: bool = False,
-        raster_scale_x: Optional[float] = None,
-        raster_scale_y: Optional[float] = None,
+        raster_scale_x: float | None = None,
+        raster_scale_y: float | None = None,
     ) -> RasterMetadata:
-        """`Put raster`
-        <https://docs.cognite.com/api/v1/#tag/Geospatial/operation/putRaster>
+        """`Put raster <https://developer.cognite.com/api#tag/Geospatial/operation/putRaster>`
 
         Args:
-            feature_type_external_id : Feature type definition for the features to create.
-            feature_external_id: one feature or a list of features to create
-            raster_property_name: the raster property name
-            raster_format: the raster input format
-            raster_srid: the associated SRID for the raster
-            file: the path to the file of the raster
-            allow_crs_transformation: When the parameter is false, requests with rasters in Coordinate Reference
-                System different from the one defined in the feature type will result in bad request response code.
-            raster_scale_x: the X component of the pixel width in units of coordinate reference system
-            raster_scale_y: the Y component of the pixel height in units of coordinate reference system
+            feature_type_external_id (str): No description.
+            feature_external_id (str): one feature or a list of features to create
+            raster_property_name (str): the raster property name
+            raster_format (str): the raster input format
+            raster_srid (int): the associated SRID for the raster
+            file (str): the path to the file of the raster
+            allow_crs_transformation (bool): When the parameter is false, requests with rasters in Coordinate Reference System different from the one defined in the feature type will result in bad request response code.
+            raster_scale_x (float | None): the X component of the pixel width in units of coordinate reference system
+            raster_scale_y (float | None): the Y component of the pixel height in units of coordinate reference system
 
         Returns:
             RasterMetadata: the raster metadata if it was ingested succesfully
@@ -1060,16 +1031,12 @@ class GeospatialAPI(APIClient):
         feature_external_id: str,
         raster_property_name: str,
     ) -> None:
-        """`Delete raster`
-        <https://docs.cognite.com/api/v1/#tag/Geospatial/operation/deleteRaster>
+        """`Delete raster <https://developer.cognite.com/api#tag/Geospatial/operation/deleteRaster>`
 
         Args:
-            feature_type_external_id : Feature type definition for the features to create.
-            feature_external_id: one feature or a list of features to create
-            raster_property_name: the raster property name
-
-        Returns:
-            None
+            feature_type_external_id (str): No description.
+            feature_external_id (str): one feature or a list of features to create
+            raster_property_name (str): the raster property name
 
         Examples:
 
@@ -1097,26 +1064,24 @@ class GeospatialAPI(APIClient):
         feature_external_id: str,
         raster_property_name: str,
         raster_format: str,
-        raster_options: Dict[str, Any] = None,
-        raster_srid: Optional[int] = None,
-        raster_scale_x: Optional[float] = None,
-        raster_scale_y: Optional[float] = None,
+        raster_options: dict[str, Any] | None = None,
+        raster_srid: int | None = None,
+        raster_scale_x: float | None = None,
+        raster_scale_y: float | None = None,
         allow_crs_transformation: bool = False,
     ) -> bytes:
-        """`Get raster`
-        <https://docs.cognite.com/api/v1/#tag/Geospatial/operation/getRaster>
+        """`Get raster <https://developer.cognite.com/api#tag/Geospatial/operation/getRaster>`
 
         Args:
-            feature_type_external_id : Feature type definition for the features to create.
-            feature_external_id: one feature or a list of features to create
-            raster_property_name: the raster property name
-            raster_format: the raster output format
-            raster_options: GDAL raster creation key-value options
-            raster_srid: the SRID for the output raster
-            raster_scale_x: the X component of the output pixel width in units of coordinate reference system
-            raster_scale_y: the Y component of the output pixel height in units of coordinate reference system
-            allow_crs_transformation: When the parameter is false, requests with output rasters in Coordinate Reference
-                System different from the one defined in the feature type will result in bad request response code.
+            feature_type_external_id (str): Feature type definition for the features to create.
+            feature_external_id (str): one feature or a list of features to create
+            raster_property_name (str): the raster property name
+            raster_format (str): the raster output format
+            raster_options (dict[str, Any] | None): GDAL raster creation key-value options
+            raster_srid (int | None): the SRID for the output raster
+            raster_scale_x (float | None): the X component of the output pixel width in units of coordinate reference system
+            raster_scale_y (float | None): the Y component of the output pixel height in units of coordinate reference system
+            allow_crs_transformation (bool): When the parameter is false, requests with output rasters in Coordinate Reference System different from the one defined in the feature type will result in bad request response code.
 
         Returns:
             bytes: the raster data
@@ -1151,16 +1116,15 @@ class GeospatialAPI(APIClient):
 
     def compute(
         self,
-        output: Dict[str, GeospatialComputeFunction],
+        output: dict[str, GeospatialComputeFunction],
     ) -> GeospatialComputedResponse:
-        """`Compute`
-        <https://docs.cognite.com/api/v1/#tag/Geospatial/operation/compute>
+        """`Compute <https://developer.cognite.com/api#tag/Geospatial/operation/compute>`
 
         Args:
-            output : Mapping of keys to compute functions.
+            output (dict[str, GeospatialComputeFunction]): No description.
 
         Returns:
-            dict: Mapping of keys to computed items.
+            GeospatialComputedResponse: Mapping of keys to computed items.
 
         Examples:
 
