@@ -1,12 +1,13 @@
 from __future__ import annotations
 
-from typing import Any, Awaitable, Dict, List, Optional, Sequence, Union
+from typing import TYPE_CHECKING, Any, Sequence
 
 from cognite.client._api.transformations.jobs import TransformationJobsAPI
 from cognite.client._api.transformations.notifications import TransformationNotificationsAPI
 from cognite.client._api.transformations.schedules import TransformationSchedulesAPI
 from cognite.client._api.transformations.schema import TransformationSchemaAPI
 from cognite.client._api_client import APIClient
+from cognite.client._constants import DEFAULT_LIMIT_READ
 from cognite.client.data_classes import Transformation, TransformationJob, TransformationList
 from cognite.client.data_classes.shared import TimestampRange
 from cognite.client.data_classes.transformations import (
@@ -17,13 +18,10 @@ from cognite.client.data_classes.transformations import (
 )
 from cognite.client.data_classes.transformations.common import NonceCredentials
 from cognite.client.utils._identifier import IdentifierSequence
-from typing import TYPE_CHECKING
-from cognite.client._constants import LIST_LIMIT_DEFAULT
-
 
 if TYPE_CHECKING:
-    from cognite.client.config import ClientConfig
     from cognite.client import CogniteClient
+    from cognite.client.config import ClientConfig
 
 __all__ = [
     "TransformationSchemaAPI",
@@ -36,25 +34,22 @@ __all__ = [
 
 class TransformationsAPI(APIClient):
     _RESOURCE_PATH = "/transformations"
-    _LIST_CLASS = TransformationList
 
-    def __init__(self, config: ClientConfig, api_version: Optional[str], cognite_client: CogniteClient) -> None:
+    def __init__(self, config: ClientConfig, api_version: str | None, cognite_client: CogniteClient) -> None:
         super().__init__(config, api_version, cognite_client)
         self.jobs = TransformationJobsAPI(config, api_version, cognite_client)
         self.schedules = TransformationSchedulesAPI(config, api_version, cognite_client)
         self.schema = TransformationSchemaAPI(config, api_version, cognite_client)
         self.notifications = TransformationNotificationsAPI(config, api_version, cognite_client)
 
-    def create(
-        self, transformation: Union[Transformation, Sequence[Transformation]]
-    ) -> Union[Transformation, TransformationList]:
-        """`Create one or more transformations. <https://docs.cognite.com/api/v1/#operation/createTransformations>`_
+    def create(self, transformation: Transformation | Sequence[Transformation]) -> Transformation | TransformationList:
+        """`Create one or more transformations. <https://developer.cognite.com/api#tag/Transformations/operation/createTransformations>`_
 
         Args:
-            transformation (Union[Transformation, List[Transformation]]): Transformation or list of transformations to create.
+            transformation (Transformation | Sequence[Transformation]): Transformation or list of transformations to create.
 
         Returns:
-            Created transformation(s)
+            Transformation | TransformationList: Created transformation(s)
 
         Examples:
 
@@ -62,7 +57,7 @@ class TransformationsAPI(APIClient):
 
                 >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import Transformation, TransformationDestination
-                >>> from cognite.client.data_classes.transformations.common import ViewInfo, EdgeType
+                >>> from cognite.client.data_classes.transformations.common import ViewInfo, EdgeType, DataModelInfo
                 >>> c = CogniteClient()
                 >>> transformations = [
                 >>>     Transformation(
@@ -76,24 +71,34 @@ class TransformationsAPI(APIClient):
                 >>>      Transformation(
                 >>>      name="transformation3",
                 >>>      view = ViewInfo(space="TypeSpace", external_id="TypeExtId", version="version"),
-                >>>      destination=TransformationDestination.instance_nodes(view, "InstanceSpace")
+                >>>      destination=TransformationDestination.nodes(view, "InstanceSpace")
                 >>>      ),
                 >>>      Transformation(
                 >>>      name="transformation4",
                 >>>      view = ViewInfo(space="TypeSpace", external_id="TypeExtId", version="version"),
-                >>>      destination=TransformationDestination.instance_edges(view, "InstanceSpace")
+                >>>      destination=TransformationDestination.edges(view, "InstanceSpace")
                 >>>      ),
                 >>>      Transformation(
                 >>>      name="transformation5",
                 >>>      edge_type = EdgeType(space="TypeSpace", external_id="TypeExtId"),
-                >>>      destination=TransformationDestination.instance_edges(edge_type,"InstanceSpace")
+                >>>      destination=TransformationDestination.edges(edge_type,"InstanceSpace")
+                >>>      ),
+                >>>      Transformation(
+                >>>      name="transformation6",
+                >>>      data_model = DataModelInfo(space="modelSpace", external_id="modelExternalId",version="modelVersion",destination_type="viewExternalId"),
+                >>>      destination=TransformationDestination.instances(data_model,"InstanceSpace")
+                >>>      ),
+                >>>      Transformation(
+                >>>      name="transformation7",
+                >>>      data_model = DataModelInfo(space="modelSpace", external_id="modelExternalId",version="modelVersion",destination_type="viewExternalId", destination_relationship_from_type="connectionPropertyName"),
+                >>>      destination=TransformationDestination.instances(data_model,"InstanceSpace")
                 >>>      ),
                 >>> ]
                 >>> res = c.transformations.create(transformations)
 
         """
         if isinstance(transformation, Sequence):
-            sessions: Dict[str, NonceCredentials] = {}
+            sessions: dict[str, NonceCredentials] = {}
             transformation = [t.copy() for t in transformation]
             for t in transformation:
                 t._cognite_client = self._cognite_client
@@ -109,20 +114,17 @@ class TransformationsAPI(APIClient):
 
     def delete(
         self,
-        id: Union[int, Sequence[int]] = None,
-        external_id: Union[str, Sequence[str]] = None,
+        id: int | Sequence[int] | None = None,
+        external_id: str | Sequence[str] | None = None,
         ignore_unknown_ids: bool = False,
     ) -> None:
-        """`Delete one or more transformations. <https://docs.cognite.com/api/v1/#operation/deleteTransformations>`_
+        """`Delete one or more transformations. <https://developer.cognite.com/api#tag/Transformations/operation/deleteTransformations>`_
 
         Args:
-            id (Union[int, List[int]): Id or list of ids.
-            external_id (Union[str, List[str]]): External ID or list of external ids.
+            id (int | Sequence[int] | None): Id or list of ids.
+            external_id (str | Sequence[str] | None): External ID or list of external ids.
             ignore_unknown_ids (bool): Ignore IDs and external IDs that are not found rather than throw an exception.
 
-        Returns:
-            None
-
         Example:
 
             Delete transformations by id or external id::
@@ -140,35 +142,35 @@ class TransformationsAPI(APIClient):
     def list(
         self,
         include_public: bool = True,
-        name_regex: str = None,
-        query_regex: str = None,
-        destination_type: str = None,
-        conflict_mode: str = None,
-        cdf_project_name: str = None,
-        has_blocked_error: bool = None,
-        created_time: Union[Dict[str, Any], TimestampRange] = None,
-        last_updated_time: Union[Dict[str, Any], TimestampRange] = None,
-        data_set_ids: List[int] = None,
-        data_set_external_ids: List[str] = None,
-        tags: Optional[TagsFilter] = None,
-        limit: Optional[int] = LIST_LIMIT_DEFAULT,
+        name_regex: str | None = None,
+        query_regex: str | None = None,
+        destination_type: str | None = None,
+        conflict_mode: str | None = None,
+        cdf_project_name: str | None = None,
+        has_blocked_error: bool | None = None,
+        created_time: dict[str, Any] | TimestampRange | None = None,
+        last_updated_time: dict[str, Any] | TimestampRange | None = None,
+        data_set_ids: list[int] | None = None,
+        data_set_external_ids: list[str] | None = None,
+        tags: TagsFilter | None = None,
+        limit: int | None = DEFAULT_LIMIT_READ,
     ) -> TransformationList:
-        """`List all transformations. <https://docs.cognite.com/api/v1/#operation/getTransformations>`_
+        """`List all transformations. <https://developer.cognite.com/api#tag/Transformations/operation/getTransformations>`_
 
         Args:
             include_public (bool): Whether public transformations should be included in the results. (default true).
-            name_regex (str): Regex expression to match the transformation name
-            query_regex (str): Regex expression to match the transformation query
-            destination_type (str): Transformation destination resource name to filter by.
-            conflict_mode (str): Filters by a selected transformation action type: abort/create, upsert, update, delete
-            cdf_project_name (str): Project name to filter by configured source and destination project
-            has_blocked_error (bool): Whether only the blocked transformations should be included in the results.
-            created_time (Union[Dict[str, Any], TimestampRange]): Range between two timestamps
-            last_updated_time (Union[Dict[str, Any], TimestampRange]): Range between two timestamps
-            data_set_ids (List[int]): Return only transformations in the specified data sets with these ids.
-            data_set_external_ids (List[str]): Return only transformations in the specified data sets with these external ids.
-            tags (TagsFilter): Return only the resource matching the specified tags constraints. It only supports ContainsAny as of now.
-            limit (int): Limits the number of results to be returned. To retrieve all results use limit=-1, default limit is 25.
+            name_regex (str | None): Regex expression to match the transformation name
+            query_regex (str | None): Regex expression to match the transformation query
+            destination_type (str | None): Transformation destination resource name to filter by.
+            conflict_mode (str | None): Filters by a selected transformation action type: abort/create, upsert, update, delete
+            cdf_project_name (str | None): Project name to filter by configured source and destination project
+            has_blocked_error (bool | None): Whether only the blocked transformations should be included in the results.
+            created_time (dict[str, Any] | TimestampRange | None): Range between two timestamps
+            last_updated_time (dict[str, Any] | TimestampRange | None): Range between two timestamps
+            data_set_ids (list[int] | None): Return only transformations in the specified data sets with these ids.
+            data_set_external_ids (list[str] | None): Return only transformations in the specified data sets with these external ids.
+            tags (TagsFilter | None): Return only the resource matching the specified tags constraints. It only supports ContainsAny as of now.
+            limit (int | None): Limits the number of results to be returned. To retrieve all results use limit=-1, default limit is 25.
 
         Returns:
             TransformationList: List of transformations
@@ -181,7 +183,7 @@ class TransformationsAPI(APIClient):
                 >>> c = CogniteClient()
                 >>> transformations_list = c.transformations.list()
         """
-        ds_ids: Optional[List[Dict[str, Any]]] = None
+        ds_ids: list[dict[str, Any]] | None = None
         if data_set_ids and data_set_external_ids:
             ds_ids = [*[{"id": i} for i in data_set_ids], *[{"externalId": i} for i in data_set_external_ids]]
         elif data_set_ids:
@@ -211,14 +213,15 @@ class TransformationsAPI(APIClient):
             filter=filter,
         )
 
-    def retrieve(self, id: Optional[int] = None, external_id: Optional[str] = None) -> Optional[Transformation]:
-        """`Retrieve a single transformation by id. <https://docs.cognite.com/api/v1/#operation/getTransformationsByIds>`_
+    def retrieve(self, id: int | None = None, external_id: str | None = None) -> Transformation | None:
+        """`Retrieve a single transformation by id. <https://developer.cognite.com/api#tag/Transformations/operation/getTransformationsByIds>`_
 
         Args:
-            id (int, optional): ID
+            id (int | None): ID
+            external_id (str | None): No description.
 
         Returns:
-            Optional[Transformation]: Requested transformation or None if it does not exist.
+            Transformation | None: Requested transformation or None if it does not exist.
 
         Examples:
 
@@ -242,13 +245,16 @@ class TransformationsAPI(APIClient):
         )
 
     def retrieve_multiple(
-        self, ids: Sequence[int] = None, external_ids: Sequence[str] = None, ignore_unknown_ids: bool = False
+        self,
+        ids: Sequence[int] | None = None,
+        external_ids: Sequence[str] | None = None,
+        ignore_unknown_ids: bool = False,
     ) -> TransformationList:
-        """`Retrieve multiple transformations. <https://docs.cognite.com/api/v1/#operation/getTransformationsByIds>`_
+        """`Retrieve multiple transformations. <https://developer.cognite.com/api#tag/Transformations/operation/getTransformationsByIds>`_
 
         Args:
-            ids (List[int]): List of ids to retrieve.
-            external_ids (List[str]): List of external ids to retrieve.
+            ids (Sequence[int] | None): List of ids to retrieve.
+            external_ids (Sequence[str] | None): List of external ids to retrieve.
             ignore_unknown_ids (bool): Ignore IDs and external IDs that are not found rather than throw an exception.
 
         Returns:
@@ -271,15 +277,15 @@ class TransformationsAPI(APIClient):
         )
 
     def update(
-        self, item: Union[Transformation, TransformationUpdate, Sequence[Union[Transformation, TransformationUpdate]]]
-    ) -> Union[Transformation, TransformationList]:
-        """`Update one or more transformations <https://docs.cognite.com/api/v1/#operation/updateTransformations>`_
+        self, item: Transformation | TransformationUpdate | Sequence[Transformation | TransformationUpdate]
+    ) -> Transformation | TransformationList:
+        """`Update one or more transformations <https://developer.cognite.com/api#tag/Transformations/operation/updateTransformations>`_
 
         Args:
-            item (Union[Transformation, TransformationUpdate, List[Union[Transformation, TransformationUpdate]]]): Transformation(s) to update
+            item (Transformation | TransformationUpdate | Sequence[Transformation | TransformationUpdate]): Transformation(s) to update
 
         Returns:
-            Union[Transformation, TransformationList]: Updated transformation(s)
+            Transformation | TransformationList: Updated transformation(s)
 
         Examples:
 
@@ -302,8 +308,8 @@ class TransformationsAPI(APIClient):
 
         if isinstance(item, Sequence):
             item = list(item).copy()
-            sessions: Dict[str, NonceCredentials] = {}
-            for (i, t) in enumerate(item):
+            sessions: dict[str, NonceCredentials] = {}
+            for i, t in enumerate(item):
                 if isinstance(t, Transformation):
                     t = t.copy()
                     item[i] = t
@@ -324,21 +330,21 @@ class TransformationsAPI(APIClient):
 
     def run(
         self,
-        transformation_id: int = None,
-        transformation_external_id: str = None,
+        transformation_id: int | None = None,
+        transformation_external_id: str | None = None,
         wait: bool = True,
-        timeout: Optional[float] = None,
+        timeout: float | None = None,
     ) -> TransformationJob:
-        """`Run a transformation. <https://docs.cognite.com/api/v1/#operation/runTransformation>`_
+        """`Run a transformation. <https://developer.cognite.com/api#tag/Transformations/operation/runTransformation>`_
 
         Args:
-            transformation_id (int): Transformation internal id
-            transformation_external_id (str): Transformation external id
+            transformation_id (int | None): Transformation internal id
+            transformation_external_id (str | None): Transformation external id
             wait (bool): Wait until the transformation run is finished. Defaults to True.
-            timeout (Optional[float]): maximum time (s) to wait, default is None (infinite time). Once the timeout is reached, it returns with the current status. Won't have any effect if wait is False.
+            timeout (float | None): maximum time (s) to wait, default is None (infinite time). Once the timeout is reached, it returns with the current status. Won't have any effect if wait is False.
 
         Returns:
-            Created transformation job
+            TransformationJob: Created transformation job
 
         Examples:
 
@@ -369,17 +375,20 @@ class TransformationsAPI(APIClient):
         return job
 
     async def run_async(
-        self, transformation_id: int = None, transformation_external_id: str = None, timeout: Optional[float] = None
+        self,
+        transformation_id: int | None = None,
+        transformation_external_id: str | None = None,
+        timeout: float | None = None,
     ) -> TransformationJob:
-        """`Run a transformation to completion asynchronously. <https://docs.cognite.com/api/v1/#operation/runTransformation>`_
+        """`Run a transformation to completion asynchronously. <https://developer.cognite.com/api#tag/Transformations/operation/runTransformation>`_
 
         Args:
-            transformation_id (int): internal Transformation id
-            transformation_external_id (str): external Transformation id
-            timeout (Optional[float]): maximum time (s) to wait, default is None (infinite time). Once the timeout is reached, it returns with the current status.
+            transformation_id (int | None): internal Transformation id
+            transformation_external_id (str | None): external Transformation id
+            timeout (float | None): maximum time (s) to wait, default is None (infinite time). Once the timeout is reached, it returns with the current status.
 
         Returns:
-            Completed (if finished) or running (if timeout reached) transformation job.
+            TransformationJob: Completed (if finished) or running (if timeout reached) transformation job.
 
         Examples:
 
@@ -403,12 +412,12 @@ class TransformationsAPI(APIClient):
         )
         return await job.wait_async(timeout=timeout)
 
-    def cancel(self, transformation_id: int = None, transformation_external_id: str = None) -> None:
-        """`Cancel a running transformation. <https://docs.cognite.com/api/v1/#operation/cancelTransformation>`_
+    def cancel(self, transformation_id: int | None = None, transformation_external_id: str | None = None) -> None:
+        """`Cancel a running transformation. <https://developer.cognite.com/api#tag/Transformations/operation/cancelTransformation>`_
 
         Args:
-            transformation_id (int): Transformation internal id
-            transformation_external_id (str): Transformation external id
+            transformation_id (int | None): Transformation internal id
+            transformation_external_id (str | None): Transformation external id
 
         Examples:
 
@@ -430,23 +439,23 @@ class TransformationsAPI(APIClient):
 
     def preview(
         self,
-        query: str = None,
+        query: str | None = None,
         convert_to_string: bool = False,
         limit: int = 100,
-        source_limit: Optional[int] = 100,
-        infer_schema_limit: Optional[int] = 1000,
+        source_limit: int | None = 100,
+        infer_schema_limit: int | None = 1000,
     ) -> TransformationPreviewResult:
-        """`Preview the result of a query. <https://docs.cognite.com/api/v1/#operation/runPreview>`_
+        """`Preview the result of a query. <https://developer.cognite.com/api#tag/Transformations/operation/runPreview>`_
 
         Args:
-            query (str): SQL query to run for preview.
+            query (str | None): SQL query to run for preview.
             convert_to_string (bool): Stringify values in the query results, default is False.
             limit (int): Maximum number of rows to return in the final result, default is 100.
-            source_limit (Union[int,str]): Maximum number of items to read from the data source or None to run without limit, default is 100.
-            infer_schema_limit: Limit for how many rows that are used for inferring result schema, default is 1000.
+            source_limit (int | None): Maximum number of items to read from the data source or None to run without limit, default is 100.
+            infer_schema_limit (int | None): Limit for how many rows that are used for inferring result schema, default is 1000.
 
         Returns:
-            Result of the executed query
+            TransformationPreviewResult: Result of the executed query
 
         Examples:
 
