@@ -1,10 +1,10 @@
 from __future__ import annotations
 
-from typing import Optional, Sequence, Union
+from typing import Sequence
 
 from cognite.client import utils
 from cognite.client._api_client import APIClient
-from cognite.client._constants import LIST_LIMIT_DEFAULT
+from cognite.client._constants import DEFAULT_LIMIT_READ
 from cognite.client.data_classes import TransformationNotification, TransformationNotificationList
 from cognite.client.data_classes.transformations.notifications import TransformationNotificationFilter
 from cognite.client.utils._identifier import IdentifierSequence
@@ -12,18 +12,17 @@ from cognite.client.utils._identifier import IdentifierSequence
 
 class TransformationNotificationsAPI(APIClient):
     _RESOURCE_PATH = "/transformations/notifications"
-    _LIST_CLASS = TransformationNotificationList
 
     def create(
-        self, notification: Union[TransformationNotification, Sequence[TransformationNotification]]
-    ) -> Union[TransformationNotification, TransformationNotificationList]:
-        """`Subscribe for notifications on the transformation errors. <https://docs.cognite.com/api/v1/#operation/createTransformationNotifications>`_
+        self, notification: TransformationNotification | Sequence[TransformationNotification]
+    ) -> TransformationNotification | TransformationNotificationList:
+        """`Subscribe for notifications on the transformation errors. <https://developer.cognite.com/api#tag/Transformation-Notifications/operation/createTransformationNotifications>`_
 
         Args:
-            notification (Union[TransformationNotification, Sequence[TransformationNotification]]): Notification or list of notifications to create.
+            notification (TransformationNotification | Sequence[TransformationNotification]): Notification or list of notifications to create.
 
         Returns:
-            Created notification(s)
+            TransformationNotification | TransformationNotificationList: Created notification(s)
 
         Examples:
 
@@ -42,18 +41,18 @@ class TransformationNotificationsAPI(APIClient):
 
     def list(
         self,
-        transformation_id: Optional[int] = None,
-        transformation_external_id: str = None,
-        destination: str = None,
-        limit: Optional[int] = LIST_LIMIT_DEFAULT,
+        transformation_id: int | None = None,
+        transformation_external_id: str | None = None,
+        destination: str | None = None,
+        limit: int | None = DEFAULT_LIMIT_READ,
     ) -> TransformationNotificationList:
-        """`List notification subscriptions. <https://docs.cognite.com/api/v1/#operation/getTransformationNotifications>`_
+        """`List notification subscriptions. <https://developer.cognite.com/api#tag/Transformation-Notifications/operation/getTransformationNotifications>`_
 
         Args:
-            transformation_id (Optional[int]): Filter by transformation internal numeric ID.
-            transformation_external_id (str): Filter by transformation externalId.
-            destination (str): Filter by notification destination.
-            limit (int): Limits the number of results to be returned. To retrieve all results use limit=-1, default limit is 25.
+            transformation_id (int | None): Filter by transformation internal numeric ID.
+            transformation_external_id (str | None): Filter by transformation externalId.
+            destination (str | None): Filter by notification destination.
+            limit (int | None): Limits the number of results to be returned. To retrieve all results use limit=-1, default limit is 25.
 
         Returns:
             TransformationNotificationList: List of transformation notifications
@@ -86,14 +85,11 @@ class TransformationNotificationsAPI(APIClient):
             filter=filter,
         )
 
-    def delete(self, id: Union[int, Sequence[int]] = None) -> None:
-        """`Deletes the specified notification subscriptions on the transformation. Does nothing when the subscriptions already don't exist <https://docs.cognite.com/api/v1/#operation/deleteTransformationNotifications>`_
+    def delete(self, id: int | Sequence[int] | None = None) -> None:
+        """`Deletes the specified notification subscriptions on the transformation. Does nothing when the subscriptions already don't exist <https://developer.cognite.com/api#tag/Transformation-Notifications/operation/deleteTransformationNotifications>`_
 
         Args:
-            id (Union[int, Sequence[int]): Id or list of transformation notification ids
-
-        Returns:
-            None
+            id (int | Sequence[int] | None): Id or list of transformation notification ids
 
         Examples:
 
