@@ -7,7 +7,7 @@ import threading
 import time
 from abc import abstractmethod
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Optional, Protocol, Tuple, Union
+from typing import Any, Callable, Protocol
 
 from msal import ConfidentialClientApplication, PublicClientApplication, SerializableTokenCache
 from oauthlib.oauth2 import BackendApplicationClient, OAuth2Error
@@ -15,10 +15,12 @@ from requests_oauthlib import OAuth2Session
 
 from cognite.client.exceptions import CogniteAuthError
 
+_TOKEN_EXPIRY_LEEWAY_SECONDS_DEFAULT = 15  # Do not change without also updating all the docstrings using it
+
 
 class CredentialProvider(Protocol):
     @abstractmethod
-    def authorization_header(self) -> Tuple[str, str]:
+    def authorization_header(self) -> tuple[str, str]:
         raise NotImplementedError
 
 
@@ -26,7 +28,7 @@ class Token(CredentialProvider):
     """Token credential provider
 
     Args:
-        token (Union[str, Callable[[], str]): A token or a token factory.
+        token (str | Callable[[], str]): A token or a token factory.
 
     Examples:
 
@@ -39,7 +41,7 @@ class Token(CredentialProvider):
         under the hood, so it will be called while holding a thread lock (threading.Lock()).
     """
 
-    def __init__(self, token: Union[str, Callable[[], str]]) -> None:
+    def __init__(self, token: str | Callable[[], str]) -> None:
         if isinstance(token, str):
             self.__token_factory = lambda: token
 
@@ -55,32 +57,42 @@ class Token(CredentialProvider):
         else:
             raise TypeError(f"'token' must be a string or a no-argument-callable returning a string, not {type(token)}")
 
-    def authorization_header(self) -> Tuple[str, str]:
+    def authorization_header(self) -> tuple[str, str]:
         return "Authorization", f"Bearer {self.__token_factory()}"
 
 
 class _OAuthCredentialProviderWithTokenRefresh(CredentialProvider):
-    # This ensures we don't return a token which expires immediately, but within minimum 3 seconds.
-    __TOKEN_EXPIRY_LEEWAY_SECONDS = 3
+    def __init__(self, token_expiry_leeway_seconds: int = _TOKEN_EXPIRY_LEEWAY_SECONDS_DEFAULT) -> None:
+        # This ensures we don't return a token which expires immediately:
+        self.token_expiry_leeway_seconds = token_expiry_leeway_seconds
 
-    def __init__(self) -> None:
         self.__token_refresh_lock = threading.Lock()
-        self.__access_token: Optional[str] = None
-        self.__access_token_expires_at: Optional[float] = None
+        self.__access_token: str | None = None
+        self.__access_token_expires_at: float | None = None
+
+    def __getstate__(self) -> dict[str, Any]:
+        # threading.Lock is not picklable, temporarily remove:
+        lock_tmp, self.__token_refresh_lock = self.__token_refresh_lock, None  # type: ignore [assignment]
+        state = self.__dict__.copy()
+        self.__token_refresh_lock = lock_tmp
+        return state
+
+    def __setstate__(self, state: dict[str, Any]) -> None:
+        self.__dict__ = state
+        self.__token_refresh_lock = threading.Lock()
 
     @abstractmethod
-    def _refresh_access_token(self) -> Tuple[str, float]:
+    def _refresh_access_token(self) -> tuple[str, float]:
         """This method should return the access_token and expiry time"""
         raise NotImplementedError
 
-    @classmethod
-    def __should_refresh_token(cls, token: Optional[str], expires_at: Optional[float]) -> bool:
+    def __should_refresh_token(self, token: str | None, expires_at: float | None) -> bool:
         no_token = token is None
-        token_is_expired = expires_at is None or time.time() > expires_at - cls.__TOKEN_EXPIRY_LEEWAY_SECONDS
+        token_is_expired = expires_at is None or time.time() > expires_at - self.token_expiry_leeway_seconds
         return no_token or token_is_expired
 
     @staticmethod
-    def _verify_credentials(credentials: Dict[str, Any]) -> None:
+    def _verify_credentials(credentials: dict[str, Any]) -> None:
         """The msal library doesnt raise anything when auth fails, but returns a dictionary with varying keys"""
         if "access_token" in credentials and "expires_in" in credentials:
             return
@@ -91,7 +103,7 @@ class _OAuthCredentialProviderWithTokenRefresh(CredentialProvider):
             f"Error generating access token! Error: {credentials['error']}, error description: {err_descr}"
         )
 
-    def authorization_header(self) -> Tuple[str, str]:
+    def authorization_header(self) -> tuple[str, str]:
         # We lock here to ensure we don't issue a herd of refresh requests in concurrent scenarios.
         # This means we will block all requests for the duration of the token refresh.
         # TODO: Consider instead having a background thread periodically refresh the token to avoid this blocking.
@@ -118,6 +130,22 @@ class _WithMsalSerializableTokenCache:
         atexit.register(__at_exit)
         return token_cache
 
+    @staticmethod
+    def _resolve_token_cache_path(token_cache_path: Path | None, client_id: str) -> Path:
+        return token_cache_path or Path(tempfile.gettempdir()) / f"cognitetokencache.{client_id}.bin"
+
+    def _create_client_app(self, token_cache_path: Path, client_id: str, authority_url: str) -> PublicClientApplication:
+        from cognite.client.config import global_config
+
+        # In addition to caching in memory, we also cache the token on disk so it can be reused across processes:
+        serializable_token_cache = self._create_serializable_token_cache(token_cache_path)
+        return PublicClientApplication(
+            client_id=client_id,
+            authority=authority_url,
+            token_cache=serializable_token_cache,
+            verify=not global_config.disable_ssl,
+        )
+
 
 class OAuthDeviceCode(_OAuthCredentialProviderWithTokenRefresh, _WithMsalSerializableTokenCache):
     """OAuth credential provider for the device code login flow.
@@ -125,9 +153,9 @@ class OAuthDeviceCode(_OAuthCredentialProviderWithTokenRefresh, _WithMsalSeriali
     Args:
         authority_url (str): OAuth authority url
         client_id (str): Your application's client id.
-        scopes (List[str]): A list of scopes.
-        token_cache_path (Path): Location to store token cache, defaults to
-                                 os temp directory/cognitetokencache.{client_id}.bin.
+        scopes (list[str]): A list of scopes.
+        token_cache_path (Path | None): Location to store token cache, defaults to os temp directory/cognitetokencache.{client_id}.bin.
+        token_expiry_leeway_seconds (int): The token is refreshed at the earliest when this number of seconds is left before expiry. Default: 15 sec
 
     Examples:
 
@@ -143,25 +171,28 @@ class OAuthDeviceCode(_OAuthCredentialProviderWithTokenRefresh, _WithMsalSeriali
         self,
         authority_url: str,
         client_id: str,
-        scopes: List[str],
-        token_cache_path: Path = None,
+        scopes: list[str],
+        token_cache_path: Path | None = None,
+        token_expiry_leeway_seconds: int = _TOKEN_EXPIRY_LEEWAY_SECONDS_DEFAULT,
     ) -> None:
-        from cognite.client.config import global_config
-
-        super().__init__()
+        super().__init__(token_expiry_leeway_seconds)
         self.__authority_url = authority_url
         self.__client_id = client_id
         self.__scopes = scopes
 
-        # In addition to caching in memory, we also cache the token on disk so it can be reused across processes.
-        token_cache_path = token_cache_path or Path(tempfile.gettempdir()) / f"cognitetokencache.{self.__client_id}.bin"
-        serializable_token_cache = self._create_serializable_token_cache(token_cache_path)
-        self.__app = PublicClientApplication(
-            client_id=self.__client_id,
-            authority=self.__authority_url,
-            token_cache=serializable_token_cache,
-            verify=not global_config.disable_ssl,
-        )
+        self._token_cache_path = self._resolve_token_cache_path(token_cache_path, client_id)
+        self.__app = self._create_client_app(self._token_cache_path, client_id, authority_url)
+
+    def __getstate__(self) -> dict[str, Any]:
+        # PublicClientApplication is not picklable, temporarily remove:
+        app_tmp, self.__app = self.__app, None
+        state = super().__getstate__()
+        self.__app = app_tmp
+        return state
+
+    def __setstate__(self, state: dict[str, Any]) -> None:
+        super().__setstate__(state)
+        self.__app = self._create_client_app(self._token_cache_path, self.__client_id, self.__authority_url)
 
     @property
     def authority_url(self) -> str:
@@ -172,19 +203,18 @@ class OAuthDeviceCode(_OAuthCredentialProviderWithTokenRefresh, _WithMsalSeriali
         return self.__client_id
 
     @property
-    def scopes(self) -> List[str]:
+    def scopes(self) -> list[str]:
         return self.__scopes
 
-    def _refresh_access_token(self) -> Tuple[str, float]:
+    def _refresh_access_token(self) -> tuple[str, float]:
         # First check if there is a serialized token cached on disk.
-        accounts = self.__app.get_accounts()
-        credentials = self.__app.acquire_token_silent(scopes=self.__scopes, account=accounts[0]) if accounts else None
-
-        # If not, we acquire a new token interactively
-        if credentials is None:
+        if accounts := self.__app.get_accounts():
+            credentials = self.__app.acquire_token_silent(scopes=self.__scopes, account=accounts[0])
+        # If not, we acquire a new token using device code auth flow:
+        else:
             device_flow = self.__app.initiate_device_flow(scopes=self.__scopes)
             # print device code user instructions to screen
-            print(f"Device code: {device_flow['message']}")
+            print(f"Device code: {device_flow['message']}")  # noqa: T201
             credentials = self.__app.acquire_token_by_device_flow(flow=device_flow)
 
         self._verify_credentials(credentials)
@@ -199,10 +229,10 @@ class OAuthInteractive(_OAuthCredentialProviderWithTokenRefresh, _WithMsalSerial
     Args:
         authority_url (str): OAuth authority url
         client_id (str): Your application's client id.
-        scopes (List[str]): A list of scopes.
-        redirect_port (List[str]): Redirect port defaults to 53000.
-        token_cache_path (Path): Location to store token cache, defaults to
-                                 os temp directory/cognitetokencache.{client_id}.bin.
+        scopes (list[str]): A list of scopes.
+        redirect_port (int): Redirect port defaults to 53000.
+        token_cache_path (Path | None): Location to store token cache, defaults to os temp directory/cognitetokencache.{client_id}.bin.
+        token_expiry_leeway_seconds (int): The token is refreshed at the earliest when this number of seconds is left before expiry. Default: 15 sec
 
     Examples:
 
@@ -218,27 +248,30 @@ class OAuthInteractive(_OAuthCredentialProviderWithTokenRefresh, _WithMsalSerial
         self,
         authority_url: str,
         client_id: str,
-        scopes: List[str],
+        scopes: list[str],
         redirect_port: int = 53000,
-        token_cache_path: Path = None,
+        token_cache_path: Path | None = None,
+        token_expiry_leeway_seconds: int = _TOKEN_EXPIRY_LEEWAY_SECONDS_DEFAULT,
     ) -> None:
-        from cognite.client.config import global_config
-
-        super().__init__()
+        super().__init__(token_expiry_leeway_seconds)
         self.__authority_url = authority_url
         self.__client_id = client_id
         self.__scopes = scopes
         self.__redirect_port = redirect_port
 
-        # In addition to caching in memory, we also cache the token on disk so it can be reused across processes.
-        token_cache_path = token_cache_path or Path(tempfile.gettempdir()) / f"cognitetokencache.{self.__client_id}.bin"
-        serializable_token_cache = self._create_serializable_token_cache(token_cache_path)
-        self.__app = PublicClientApplication(
-            client_id=self.__client_id,
-            authority=self.__authority_url,
-            token_cache=serializable_token_cache,
-            verify=not global_config.disable_ssl,
-        )
+        self._token_cache_path = self._resolve_token_cache_path(token_cache_path, client_id)
+        self.__app = self._create_client_app(self._token_cache_path, client_id, authority_url)
+
+    def __getstate__(self) -> dict[str, Any]:
+        # PublicClientApplication is not picklable, temporarily remove:
+        app_tmp, self.__app = self.__app, None
+        state = super().__getstate__()
+        self.__app = app_tmp
+        return state
+
+    def __setstate__(self, state: dict[str, Any]) -> None:
+        super().__setstate__(state)
+        self.__app = self._create_client_app(self._token_cache_path, self.__client_id, self.__authority_url)
 
     @property
     def authority_url(self) -> str:
@@ -249,21 +282,55 @@ class OAuthInteractive(_OAuthCredentialProviderWithTokenRefresh, _WithMsalSerial
         return self.__client_id
 
     @property
-    def scopes(self) -> List[str]:
+    def scopes(self) -> list[str]:
         return self.__scopes
 
-    def _refresh_access_token(self) -> Tuple[str, float]:
+    def _refresh_access_token(self) -> tuple[str, float]:
         # First check if there is a serialized token cached on disk.
-        accounts = self.__app.get_accounts()
-        credentials = self.__app.acquire_token_silent(scopes=self.__scopes, account=accounts[0]) if accounts else None
-
+        if accounts := self.__app.get_accounts():
+            credentials = self.__app.acquire_token_silent(scopes=self.__scopes, account=accounts[0])
         # If not, we acquire a new token interactively
-        if credentials is None:
+        else:
             credentials = self.__app.acquire_token_interactive(scopes=self.__scopes, port=self.__redirect_port)
 
         self._verify_credentials(credentials)
         return credentials["access_token"], time.time() + credentials["expires_in"]
 
+    @classmethod
+    def default_for_azure_ad(
+        cls,
+        tenant_id: str,
+        client_id: str,
+        cdf_cluster: str,
+        token_expiry_leeway_seconds: int = _TOKEN_EXPIRY_LEEWAY_SECONDS_DEFAULT,
+        **token_custom_args: Any,
+    ) -> OAuthInteractive:
+        """
+        Create an OAuthClientCredentials instance for Azure with default token URL and scopes.
+
+        The default configuration creates the URLs based on the tenant id and cluster:
+
+        * Authority URL: "https://login.microsoftonline.com/{tenant_id}"
+        * Scopes: [f"https://{cdf_cluster}.cognitedata.com/.default"]
+
+        Args:
+            tenant_id (str): The Azure tenant id
+            client_id (str): Your application's client id.
+            cdf_cluster (str): The CDF cluster where the CDF project is located.
+            token_expiry_leeway_seconds (int): The token is refreshed at the earliest when this number of seconds is left before expiry. Default: 15 sec
+            **token_custom_args (Any): Optional additional arguments to pass as query parameters to the token fetch request.
+
+        Returns:
+            OAuthInteractive: An OAuthInteractive instance
+        """
+        return cls(
+            authority_url=f"https://login.microsoftonline.com/{tenant_id}",
+            client_id=client_id,
+            scopes=[f"https://{cdf_cluster}.cognitedata.com/.default"],
+            token_expiry_leeway_seconds=token_expiry_leeway_seconds,
+            **token_custom_args,
+        )
+
 
 class OAuthClientCredentials(_OAuthCredentialProviderWithTokenRefresh):
     """OAuth credential provider for the "Client Credentials" flow.
@@ -272,7 +339,8 @@ class OAuthClientCredentials(_OAuthCredentialProviderWithTokenRefresh):
         token_url (str): OAuth token url
         client_id (str): Your application's client id.
         client_secret (str): Your application's client secret
-        scopes (List[str]): A list of scopes.
+        scopes (list[str]): A list of scopes.
+        token_expiry_leeway_seconds (int): The token is refreshed at the earliest when this number of seconds is left before expiry. Default: 15 sec
         **token_custom_args (Any): Optional additional arguments to pass as query parameters to the token fetch request.
 
     Examples:
@@ -294,18 +362,22 @@ class OAuthClientCredentials(_OAuthCredentialProviderWithTokenRefresh):
         token_url: str,
         client_id: str,
         client_secret: str,
-        scopes: List[str],
+        scopes: list[str],
+        token_expiry_leeway_seconds: int = _TOKEN_EXPIRY_LEEWAY_SECONDS_DEFAULT,
         **token_custom_args: Any,
-    ):
-        super().__init__()
+    ) -> None:
+        super().__init__(token_expiry_leeway_seconds)
         self.__token_url = token_url
         self.__client_id = client_id
         self.__client_secret = client_secret
         self.__scopes = scopes
-        self.__token_custom_args: Dict[str, Any] = token_custom_args
-        self.__oauth = OAuth2Session(client=BackendApplicationClient(client_id=self.__client_id, scope=self.__scopes))
+        self.__token_custom_args: dict[str, Any] = token_custom_args
+        self.__oauth = self._create_oauth_session()
         self._validate_token_custom_args()
 
+    def _create_oauth_session(self) -> OAuth2Session:
+        return OAuth2Session(client=BackendApplicationClient(client_id=self.__client_id, scope=self.__scopes))
+
     def _validate_token_custom_args(self) -> None:
         # We make sure that whatever is passed as part of 'token_custom_args' can't set or override any of the
         # named parameters that 'fetch_token' accepts:
@@ -316,6 +388,17 @@ class OAuthClientCredentials(_OAuthCredentialProviderWithTokenRefresh):
                 f"reserved custom args is: {sorted(reserved)}."
             )
 
+    def __getstate__(self) -> dict[str, Any]:
+        # OAuth2Session is not picklable, temporarily remove:
+        oauth_session_tmp, self.__oauth = self.__oauth, None
+        state = super().__getstate__()
+        self.__oauth = oauth_session_tmp
+        return state
+
+    def __setstate__(self, state: dict[str, Any]) -> None:
+        super().__setstate__(state)
+        self.__oauth = self._create_oauth_session()
+
     @property
     def token_url(self) -> str:
         return self.__token_url
@@ -329,14 +412,14 @@ class OAuthClientCredentials(_OAuthCredentialProviderWithTokenRefresh):
         return self.__client_secret
 
     @property
-    def scopes(self) -> List[str]:
+    def scopes(self) -> list[str]:
         return self.__scopes
 
     @property
-    def token_custom_args(self) -> Dict[str, Any]:
+    def token_custom_args(self) -> dict[str, Any]:
         return self.__token_custom_args
 
-    def _refresh_access_token(self) -> Tuple[str, float]:
+    def _refresh_access_token(self) -> tuple[str, float]:
         from cognite.client.config import global_config
 
         try:
@@ -352,6 +435,44 @@ class OAuthClientCredentials(_OAuthCredentialProviderWithTokenRefresh):
                 f"Error generating access token: {oauth_err.error}, {oauth_err.status_code}, {oauth_err.description}"
             ) from oauth_err
 
+    @classmethod
+    def default_for_azure_ad(
+        cls,
+        tenant_id: str,
+        client_id: str,
+        client_secret: str,
+        cdf_cluster: str,
+        token_expiry_leeway_seconds: int = _TOKEN_EXPIRY_LEEWAY_SECONDS_DEFAULT,
+        **token_custom_args: Any,
+    ) -> OAuthClientCredentials:
+        """
+        Create an OAuthClientCredentials instance for Azure with default token URL and scopes.
+
+        The default configuration creates the URLs based on the tenant id and cluster/oauth2/v2.0/token:
+
+        * Token URL: "https://login.microsoftonline.com/{tenant_id}"
+        * Scopes: [f"https://{cdf_cluster}.cognitedata.com/.default"]
+
+        Args:
+            tenant_id (str): The Azure tenant id
+            client_id (str): Your application's client id.
+            client_secret (str): Your application's client secret.
+            cdf_cluster (str): The CDF cluster where the CDF project is located.
+            token_expiry_leeway_seconds (int): The token is refreshed at the earliest when this number of seconds is left before expiry. Default: 15 sec
+            **token_custom_args (Any): Optional additional arguments to pass as query parameters to the token fetch request.
+
+        Returns:
+            OAuthClientCredentials: An OAuthClientCredentials instance
+        """
+        return cls(
+            token_url=f"https://login.microsoftonline.com/{tenant_id}/oauth2/v2.0/token",
+            client_id=client_id,
+            client_secret=client_secret,
+            scopes=[f"https://{cdf_cluster}.cognitedata.com/.default"],
+            token_expiry_leeway_seconds=token_expiry_leeway_seconds,
+            **token_custom_args,
+        )
+
 
 class OAuthClientCertificate(_OAuthCredentialProviderWithTokenRefresh):
     """OAuth credential provider for authenticating with a client certificate.
@@ -361,7 +482,8 @@ class OAuthClientCertificate(_OAuthCredentialProviderWithTokenRefresh):
         client_id (str): Your application's client id.
         cert_thumbprint (str): Your certificate's thumbprint. You get it when you upload your certificate to Azure AD.
         certificate (str): Your private certificate, typically read from a .pem file
-        scopes (List[str]): A list of scopes.
+        scopes (list[str]): A list of scopes.
+        token_expiry_leeway_seconds (int): The token is refreshed at the earliest when this number of seconds is left before expiry. Default: 15 sec
 
     Examples:
 
@@ -376,8 +498,16 @@ class OAuthClientCertificate(_OAuthCredentialProviderWithTokenRefresh):
             ... )
     """
 
-    def __init__(self, authority_url: str, client_id: str, cert_thumbprint: str, certificate: str, scopes: List[str]):
-        super().__init__()
+    def __init__(
+        self,
+        authority_url: str,
+        client_id: str,
+        cert_thumbprint: str,
+        certificate: str,
+        scopes: list[str],
+        token_expiry_leeway_seconds: int = _TOKEN_EXPIRY_LEEWAY_SECONDS_DEFAULT,
+    ) -> None:
+        super().__init__(token_expiry_leeway_seconds)
         self.__authority_url = authority_url
         self.__client_id = client_id
         self.__cert_thumbprint = cert_thumbprint
@@ -407,10 +537,10 @@ class OAuthClientCertificate(_OAuthCredentialProviderWithTokenRefresh):
         return self.__certificate
 
     @property
-    def scopes(self) -> List[str]:
+    def scopes(self) -> list[str]:
         return self.__scopes
 
-    def _refresh_access_token(self) -> Tuple[str, float]:
+    def _refresh_access_token(self) -> tuple[str, float]:
         credentials = self.__app.acquire_token_for_client(scopes=self.__scopes)
 
         self._verify_credentials(credentials)
