@@ -1,3 +1,6 @@
+from __future__ import annotations
+
+from cognite.client.data_classes._base import Geometry
 from cognite.client.data_classes.annotations import (
     Annotation,
     AnnotationFilter,
@@ -22,6 +25,35 @@ from cognite.client.data_classes.contextualization import (
     EntityMatchingModelUpdate,
     JobStatus,
 )
+from cognite.client.data_classes.data_sets import (
+    DataSet,
+    DataSetAggregate,
+    DataSetFilter,
+    DataSetList,
+    DataSetUpdate,
+)
+from cognite.client.data_classes.datapoints import (
+    Datapoint,
+    Datapoints,
+    DatapointsArray,
+    DatapointsArrayList,
+    DatapointsList,
+    LatestDatapointQuery,
+)
+from cognite.client.data_classes.datapoints_subscriptions import (
+    DatapointSubscription,
+    DataPointSubscriptionCreate,
+    DatapointSubscriptionList,
+    DataPointSubscriptionUpdate,
+)
+from cognite.client.data_classes.documents import (
+    Document,
+    DocumentHighlight,
+    DocumentHighlightList,
+    DocumentList,
+    SourceFile,
+)
+from cognite.client.data_classes.events import EndTimeFilter, Event, EventFilter, EventList, EventUpdate
 from cognite.client.data_classes.extractionpipelines import (
     ExtractionPipeline,
     ExtractionPipelineConfig,
@@ -41,6 +73,32 @@ from cognite.client.data_classes.files import (
     FileMetadataList,
     FileMetadataUpdate,
 )
+from cognite.client.data_classes.functions import (
+    Function,
+    FunctionCall,
+    FunctionCallList,
+    FunctionCallLog,
+    FunctionCallLogEntry,
+    FunctionFilter,
+    FunctionList,
+    FunctionSchedule,
+    FunctionSchedulesFilter,
+    FunctionSchedulesList,
+    FunctionsLimits,
+)
+from cognite.client.data_classes.geospatial import (
+    CoordinateReferenceSystem,
+    CoordinateReferenceSystemList,
+    Feature,
+    FeatureAggregate,
+    FeatureAggregateList,
+    FeatureList,
+    FeatureType,
+    FeatureTypeList,
+    FeatureTypePatch,
+    FeatureTypeUpdate,
+    FeatureTypeUpdateList,
+)
 from cognite.client.data_classes.iam import (
     ClientCredentials,
     CreatedSession,
@@ -58,6 +116,7 @@ from cognite.client.data_classes.labels import (
     LabelDefinitionList,
     LabelFilter,
 )
+from cognite.client.data_classes.raw import Database, DatabaseList, Row, RowList, Table, TableList
 from cognite.client.data_classes.relationships import (
     Relationship,
     RelationshipFilter,
@@ -74,6 +133,14 @@ from cognite.client.data_classes.sequences import (
     SequenceList,
     SequenceUpdate,
 )
+from cognite.client.data_classes.shared import (
+    AggregateResult,
+    AggregateUniqueValuesResult,
+    GeoLocation,
+    GeoLocationFilter,
+    GeometryFilter,
+    TimestampRange,
+)
 from cognite.client.data_classes.templates import (
     ConstantResolver,
     Source,
@@ -137,65 +204,34 @@ from cognite.client.data_classes.transformations.schedules import (
     TransformationScheduleList,
     TransformationScheduleUpdate,
 )
-
 from cognite.client.data_classes.transformations.schema import (
     TransformationSchemaColumn,
     TransformationSchemaColumnList,
 )
-
-from cognite.client.data_classes.data_sets import (
-    DataSet,
-    DataSetAggregate,
-    DataSetFilter,
-    DataSetList,
-    DataSetUpdate,
-)
-
-from cognite.client.data_classes.shared import (
-    AggregateResult,
-    AggregateUniqueValuesResult,
-    GeoLocation,
-    GeoLocationFilter,
-    Geometry,
-    GeometryFilter,
-    TimestampRange,
-)
-
-from cognite.client.data_classes.datapoints import (
-    Datapoint,
-    Datapoints,
-    DatapointsList,
-    DatapointsArray,
-    DatapointsArrayList,
-    LatestDatapointQuery,
-)
-from cognite.client.data_classes.events import EndTimeFilter, Event, EventFilter, EventList, EventUpdate
-from cognite.client.data_classes.raw import Database, DatabaseList, Row, RowList, Table, TableList
-from cognite.client.data_classes.functions import (
-    Function,
-    FunctionFilter,
-    FunctionSchedule,
-    FunctionSchedulesFilter,
-    FunctionSchedulesList,
-    FunctionList,
-    FunctionCall,
-    FunctionCallList,
-    FunctionCallLogEntry,
-    FunctionCallLog,
-    FunctionsLimits,
-)
-from cognite.client.data_classes.geospatial import (
-    Feature,
-    FeatureList,
-    FeatureType,
-    FeatureTypeList,
-    FeatureTypePatch,
-    FeatureAggregate,
-    FeatureTypeUpdate,
-    FeatureAggregateList,
-    FeatureTypeUpdateList,
-    CoordinateReferenceSystemList,
-    CoordinateReferenceSystem,
+from cognite.client.data_classes.user_profiles import UserProfile, UserProfileList
+from cognite.client.data_classes.workflows import (
+    CDFTaskOutput,
+    CDFTaskParameters,
+    DynamicTaskOutput,
+    DynamicTaskParameters,
+    FunctionTaskOutput,
+    FunctionTaskParameters,
+    TransformationTaskOutput,
+    TransformationTaskParameters,
+    Workflow,
+    WorkflowDefinition,
+    WorkflowDefinitionUpsert,
+    WorkflowExecution,
+    WorkflowExecutionDetailed,
+    WorkflowExecutionList,
+    WorkflowList,
+    WorkflowTask,
+    WorkflowTaskExecution,
+    WorkflowUpsert,
+    WorkflowVersion,
+    WorkflowVersionId,
+    WorkflowVersionList,
+    WorkflowVersionUpsert,
 )
 
 __all__ = [
@@ -243,16 +279,17 @@ __all__ = [
     "FileMetadataFilter",
     "FileMetadataList",
     "FileMetadataUpdate",
-    "APIKey",
-    "APIKeyList",
+    "SourceFile",
+    "Document",
+    "DocumentList",
+    "DocumentHighlight",
+    "DocumentHighlightList",
     "ClientCredentials",
     "CreatedSession",
     "Group",
     "GroupList",
     "SecurityCategory",
     "SecurityCategoryList",
-    "ServiceAccount",
-    "ServiceAccountList",
     "Session",
     "SessionList",
     "Label",
@@ -300,6 +337,10 @@ __all__ = [
     "TimeSeriesFilter",
     "TimeSeriesList",
     "TimeSeriesUpdate",
+    "DatapointSubscription",
+    "DataPointSubscriptionCreate",
+    "DatapointSubscriptionList",
+    "DataPointSubscriptionUpdate",
     "OidcCredentials",
     "RawTable",
     "Transformation",
@@ -330,8 +371,8 @@ __all__ = [
     "AggregateUniqueValuesResult",
     "GeoLocation",
     "GeoLocationFilter",
-    "Geometry",
     "GeometryFilter",
+    "Geometry",
     "TimestampRange",
     "Datapoint",
     "Datapoints",
@@ -361,4 +402,28 @@ __all__ = [
     "FeatureTypeUpdateList",
     "CoordinateReferenceSystemList",
     "CoordinateReferenceSystem",
+    "UserProfile",
+    "UserProfileList",
+    "WorkflowUpsert",
+    "WorkflowExecution",
+    "WorkflowExecutionDetailed",
+    "WorkflowExecutionList",
+    "WorkflowList",
+    "WorkflowVersion",
+    "WorkflowVersionUpsert",
+    "WorkflowVersionId",
+    "WorkflowVersionList",
+    "FunctionTaskParameters",
+    "TransformationTaskParameters",
+    "CDFTaskParameters",
+    "DynamicTaskParameters",
+    "FunctionTaskOutput",
+    "TransformationTaskOutput",
+    "CDFTaskOutput",
+    "DynamicTaskOutput",
+    "WorkflowDefinition",
+    "WorkflowDefinitionUpsert",
+    "WorkflowTaskExecution",
+    "Workflow",
+    "WorkflowTask",
 ]
