@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import json
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union, cast
+from typing import TYPE_CHECKING, Any, cast
 
 from cognite.client.data_classes._base import (
     CogniteFilter,
@@ -10,6 +10,7 @@ from cognite.client.data_classes._base import (
     CogniteResource,
     CogniteResourceList,
     CogniteUpdate,
+    PropertySpec,
 )
 from cognite.client.utils._text import to_snake_case
 
@@ -24,19 +25,11 @@ class Annotation(CogniteResource):
         annotation_type (str): The type of the annotation. This uniquely decides what the structure of the 'data' block will be.
         data (dict): The annotation information. The format of this object is decided by and validated against the 'annotation_type' attribute.
         status (str): The status of the annotation, e.g. "suggested", "approved", "rejected".
-
-        annotated_resource_type (str): Type name of the CDF resource that is annotated, e.g. "file".
-        annotated_resource_id (int, optional): The internal ID of the annotated resource.
-
         creating_app (str): The name of the app from which this annotation was created.
         creating_app_version (str): The version of the app that created this annotation. Must be a valid semantic versioning (SemVer) string.
-        creating_user: (str, optional): A username, or email, or name. This is not checked nor enforced. If the value is None, it means the annotation was created by a service.
-
-        id (int, optional): A server-generated id for the object. Read-only.
-        created_time (int, optional): Time when this annotation was created in CDF. The time is measured in milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds. Read-only.
-        last_updated_time (int, optional): Time when this annotation was last updated in CDF. The time is measured in milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds. Read-only.
-
-        cognite_client (CogniteClient, optional): The client to associate with this object. Read-only.
+        creating_user (str | None): (str, optional): A username, or email, or name. This is not checked nor enforced. If the value is None, it means the annotation was created by a service.
+        annotated_resource_type (str): Type name of the CDF resource that is annotated, e.g. "file".
+        annotated_resource_id (int | None): The internal ID of the annotated resource.
     """
 
     def __init__(
@@ -46,9 +39,9 @@ class Annotation(CogniteResource):
         status: str,
         creating_app: str,
         creating_app_version: str,
-        creating_user: Optional[str],
+        creating_user: str | None,
         annotated_resource_type: str,
-        annotated_resource_id: Optional[int] = None,
+        annotated_resource_id: int | None = None,
     ) -> None:
         self.annotation_type = annotation_type
         self.data = data
@@ -58,13 +51,13 @@ class Annotation(CogniteResource):
         self.creating_user = creating_user
         self.annotated_resource_type = annotated_resource_type
         self.annotated_resource_id = annotated_resource_id
-        self.id: Optional[int] = None  # Read only
-        self.created_time: Optional[int] = None  # Read only
-        self.last_updated_time: Optional[int] = None  # Read only
+        self.id: int | None = None  # Read only
+        self.created_time: int | None = None  # Read only
+        self.last_updated_time: int | None = None  # Read only
         self._cognite_client: CogniteClient = cast("CogniteClient", None)  # Read only
 
     @classmethod
-    def _load(cls, resource: Union[Dict[str, Any], str], cognite_client: CogniteClient = None) -> Annotation:
+    def _load(cls, resource: dict[str, Any] | str, cognite_client: CogniteClient | None = None) -> Annotation:
         if isinstance(resource, str):
             return cls._load(json.loads(resource), cognite_client=cognite_client)
         elif isinstance(resource, dict):
@@ -72,7 +65,7 @@ class Annotation(CogniteResource):
         raise TypeError(f"Resource must be json str or dict, not {type(resource)}")
 
     @classmethod
-    def from_dict(cls, resource: Dict[str, Any], cognite_client: CogniteClient = None) -> Annotation:
+    def from_dict(cls, resource: dict[str, Any], cognite_client: CogniteClient | None = None) -> Annotation:
         # Create base annotation
         data = {to_snake_case(key): val for key, val in resource.items()}
         annotation = Annotation(
@@ -92,7 +85,7 @@ class Annotation(CogniteResource):
         annotation._cognite_client = cast("CogniteClient", cognite_client)
         return annotation
 
-    def dump(self, camel_case: bool = False) -> Dict[str, Any]:
+    def dump(self, camel_case: bool = False) -> dict[str, Any]:
         result = super().dump(camel_case=camel_case)
         # Special handling of created_user, which has a valid None value
         key = "creatingUser" if camel_case else "creating_user"
@@ -100,33 +93,30 @@ class Annotation(CogniteResource):
         return result
 
 
-class AnnotationFilter(CogniteFilter):
+class AnnotationReverseLookupFilter(CogniteFilter):
     """Filter on annotations with various criteria
 
     Args:
         annotated_resource_type (str): The type of the CDF resource that is annotated, e.g. "file".
-        annotated_resource_ids (List[Dict[str, Any]]): List of ids of the annotated CDF resources to filter in. Example format: [{"id": 1234}, {"id": "4567"}]. Must contain at least one item.
-        status (str, optional): Status of annotations to filter for, e.g. "suggested", "approved", "rejected".
-        creating_user (str, optional): Name of the user who created the annotations to filter for. Can be set explicitly to "None" to filter for annotations created by a service.
-        creating_app (str, optional): Name of the app from which the annotations to filter for where created.
-        creating_app_version (str, optional): Version of the app from which the annotations to filter for were created.
-        annotation_type(str, optional): Type name of the annotations.
-        data(Dict[str, Any], optional): The annotation data to filter by. Example format: {"label": "cat", "confidence": 0.9}
+        status (str | None): Status of annotations to filter for, e.g. "suggested", "approved", "rejected".
+        creating_user (str | None): Name of the user who created the annotations to filter for. Can be set explicitly to "None" to filter for annotations created by a service.
+        creating_app (str | None): Name of the app from which the annotations to filter for where created.
+        creating_app_version (str | None): Version of the app from which the annotations to filter for were created.
+        annotation_type (str | None): Type name of the annotations.
+        data (dict[str, Any] | None): The annotation data to filter by. Example format: {"label": "cat", "confidence": 0.9}
     """
 
     def __init__(
         self,
         annotated_resource_type: str,
-        annotated_resource_ids: List[Dict[str, int]],
-        status: Optional[str] = None,
-        creating_user: Optional[str] = "",  # None means filtering for a service
-        creating_app: Optional[str] = None,
-        creating_app_version: Optional[str] = None,
-        annotation_type: Optional[str] = None,
-        data: Optional[Dict[str, Any]] = None,
+        status: str | None = None,
+        creating_user: str | None = "",  # None means filtering for a service
+        creating_app: str | None = None,
+        creating_app_version: str | None = None,
+        annotation_type: str | None = None,
+        data: dict[str, Any] | None = None,
     ) -> None:
         self.annotated_resource_type = annotated_resource_type
-        self.annotated_resource_ids = annotated_resource_ids
         self.status = status
         self.creating_user = creating_user
         self.creating_app = creating_app
@@ -134,7 +124,7 @@ class AnnotationFilter(CogniteFilter):
         self.annotation_type = annotation_type
         self.data = data
 
-    def dump(self, camel_case: bool = False) -> Dict[str, Any]:
+    def dump(self, camel_case: bool = False) -> dict[str, Any]:
         result = super().dump(camel_case=camel_case)
         # Special handling for creating_user, which has a valid None value
         key = "creatingUser" if camel_case else "creating_user"
@@ -147,6 +137,43 @@ class AnnotationFilter(CogniteFilter):
         return result
 
 
+class AnnotationFilter(AnnotationReverseLookupFilter):
+    """Filter on annotations with various criteria
+
+    Args:
+        annotated_resource_type (str): The type of the CDF resource that is annotated, e.g. "file".
+        annotated_resource_ids (list[dict[str, int]]): List of ids of the annotated CDF resources to filter in. Example format: [{"id": 1234}, {"id": "4567"}]. Must contain at least one item.
+        status (str | None): Status of annotations to filter for, e.g. "suggested", "approved", "rejected".
+        creating_user (str | None): Name of the user who created the annotations to filter for. Can be set explicitly to "None" to filter for annotations created by a service.
+        creating_app (str | None): Name of the app from which the annotations to filter for where created.
+        creating_app_version (str | None): Version of the app from which the annotations to filter for were created.
+        annotation_type (str | None): Type name of the annotations.
+        data (dict[str, Any] | None): The annotation data to filter by. Example format: {"label": "cat", "confidence": 0.9}
+    """
+
+    def __init__(
+        self,
+        annotated_resource_type: str,
+        annotated_resource_ids: list[dict[str, int]],
+        status: str | None = None,
+        creating_user: str | None = "",  # None means filtering for a service
+        creating_app: str | None = None,
+        creating_app_version: str | None = None,
+        annotation_type: str | None = None,
+        data: dict[str, Any] | None = None,
+    ) -> None:
+        self.annotated_resource_ids = annotated_resource_ids
+        super().__init__(
+            annotated_resource_type=annotated_resource_type,
+            status=status,
+            creating_user=creating_user,
+            creating_app=creating_app,
+            creating_app_version=creating_app_version,
+            annotation_type=annotation_type,
+            data=data,
+        )
+
+
 class AnnotationUpdate(CogniteUpdate):
     """Changes applied to annotation
 
@@ -154,7 +181,7 @@ class AnnotationUpdate(CogniteUpdate):
         id (int): A server-generated ID for the object.
     """
 
-    def __init__(self, id: int):
+    def __init__(self, id: int) -> None:
         super().__init__(id=id)
 
     class _StrUpdate(CognitePrimitiveUpdate):
@@ -166,19 +193,19 @@ class AnnotationUpdate(CogniteUpdate):
     class _OptionalStrUpdate(CognitePrimitiveUpdate):
         """Set and set_null"""
 
-        def set(self, value: Optional[str]) -> AnnotationUpdate:
+        def set(self, value: str | None) -> AnnotationUpdate:
             return self._set(value)
 
     class _DictUpdate(CogniteObjectUpdate):
         """Only set, no set_null"""
 
-        def set(self, value: Dict[str, Any]) -> AnnotationUpdate:
+        def set(self, value: dict[str, Any]) -> AnnotationUpdate:
             return self._set(value)
 
     class _OptionalIntUpdate(CognitePrimitiveUpdate):
         """Set and set_null"""
 
-        def set(self, value: Optional[int]) -> AnnotationUpdate:
+        def set(self, value: int | None) -> AnnotationUpdate:
             return self._set(value)
 
     @property
@@ -193,6 +220,14 @@ class AnnotationUpdate(CogniteUpdate):
     def annotation_type(self) -> AnnotationUpdate._StrUpdate:
         return AnnotationUpdate._StrUpdate(self, "annotationType")
 
+    @classmethod
+    def _get_update_properties(cls) -> list[PropertySpec]:
+        return [
+            PropertySpec("data", is_nullable=False),
+            PropertySpec("status", is_nullable=False),
+            PropertySpec("annotation_type", is_nullable=False),
+        ]
 
-class AnnotationList(CogniteResourceList):
+
+class AnnotationList(CogniteResourceList[Annotation]):
     _RESOURCE = Annotation
