@@ -8,7 +8,6 @@ import threading
 import warnings
 from collections import Counter, defaultdict
 from functools import lru_cache
-from graphlib import TopologicalSorter
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
@@ -19,14 +18,14 @@ from typing import (
     Literal,
     Optional,
     Sequence,
-    Set,
     TextIO,
-    Tuple,
-    Type,
     Union,
     cast,
 )
 
+from graphlib import TopologicalSorter
+from typing_extensions import TypeAlias
+
 from cognite.client.data_classes._base import (
     CogniteFilter,
     CogniteLabelUpdate,
@@ -36,7 +35,11 @@ from cognite.client.data_classes._base import (
     CognitePropertyClassUtil,
     CogniteResource,
     CogniteResourceList,
+    CogniteSort,
     CogniteUpdate,
+    EnumProperty,
+    IdTransformerMixin,
+    PropertySpec,
 )
 from cognite.client.data_classes.labels import Label, LabelDefinition, LabelFilter
 from cognite.client.data_classes.shared import GeoLocation, GeoLocationFilter, TimestampRange
@@ -58,10 +61,11 @@ class AssetAggregate(dict):
     """Aggregation group of assets
 
     Args:
-        count (int): Size of the aggregation group
+        count (int | None): Size of the aggregation group
+        **kwargs (Any): No description.
     """
 
-    def __init__(self, count: int = None, **kwargs: Any) -> None:
+    def __init__(self, count: int | None = None, **kwargs: Any) -> None:
         self.count = count
         self.update(kwargs)
 
@@ -72,13 +76,18 @@ class AggregateResultItem(dict):
     """Aggregated metrics of the asset
 
     Args:
-        child_count (int): Number of direct descendants for the asset
-        depth (int): Asset path depth (number of levels below root node).
-        path (List[Dict[str, Any]]): IDs of assets on the path to the asset.
+        child_count (int | None): Number of direct descendants for the asset
+        depth (int | None): Asset path depth (number of levels below root node).
+        path (list[dict[str, Any]] | None): IDs of assets on the path to the asset.
+        **kwargs (Any): No description.
     """
 
     def __init__(
-        self, child_count: int = None, depth: int = None, path: List[Dict[str, Any]] = None, **kwargs: Any
+        self,
+        child_count: int | None = None,
+        depth: int | None = None,
+        path: list[dict[str, Any]] | None = None,
+        **kwargs: Any,
     ) -> None:
         self.child_count = child_count
         self.depth = depth
@@ -94,43 +103,43 @@ class Asset(CogniteResource):
     """A representation of a physical asset, for example a factory or a piece of equipment.
 
     Args:
-        external_id (str): The external ID provided by the client. Must be unique for the resource type.
-        name (str): The name of the asset.
-        parent_id (int): The parent of the node, null if it is the root node.
-        parent_external_id (str): The external ID of the parent. The property is omitted if the asset doesn't have a parent or if the parent doesn't have externalId.
-        description (str): The description of the asset.
-        data_set_id (int): The id of the dataset this asset belongs to.
-        metadata (Dict[str, str]): Custom, application specific metadata. String key -> String value. Limits: Maximum length of key is 128 bytes, value 10240 bytes, up to 256 key-value pairs, of total size at most 10240.
-        source (str): The source of the asset.
-        labels (List[Label]): A list of the labels associated with this resource item.
-        geo_location (GeoLocation): The geographic metadata of the asset.
-        id (int): A server-generated ID for the object.
-        created_time (int): The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.
-        last_updated_time (int): The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.
-        root_id (int): ID of the root asset.
-        aggregates (Union[Dict[str, Any], AggregateResultItem]): Aggregated metrics of the asset
-        cognite_client (CogniteClient): The client to associate with this object.
+        external_id (str | None): The external ID provided by the client. Must be unique for the resource type.
+        name (str | None): The name of the asset.
+        parent_id (int | None): The parent of the node, null if it is the root node.
+        parent_external_id (str | None): The external ID of the parent. The property is omitted if the asset doesn't have a parent or if the parent doesn't have externalId.
+        description (str | None): The description of the asset.
+        data_set_id (int | None): The id of the dataset this asset belongs to.
+        metadata (dict[str, str] | None): Custom, application specific metadata. String key -> String value. Limits: Maximum length of key is 128 bytes, value 10240 bytes, up to 256 key-value pairs, of total size at most 10240.
+        source (str | None): The source of the asset.
+        labels (list[Label | str | LabelDefinition | dict] | None): A list of the labels associated with this resource item.
+        geo_location (GeoLocation | None): The geographic metadata of the asset.
+        id (int | None): A server-generated ID for the object.
+        created_time (int | None): The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.
+        last_updated_time (int | None): The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.
+        root_id (int | None): ID of the root asset.
+        aggregates (dict[str, Any] | AggregateResultItem | None): Aggregated metrics of the asset
+        cognite_client (CogniteClient | None): The client to associate with this object.
     """
 
     def __init__(
         self,
-        external_id: str = None,
-        name: str = None,
-        parent_id: int = None,
-        parent_external_id: str = None,
-        description: str = None,
-        data_set_id: int = None,
-        metadata: Dict[str, str] = None,
-        source: str = None,
-        labels: List[Union[Label, str, LabelDefinition, dict]] = None,
-        geo_location: GeoLocation = None,
-        id: int = None,
-        created_time: int = None,
-        last_updated_time: int = None,
-        root_id: int = None,
-        aggregates: Union[Dict[str, Any], AggregateResultItem] = None,
-        cognite_client: CogniteClient = None,
-    ):
+        external_id: str | None = None,
+        name: str | None = None,
+        parent_id: int | None = None,
+        parent_external_id: str | None = None,
+        description: str | None = None,
+        data_set_id: int | None = None,
+        metadata: dict[str, str] | None = None,
+        source: str | None = None,
+        labels: list[Label | str | LabelDefinition | dict] | None = None,
+        geo_location: GeoLocation | None = None,
+        id: int | None = None,
+        created_time: int | None = None,
+        last_updated_time: int | None = None,
+        root_id: int | None = None,
+        aggregates: dict[str, Any] | AggregateResultItem | None = None,
+        cognite_client: CogniteClient | None = None,
+    ) -> None:
         if geo_location is not None and not isinstance(geo_location, GeoLocation):
             raise TypeError("Asset.geo_location should be of type GeoLocation")
         self.external_id = external_id
@@ -151,7 +160,7 @@ class Asset(CogniteResource):
         self._cognite_client = cast("CogniteClient", cognite_client)
 
     @classmethod
-    def _load(cls, resource: Union[Dict, str], cognite_client: CogniteClient = None) -> Asset:
+    def _load(cls, resource: dict | str, cognite_client: CogniteClient | None = None) -> Asset:
         instance = super()._load(resource, cognite_client)
         if isinstance(resource, Dict):
             if instance.aggregates is not None:
@@ -165,14 +174,14 @@ class Asset(CogniteResource):
         return hash(self.external_id)
 
     def parent(self) -> Asset:
-        """Returns this assets parent.
+        """Returns this asset's parent.
 
         Returns:
             Asset: The parent asset.
         """
         if self.parent_id is None:
-            raise ValueError("parent_id is None")
-        return self._cognite_client.assets.retrieve(id=self.parent_id)
+            raise ValueError("parent_id is None, is this a root asset?")
+        return cast(Asset, self._cognite_client.assets.retrieve(id=self.parent_id))
 
     def children(self) -> AssetList:
         """Returns the children of this asset.
@@ -180,66 +189,82 @@ class Asset(CogniteResource):
         Returns:
             AssetList: The requested assets
         """
+        assert self.id is not None
         return self._cognite_client.assets.list(parent_ids=[self.id], limit=None)
 
-    def subtree(self, depth: int = None) -> AssetList:
+    def subtree(self, depth: int | None = None) -> AssetList:
         """Returns the subtree of this asset up to a specified depth.
 
         Args:
-            depth (int, optional): Retrieve assets up to this depth below the asset.
+            depth (int | None): Retrieve assets up to this depth below the asset.
 
         Returns:
             AssetList: The requested assets sorted topologically.
         """
+        assert self.id is not None
         return self._cognite_client.assets.retrieve_subtree(id=self.id, depth=depth)
 
     def time_series(self, **kwargs: Any) -> TimeSeriesList:
         """Retrieve all time series related to this asset.
 
+        Args:
+            **kwargs (Any): All extra keyword arguments are passed to time_series/list. NB: 'asset_ids' can't be used.
         Returns:
             TimeSeriesList: All time series related to this asset.
         """
+        assert self.id is not None
         return self._cognite_client.time_series.list(asset_ids=[self.id], **kwargs)
 
     def sequences(self, **kwargs: Any) -> SequenceList:
         """Retrieve all sequences related to this asset.
 
+        Args:
+            **kwargs (Any): All extra keyword arguments are passed to sequences/list. NB: 'asset_ids' can't be used.
         Returns:
             SequenceList: All sequences related to this asset.
         """
+        assert self.id is not None
         return self._cognite_client.sequences.list(asset_ids=[self.id], **kwargs)
 
     def events(self, **kwargs: Any) -> EventList:
         """Retrieve all events related to this asset.
 
+        Args:
+            **kwargs (Any): All extra keyword arguments are passed to events/list. NB: 'asset_ids' can't be used.
         Returns:
             EventList: All events related to this asset.
         """
-
+        assert self.id is not None
         return self._cognite_client.events.list(asset_ids=[self.id], **kwargs)
 
     def files(self, **kwargs: Any) -> FileMetadataList:
         """Retrieve all files metadata related to this asset.
 
+        Args:
+            **kwargs (Any): All extra keyword arguments are passed to files/list. NB: 'asset_ids' can't be used.
         Returns:
             FileMetadataList: Metadata about all files related to this asset.
         """
+        assert self.id is not None
         return self._cognite_client.files.list(asset_ids=[self.id], **kwargs)
 
-    def dump(self, camel_case: bool = False) -> Dict[str, Any]:
+    def dump(self, camel_case: bool = False) -> dict[str, Any]:
         result = super().dump(camel_case)
         if self.labels is not None:
             result["labels"] = [label.dump(camel_case) for label in self.labels]
         return result
 
     def to_pandas(
-        self, expand: Sequence[str] = ("metadata", "aggregates"), ignore: List[str] = None, camel_case: bool = False
+        self,
+        expand: Sequence[str] = ("metadata", "aggregates"),
+        ignore: list[str] | None = None,
+        camel_case: bool = False,
     ) -> pandas.DataFrame:
         """Convert the instance into a pandas DataFrame.
 
         Args:
-            expand (List[str]): List of row keys to expand, only works if the value is a Dict.
-            ignore (List[str]): List of row keys to not include when converting to a data frame.
+            expand (Sequence[str]): List of row keys to expand, only works if the value is a Dict.
+            ignore (list[str] | None): List of row keys to not include when converting to a data frame.
             camel_case (bool): Convert column names to camel case (e.g. `externalId` instead of `external_id`)
 
         Returns:
@@ -261,33 +286,33 @@ class AssetUpdate(CogniteUpdate):
             return self._set(value)
 
     class _ObjectAssetUpdate(CogniteObjectUpdate):
-        def set(self, value: Dict) -> AssetUpdate:
+        def set(self, value: dict) -> AssetUpdate:
             return self._set(value)
 
-        def add(self, value: Dict) -> AssetUpdate:
+        def add(self, value: dict) -> AssetUpdate:
             return self._add(value)
 
-        def remove(self, value: List) -> AssetUpdate:
+        def remove(self, value: list) -> AssetUpdate:
             return self._remove(value)
 
     class _ListAssetUpdate(CogniteListUpdate):
-        def set(self, value: List) -> AssetUpdate:
+        def set(self, value: list) -> AssetUpdate:
             return self._set(value)
 
-        def add(self, value: List) -> AssetUpdate:
+        def add(self, value: list) -> AssetUpdate:
             return self._add(value)
 
-        def remove(self, value: List) -> AssetUpdate:
+        def remove(self, value: list) -> AssetUpdate:
             return self._remove(value)
 
     class _LabelAssetUpdate(CogniteLabelUpdate):
-        def set(self, value: Union[str, List[str]]) -> AssetUpdate:
+        def set(self, value: str | list[str]) -> AssetUpdate:
             return self._set(value)
 
-        def add(self, value: Union[str, List[str]]) -> AssetUpdate:
+        def add(self, value: str | list[str]) -> AssetUpdate:
             return self._add(value)
 
-        def remove(self, value: Union[str, List[str]]) -> AssetUpdate:
+        def remove(self, value: str | list[str]) -> AssetUpdate:
             return self._remove(value)
 
     @property
@@ -330,11 +355,27 @@ class AssetUpdate(CogniteUpdate):
     def geo_location(self) -> _PrimitiveAssetUpdate:
         return AssetUpdate._PrimitiveAssetUpdate(self, "geoLocation")
 
+    @classmethod
+    def _get_update_properties(cls) -> list[PropertySpec]:
+        return [
+            # External ID is nullable, but is used in the upsert logic and thus cannot be nulled out.
+            PropertySpec("external_id", is_nullable=False),
+            PropertySpec("name", is_nullable=False),
+            PropertySpec("description"),
+            PropertySpec("data_set_id"),
+            PropertySpec("metadata", is_container=True),
+            PropertySpec("source"),
+            PropertySpec("parent_id", is_nullable=False),
+            PropertySpec("parent_external_id", is_nullable=False),
+            PropertySpec("labels", is_container=True),
+            PropertySpec("geo_location"),
+        ]
 
-class AssetList(CogniteResourceList):
+
+class AssetList(CogniteResourceList[Asset], IdTransformerMixin):
     _RESOURCE = Asset
 
-    def __init__(self, resources: Collection[Any], cognite_client: CogniteClient = None):
+    def __init__(self, resources: Collection[Any], cognite_client: CogniteClient | None = None) -> None:
         super().__init__(resources, cognite_client)
         self._retrieve_chunk_size = 100
 
@@ -379,13 +420,13 @@ class AssetList(CogniteResourceList):
         return self._retrieve_related_resources(FileMetadataList, self._cognite_client.files)
 
     def _retrieve_related_resources(
-        self, resource_list_class: Type[T_CogniteResourceList], resource_api: Any
+        self, resource_list_class: type[T_CogniteResourceList], resource_api: Any
     ) -> T_CogniteResourceList:
-        seen: Set[int] = set()
+        seen: set[int] = set()
         add_to_seen = seen.add
         lock = threading.Lock()
 
-        def retrieve_and_deduplicate(asset_ids: List[int]) -> List[T_CogniteResource]:
+        def retrieve_and_deduplicate(asset_ids: list[int]) -> list[T_CogniteResource]:
             res = resource_api.list(asset_ids=asset_ids, limit=-1)
             with lock:
                 return [r for r in res if not (r.id in seen or add_to_seen(r.id))]
@@ -400,39 +441,37 @@ class AssetFilter(CogniteFilter):
     """Filter on assets with strict matching.
 
     Args:
-        name (str): The name of the asset.
-        parent_ids (Sequence[int]): Return only the direct descendants of the specified assets.
-        parent_external_ids (Sequence[str]): Return only the direct descendants of the specified assets.
-        asset_subtree_ids (Sequence[Dict[str, Any]]): Only include assets in subtrees rooted at the specified assets (including the roots given). If the total size of the given subtrees exceeds 100,000 assets, an error will be returned.
-        data_set_ids (Sequence[Dict[str, Any]]): No description.
-        metadata (Dict[str, str]): Custom, application specific metadata. String key -> String value. Limits: Maximum length of key is 128 bytes, value 10240 bytes, up to 256 key-value pairs, of total size at most 10240.
-        source (str): The source of the asset.
-        created_time (Union[Dict[str, Any], TimestampRange]): Range between two timestamps.
-        last_updated_time (Union[Dict[str, Any], TimestampRange]): Range between two timestamps.
-        root (bool): Whether the filtered assets are root assets, or not. Set to True to only list root assets.
-        external_id_prefix (str): Filter by this (case-sensitive) prefix for the external ID.
-        labels (LabelFilter): Return only the resource matching the specified label constraints.
-        geo_location (GeoLocationFilter): Only include files matching the specified geographic relation.
-        cognite_client (CogniteClient): The client to associate with this object.
+        name (str | None): The name of the asset.
+        parent_ids (Sequence[int] | None): Return only the direct descendants of the specified assets.
+        parent_external_ids (Sequence[str] | None): Return only the direct descendants of the specified assets.
+        asset_subtree_ids (Sequence[dict[str, Any]] | None): Only include assets in subtrees rooted at the specified assets (including the roots given). If the total size of the given subtrees exceeds 100,000 assets, an error will be returned.
+        data_set_ids (Sequence[dict[str, Any]] | None): No description.
+        metadata (dict[str, str] | None): Custom, application specific metadata. String key -> String value. Limits: Maximum length of key is 128 bytes, value 10240 bytes, up to 256 key-value pairs, of total size at most 10240.
+        source (str | None): The source of the asset.
+        created_time (dict[str, Any] | TimestampRange | None): Range between two timestamps.
+        last_updated_time (dict[str, Any] | TimestampRange | None): Range between two timestamps.
+        root (bool | None): Whether the filtered assets are root assets, or not. Set to True to only list root assets.
+        external_id_prefix (str | None): Filter by this (case-sensitive) prefix for the external ID.
+        labels (LabelFilter | None): Return only the resource matching the specified label constraints.
+        geo_location (GeoLocationFilter | None): Only include files matching the specified geographic relation.
     """
 
     def __init__(
         self,
-        name: str = None,
-        parent_ids: Sequence[int] = None,
-        parent_external_ids: Sequence[str] = None,
-        asset_subtree_ids: Sequence[Dict[str, Any]] = None,
-        data_set_ids: Sequence[Dict[str, Any]] = None,
-        metadata: Dict[str, str] = None,
-        source: str = None,
-        created_time: Union[Dict[str, Any], TimestampRange] = None,
-        last_updated_time: Union[Dict[str, Any], TimestampRange] = None,
-        root: bool = None,
-        external_id_prefix: str = None,
-        labels: LabelFilter = None,
-        geo_location: GeoLocationFilter = None,
-        cognite_client: CogniteClient = None,
-    ):
+        name: str | None = None,
+        parent_ids: Sequence[int] | None = None,
+        parent_external_ids: Sequence[str] | None = None,
+        asset_subtree_ids: Sequence[dict[str, Any]] | None = None,
+        data_set_ids: Sequence[dict[str, Any]] | None = None,
+        metadata: dict[str, str] | None = None,
+        source: str | None = None,
+        created_time: dict[str, Any] | TimestampRange | None = None,
+        last_updated_time: dict[str, Any] | TimestampRange | None = None,
+        root: bool | None = None,
+        external_id_prefix: str | None = None,
+        labels: LabelFilter | None = None,
+        geo_location: GeoLocationFilter | None = None,
+    ) -> None:
         self.name = name
         self.parent_ids = parent_ids
         self.parent_external_ids = parent_external_ids
@@ -446,22 +485,11 @@ class AssetFilter(CogniteFilter):
         self.external_id_prefix = external_id_prefix
         self.labels = labels
         self.geo_location = geo_location
-        self._cognite_client = cast("CogniteClient", cognite_client)
 
         if labels is not None and not isinstance(labels, LabelFilter):
             raise TypeError("AssetFilter.labels must be of type LabelFilter")
 
-    @classmethod
-    def _load(cls, resource: Union[Dict, str]) -> AssetFilter:
-        instance = super()._load(resource)
-        if isinstance(resource, Dict):
-            if instance.created_time is not None:
-                instance.created_time = TimestampRange(**instance.created_time)
-            if instance.last_updated_time is not None:
-                instance.last_updated_time = TimestampRange(**instance.last_updated_time)
-        return instance
-
-    def dump(self, camel_case: bool = False) -> Dict[str, Any]:
+    def dump(self, camel_case: bool = False) -> dict[str, Any]:
         result = super().dump(camel_case)
         if isinstance(self.labels, LabelFilter):
             result["labels"] = self.labels.dump(camel_case)
@@ -473,7 +501,11 @@ class AssetHierarchy:
     This is done "offline", meaning CDF is -not- queried for the already existing assets. As a result,
     any asset providing a parent link by ID instead of external ID, are assumed valid.
 
-    Example usage:
+    Args:
+        assets (Sequence[Asset]): Sequence of assets to be inspected for validity.
+        ignore_orphans (bool): If true, orphan assets are assumed valid and won't raise.
+
+    Examples:
 
         >>> from cognite.client import CogniteClient
         >>> from cognite.client.data_classes import AssetHierarchy
@@ -491,9 +523,9 @@ class AssetHierarchy:
         ...     hierarchy.duplicates
         ...     hierarchy.cycles  # Requires no other basic issues
 
-    There are other ways to generate the report than to write directly to screen. You may pass an
-    ``output_file`` which can be either a ``Path`` object (writes are done in append-mode) or a
-    file-like object supporting ``write`` (default is ``None`` which is just regular ``print``):
+        There are other ways to generate the report than to write directly to screen. You may pass an
+        ``output_file`` which can be either a ``Path`` object (writes are done in append-mode) or a
+        file-like object supporting ``write`` (default is ``None`` which is just regular ``print``):
 
         >>> # Get a report written to file:
         >>> from pathlib import Path
@@ -510,13 +542,13 @@ class AssetHierarchy:
 
     def __init__(self, assets: Sequence[Asset], ignore_orphans: bool = False) -> None:
         self._assets = assets
-        self._roots: Optional[List[Asset]] = None
-        self._orphans: Optional[List[Asset]] = None
+        self._roots: list[Asset] | None = None
+        self._orphans: list[Asset] | None = None
         self._ignore_orphans = ignore_orphans
-        self._invalid: Optional[List[Asset]] = None
-        self._unsure_parents: Optional[List[Asset]] = None
-        self._duplicates: Optional[Dict[str, List[Asset]]] = None
-        self._cycles: Optional[List[List[str]]] = None
+        self._invalid: list[Asset] | None = None
+        self._unsure_parents: list[Asset] | None = None
+        self._duplicates: dict[str, list[Asset]] | None = None
+        self._cycles: list[list[str]] | None = None
 
         self.__validation_has_run = False
 
@@ -559,14 +591,14 @@ class AssetHierarchy:
         return AssetList(self._unsure_parents)
 
     @property
-    def duplicates(self) -> Dict[str, List[Asset]]:
+    def duplicates(self) -> dict[str, list[Asset]]:
         if self._duplicates is None:
             raise RuntimeError("Unable to list duplicate assets before validation has run")
         # NB: Do not return AssetList (as it does not handle duplicates well):
         return {xid: assets for xid, assets in self._duplicates.items()}
 
     @property
-    def cycles(self) -> List[List[str]]:
+    def cycles(self) -> list[list[str]]:
         if self._cycles is None:
             if self.__validation_has_run:
                 self._preconditions_for_cycle_check_are_met(on_error="raise")
@@ -576,7 +608,7 @@ class AssetHierarchy:
     def validate(
         self,
         verbose: bool = False,
-        output_file: Optional[Path] = None,
+        output_file: Path | None = None,
         on_error: Literal["ignore", "warn", "raise"] = "warn",
     ) -> AssetHierarchy:
         self._roots, self._orphans, self._invalid, self._unsure_parents, self._duplicates = self._inspect_attributes()
@@ -596,10 +628,10 @@ class AssetHierarchy:
         self.is_valid(on_error=on_error)
         return self
 
-    def validate_and_report(self, output_file: Optional[Path] = None) -> AssetHierarchy:
+    def validate_and_report(self, output_file: Path | None = None) -> AssetHierarchy:
         return self.validate(verbose=True, output_file=output_file, on_error="ignore")
 
-    def groupby_parent_xid(self) -> Dict[Optional[str], List[Asset]]:
+    def groupby_parent_xid(self) -> dict[str | None, list[Asset]]:
         """Returns a mapping from parent external ID to a list of its direct children.
 
         Note:
@@ -607,7 +639,9 @@ class AssetHierarchy:
             if any, are returned as part of the root assets in the mapping and can be accessed by
             `mapping[None]`.
             The same is true for all assets linking its parent by ID.
-        """
+
+        Returns:
+            dict[str | None, list[Asset]]: No description."""
         self.is_valid(on_error="raise")
 
         # Sort (on parent) as required by groupby. This is tricky as we need to avoid comparing string with None,
@@ -638,14 +672,14 @@ class AssetHierarchy:
         )
         return mapping
 
-    def count_subtree(self, mapping: Dict[Optional[str], List[Asset]]) -> Dict[str, int]:
+    def count_subtree(self, mapping: dict[str | None, list[Asset]]) -> dict[str, int]:
         """Returns a mapping from asset external ID to the size of its subtree (children and children of chidren etc.).
 
         Args:
-            mapping (Dict | None): The mapping returned by `groupby_parent_xid()`. If None is passed, will be recreated (slightly expensive).
+            mapping (dict[str | None, list[Asset]]): The mapping returned by `groupby_parent_xid()`. If None is passed, will be recreated (slightly expensive).
 
         Returns:
-            Dict[str, int]: Lookup from external ID to descendant count.
+            dict[str, int]: Lookup from external ID to descendant count.
         """
         if mapping is None:
             mapping = self.groupby_parent_xid()
@@ -677,7 +711,7 @@ class AssetHierarchy:
             self._invalid or self._unsure_parents or self._duplicates or (self._orphans and not self._ignore_orphans)
         )
 
-    def _inspect_attributes(self) -> Tuple[List[Asset], List[Asset], List[Asset], List[Asset], Dict[str, List[Asset]]]:
+    def _inspect_attributes(self) -> tuple[list[Asset], list[Asset], list[Asset], list[Asset], dict[str, list[Asset]]]:
         invalid, orphans, roots, unsure_parents, duplicates = [], [], [], [], defaultdict(list)
         xid_count = Counter(a.external_id for a in self._assets)
 
@@ -690,12 +724,20 @@ class AssetHierarchy:
             if xid_count[xid] > 1:
                 duplicates[xid].append(asset)
 
-            if (pxid := asset.parent_external_id) is (pid := asset.parent_id) is None:
+            has_parent_id = asset.parent_id is not None
+            has_parent_xid = (parent_xid := asset.parent_external_id) is not None
+
+            if not has_parent_xid and not has_parent_id:
                 roots.append(asset)
-            elif pxid is not None and pid is not None:
-                unsure_parents.append(asset)  # Both parent links are given
-            elif pxid not in xid_count:
-                orphans.append(asset)  # Only parent XID given, but not part of assets given
+
+            elif has_parent_xid:
+                if has_parent_id:  # Both parent links are given
+                    unsure_parents.append(asset)
+
+                elif parent_xid not in xid_count:  # Only parent XID given, but not part of assets given
+                    orphans.append(asset)
+
+            # Only case left is when parent is only given by ID, which we assume is valid
 
         return roots, orphans, invalid, unsure_parents, dict(duplicates)
 
@@ -705,7 +747,7 @@ class AssetHierarchy:
         self._on_error(on_error, "Unable to run cycle-check before basic issues are fixed.")
         return False
 
-    def _locate_cycles(self) -> Tuple[int, List[List[str]]]:
+    def _locate_cycles(self) -> tuple[int, list[list[str]]]:
         has_cycles = set()
         no_cycles = {None, *(a.external_id for a in self._roots or [])}
         edges = cast(Dict[str, Optional[str]], {a.external_id: a.parent_external_id for a in self._assets})
@@ -720,17 +762,17 @@ class AssetHierarchy:
             elif parent in has_cycles or xid == parent:
                 has_cycles.add(xid)
             else:
-                self._cycle_search(cast(str, xid), parent, edges, no_cycles, has_cycles)
+                self._cycle_search(xid, parent, edges, no_cycles, has_cycles)
 
         return len(has_cycles), find_all_cycles_with_elements(has_cycles, edges)
 
     @staticmethod
     def _cycle_search(
         xid: str,
-        parent: Optional[str],
-        edges: Dict[str, Optional[str]],
-        no_cycles: Set[str | None],
-        has_cycles: Set[str],
+        parent: str | None,
+        edges: dict[str, str | None],
+        no_cycles: set[str | None],
+        has_cycles: set[str],
     ) -> None:
         seen = {xid}
         while True:
@@ -744,12 +786,12 @@ class AssetHierarchy:
                 return
 
     @staticmethod
-    def print_to(*args: Any, output_file: Union[Path, TextIO, None]) -> None:
+    def print_to(*args: Any, output_file: Path | TextIO | None) -> None:
         out = "\n".join(s.rstrip() for s in map(str, args))
         if output_file is None:
-            print(out)
+            print(out)  # noqa: T201
         elif isinstance(output_file, Path):
-            with output_file.open("a") as file:
+            with output_file.open("a", encoding="utf-8") as file:
                 print(out, file=file)
         else:
             try:
@@ -758,10 +800,10 @@ class AssetHierarchy:
             except Exception as e:
                 raise TypeError("Unable to write to `output_file`, a file-like object is required") from e
 
-    def _report_on_identifiers(self, output_file: Optional[Path]) -> None:
+    def _report_on_identifiers(self, output_file: Path | None) -> None:
         print_fn = functools.partial(self.print_to, output_file=output_file)
 
-        def print_header(title: str, columns: List[str]) -> None:
+        def print_header(title: str, columns: list[str]) -> None:
             print_fn(
                 title,
                 DrawTables.TOPLINE.join(DrawTables.HLINE * 20 for _ in columns),
@@ -769,7 +811,7 @@ class AssetHierarchy:
                 DrawTables.XLINE.join(DrawTables.HLINE * 20 for _ in columns),
             )
 
-        def print_table(lst: List[Asset], columns: List[str]) -> None:
+        def print_table(lst: list[Asset], columns: list[str]) -> None:
             for entry in lst:
                 cols = (f"{shorten(getattr(entry, col)):<20}" for col in columns)
                 print_fn(DrawTables.VLINE.join(cols))
@@ -810,7 +852,7 @@ class AssetHierarchy:
             for dupe_assets in self._duplicates.values():
                 print_table(dupe_assets, attrs_with_description)
 
-    def _report_on_cycles(self, output_file: Optional[Path], cycle_subtree_size: int) -> None:
+    def _report_on_cycles(self, output_file: Path | None, cycle_subtree_size: int) -> None:
         if not (cycles := self._cycles):
             return
 
@@ -830,3 +872,53 @@ class AssetHierarchy:
                 f"Cycle {i}/{n_cycles}:",
                 textwrap.fill(" -> ".join(map(repr, cycle)), width=80, break_on_hyphens=False),
             )
+
+
+class AssetProperty(EnumProperty):
+    labels = "labels"
+    created_time = "createdTime"
+    data_set_id = "dataSetId"
+    id = "id"
+    last_updated_time = "lastUpdatedTime"
+    parent_id = "parentId"
+    root_id = "rootId"
+    description = "description"
+    external_id = "externalId"
+    metadata = "metadata"
+    name = "name"
+    source = "source"
+
+    @staticmethod
+    def metadata_key(key: str) -> list[str]:
+        return ["metadata", key]
+
+
+AssetPropertyLike: TypeAlias = Union[AssetProperty, str, List[str]]
+
+
+class SortableAssetProperty(EnumProperty):
+    created_time = "createdTime"
+    data_set_id = "dataSetId"
+    description = "description"
+    external_id = "externalId"
+    last_updated_time = "lastUpdatedTime"
+    name = "name"
+    source = "source"
+    score = "_score_"
+
+    @staticmethod
+    def metadata_key(key: str) -> list[str]:
+        return ["metadata", key]
+
+
+SortableAssetPropertyLike: TypeAlias = Union[SortableAssetProperty, str, List[str]]
+
+
+class AssetSort(CogniteSort):
+    def __init__(
+        self,
+        property: SortableAssetProperty,
+        order: Literal["asc", "desc"] = "asc",
+        nulls: Literal["auto", "first", "last"] = "auto",
+    ):
+        super().__init__(property, order, nulls)
