@@ -4,7 +4,7 @@ import time
 import warnings
 from dataclasses import dataclass
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Sequence, Set, Tuple, Type, TypeVar, Union, cast
+from typing import TYPE_CHECKING, Any, Dict, Sequence, Type, TypeVar, Union, cast
 
 from requests.utils import CaseInsensitiveDict
 
@@ -14,6 +14,7 @@ from cognite.client.data_classes._base import (
     CogniteResource,
     CogniteResourceList,
     CogniteUpdate,
+    PropertySpec,
 )
 from cognite.client.data_classes.annotation_types.images import AssetLink, ObjectDetection, TextRegion
 from cognite.client.data_classes.annotation_types.primitives import VisionResource
@@ -59,6 +60,8 @@ class ContextualizationJobType(Enum):
 
 
 class ContextualizationJob(CogniteResource):
+    """Data class for the result of a contextualization job."""
+
     _COMMON_FIELDS = frozenset(
         {
             "status",
@@ -76,18 +79,17 @@ class ContextualizationJob(CogniteResource):
 
     def __init__(
         self,
-        job_id: int = None,
-        model_id: int = None,
-        status: str = None,
-        error_message: str = None,
-        created_time: int = None,
-        start_time: int = None,
-        status_time: int = None,
-        status_path: str = None,
-        job_token: str = None,
-        cognite_client: CogniteClient = None,
-    ):
-        """Data class for the result of a contextualization job."""
+        job_id: int | None = None,
+        model_id: int | None = None,
+        status: str | None = None,
+        error_message: str | None = None,
+        created_time: int | None = None,
+        start_time: int | None = None,
+        status_time: int | None = None,
+        status_path: str | None = None,
+        job_token: str | None = None,
+        cognite_client: CogniteClient | None = None,
+    ) -> None:
         self.job_id = job_id
         self.model_id = model_id
         self.status = status
@@ -97,7 +99,7 @@ class ContextualizationJob(CogniteResource):
         self.error_message = error_message
         self.job_token = job_token
         self._cognite_client = cast("CogniteClient", cognite_client)
-        self._result: Optional[Dict[str, Any]] = None
+        self._result: dict[str, Any] | None = None
         self._status_path = status_path
 
     def update_status(self) -> str:
@@ -117,11 +119,11 @@ class ContextualizationJob(CogniteResource):
         assert self.status is not None
         return self.status
 
-    def wait_for_completion(self, timeout: int = None, interval: int = 1) -> None:
+    def wait_for_completion(self, timeout: int | None = None, interval: int = 1) -> None:
         """Waits for job completion. This is generally not needed to call directly, as `.result` will do so automatically.
 
         Args:
-            timeout (int): Time out after this many seconds. (None means wait indefinitely)
+            timeout (int | None): Time out after this many seconds. (None means wait indefinitely)
             interval (int): Poll status every this many seconds.
 
         Raises:
@@ -137,7 +139,7 @@ class ContextualizationJob(CogniteResource):
             raise ModelFailedException(self.__class__.__name__, cast(int, self.job_id), cast(str, self.error_message))
 
     @property
-    def result(self) -> Dict[str, Any]:
+    def result(self) -> dict[str, Any]:
         """Waits for the job to finish and returns the results."""
         if not self._result:
             self.wait_for_completion()
@@ -149,47 +151,51 @@ class ContextualizationJob(CogniteResource):
 
     @classmethod
     def _load_with_status(
-        cls: Type[T_ContextualizationJob],
+        cls: type[T_ContextualizationJob],
         *,
-        data: Dict[str, Any],
+        data: dict[str, Any],
         headers: CaseInsensitiveDict[str],
         status_path: str,
         cognite_client: Any,
     ) -> T_ContextualizationJob:
         obj = cls._load({**data, "jobToken": headers.get("X-Job-Token")}, cognite_client=cognite_client)
         obj._status_path = status_path
+        # '_load' does not see properties (real attribute stored under a different name, e.g. '_items' not 'items'):
+        if "items" in data and hasattr(obj, "items"):
+            obj.items = data["items"]
         return obj
 
 
 T_ContextualizationJob = TypeVar("T_ContextualizationJob", bound=ContextualizationJob)
 
 
-class ContextualizationJobList(CogniteResourceList):
+class ContextualizationJobList(CogniteResourceList[ContextualizationJob]):
     _RESOURCE = ContextualizationJob
 
 
 class EntityMatchingModel(CogniteResource):
+    """Entity matching model. See the `fit` method for the meaning of these fields."""
+
     _RESOURCE_PATH = "/context/entitymatching"
     _STATUS_PATH = _RESOURCE_PATH + "/"
 
     def __init__(
         self,
-        id: int = None,
-        status: str = None,
-        error_message: str = None,
-        created_time: int = None,
-        start_time: int = None,
-        status_time: int = None,
-        classifier: str = None,
-        feature_type: str = None,
-        match_fields: List[str] = None,
-        model_type: str = None,
-        name: str = None,
-        description: str = None,
-        external_id: str = None,
-        cognite_client: CogniteClient = None,
-    ):
-        """Entity matching model. See the `fit` method for the meaning of these fields."""
+        id: int | None = None,
+        status: str | None = None,
+        error_message: str | None = None,
+        created_time: int | None = None,
+        start_time: int | None = None,
+        status_time: int | None = None,
+        classifier: str | None = None,
+        feature_type: str | None = None,
+        match_fields: list[str] | None = None,
+        model_type: str | None = None,
+        name: str | None = None,
+        description: str | None = None,
+        external_id: str | None = None,
+        cognite_client: CogniteClient | None = None,
+    ) -> None:
         self.id = id
         self.status = status
         self.created_time = created_time
@@ -219,12 +225,12 @@ class EntityMatchingModel(CogniteResource):
         assert self.status is not None
         return self.status
 
-    def wait_for_completion(self, timeout: int = None, interval: int = 1) -> None:
+    def wait_for_completion(self, timeout: int | None = None, interval: int = 1) -> None:
         """Waits for model completion. This is generally not needed to call directly, as `.result` will do so automatically.
 
         Args:
-            timeout: Time out after this many seconds. (None means wait indefinitely)
-            interval: Poll status every this many seconds.
+            timeout (int | None): Time out after this many seconds. (None means wait indefinitely)
+            interval (int): Poll status every this many seconds.
 
         Raises:
             ModelFailedException: The model fit failed.
@@ -242,19 +248,18 @@ class EntityMatchingModel(CogniteResource):
 
     def predict(
         self,
-        sources: Optional[List[Dict]] = None,
-        targets: Optional[List[Dict]] = None,
+        sources: list[dict] | None = None,
+        targets: list[dict] | None = None,
         num_matches: int = 1,
-        score_threshold: float = None,
+        score_threshold: float | None = None,
     ) -> ContextualizationJob:
         """Predict entity matching. NB. blocks and waits for the model to be ready if it has been recently created.
 
         Args:
-            sources: entities to match from, does not need an 'id' field. Tolerant to passing more than is needed or used (e.g. json dump of time series list). If omitted, will use data from fit.
-            targets: entities to match to, does not need an 'id' field.  Tolerant to passing more than is needed or used. If omitted, will use data from fit.
+            sources (list[dict] | None): entities to match from, does not need an 'id' field. Tolerant to passing more than is needed or used (e.g. json dump of time series list). If omitted, will use data from fit.
+            targets (list[dict] | None): entities to match to, does not need an 'id' field.  Tolerant to passing more than is needed or used. If omitted, will use data from fit.
             num_matches (int): number of matches to return for each item.
-            score_threshold (float): only return matches with a score above this threshold
-            ignore_missing_fields (bool): whether missing data in match_fields should be filled in with an empty string.
+            score_threshold (float | None): only return matches with a score above this threshold
 
         Returns:
             ContextualizationJob: object which can be used to wait for and retrieve results."""
@@ -272,14 +277,11 @@ class EntityMatchingModel(CogniteResource):
             },
         )
 
-    def refit(
-        self, true_matches: Sequence[Union[Dict, Tuple[Union[int, str], Union[int, str]]]]
-    ) -> EntityMatchingModel:
+    def refit(self, true_matches: Sequence[dict | tuple[int | str, int | str]]) -> EntityMatchingModel:
         """Re-fits an entity matching model, using the combination of the old and new true matches.
 
         Args:
-            true_matches: Updated known valid matches given as a list of dicts with keys 'fromId', 'fromExternalId', 'toId', 'toExternalId').
-                A tuple can be used instead of the dictionary for convenience, interpreted as id/externalId based on type.
+            true_matches (Sequence[dict | tuple[int | str, int | str]]): Updated known valid matches given as a list of dicts with keys 'fromId', 'fromExternalId', 'toId', 'toExternalId'). A tuple can be used instead of the dictionary for convenience, interpreted as id/externalId based on type.
         Returns:
             EntityMatchingModel: new model refitted to true_matches."""
         true_matches = [convert_true_match(true_match) for true_match in true_matches]
@@ -290,7 +292,7 @@ class EntityMatchingModel(CogniteResource):
         return self._load(response.json(), cognite_client=self._cognite_client)
 
     @staticmethod
-    def _flatten_entity(entity: Union[dict, CogniteResource]) -> Dict:
+    def _flatten_entity(entity: dict | CogniteResource) -> dict:
         if isinstance(entity, CogniteResource):
             entity = entity.dump(camel_case=True)
         if "metadata" in entity and isinstance(entity["metadata"], dict):
@@ -299,7 +301,7 @@ class EntityMatchingModel(CogniteResource):
         return {k: v for k, v in entity.items() if k == "id" or isinstance(v, str)}
 
     @staticmethod
-    def _dump_entities(entities: Optional[Sequence[Union[Dict, CogniteResource]]]) -> Optional[List[Dict]]:
+    def _dump_entities(entities: Sequence[dict | CogniteResource] | None) -> list[dict] | None:
         if entities:
             return [EntityMatchingModel._flatten_entity(e) for e in entities]
         return None
@@ -325,19 +327,26 @@ class EntityMatchingModelUpdate(CogniteUpdate):
     def description(self) -> _PrimitiveUpdate:
         return EntityMatchingModelUpdate._PrimitiveUpdate(self, "description")
 
+    @classmethod
+    def _get_update_properties(cls) -> list[PropertySpec]:
+        return [
+            PropertySpec("name", is_nullable=False),
+            PropertySpec("description", is_nullable=False),
+        ]
 
-class EntityMatchingModelList(CogniteResourceList):
+
+class EntityMatchingModelList(CogniteResourceList[EntityMatchingModel]):
     _RESOURCE = EntityMatchingModel
 
 
 class FileReference:
     def __init__(
         self,
-        file_id: Optional[int] = None,
-        file_external_id: Optional[str] = None,
-        first_page: Optional[int] = None,
-        last_page: Optional[int] = None,
-    ):
+        file_id: int | None = None,
+        file_external_id: str | None = None,
+        first_page: int | None = None,
+        last_page: int | None = None,
+    ) -> None:
         self.file_id = file_id
         self.file_external_id = file_external_id
         self.first_page = first_page
@@ -348,9 +357,9 @@ class FileReference:
         if exactly_one_is_not_none(first_page, last_page):
             raise ValueError("If the page range feature is used, both first page and last page must be set")
 
-    def to_api_item(self) -> Dict[str, Union[str, int, Dict[str, int]]]:
+    def to_api_item(self) -> dict[str, str | int | dict[str, int]]:
         if self.file_id is None and self.file_external_id is not None:
-            item: Dict[str, Union[str, int, Dict[str, int]]] = {"fileExternalId": self.file_external_id}
+            item: dict[str, str | int | dict[str, int]] = {"fileExternalId": self.file_external_id}
         if self.file_id is not None and self.file_external_id is None:
             item = {"fileId": self.file_id}
         if self.first_page is not None and self.last_page is not None:
@@ -360,26 +369,30 @@ class FileReference:
 
 class DiagramConvertPage(CogniteResource):
     def __init__(
-        self, page: int = None, png_url: str = None, svg_url: str = None, cognite_client: CogniteClient = None
-    ):
+        self,
+        page: int | None = None,
+        png_url: str | None = None,
+        svg_url: str | None = None,
+        cognite_client: CogniteClient | None = None,
+    ) -> None:
         self.page = page
         self.png_url = png_url
         self.svg_url = svg_url
         self._cognite_client = cast("CogniteClient", cognite_client)
 
 
-class DiagramConvertPageList(CogniteResourceList):
+class DiagramConvertPageList(CogniteResourceList[DiagramConvertPage]):
     _RESOURCE = DiagramConvertPage
 
 
 class DiagramConvertItem(CogniteResource):
     def __init__(
         self,
-        file_id: int = None,
-        file_external_id: str = None,
-        results: list = None,
-        cognite_client: CogniteClient = None,
-    ):
+        file_id: int | None = None,
+        file_external_id: str | None = None,
+        results: list | None = None,
+        cognite_client: CogniteClient | None = None,
+    ) -> None:
         self.file_id = file_id
         self.file_external_id = file_external_id
         self.results = results
@@ -413,7 +426,7 @@ class DiagramConvertResults(ContextualizationJob):
 
     def __init__(self, **kwargs: Any) -> None:
         super().__init__(**kwargs)
-        self._items: Optional[list] = None
+        self._items: list | None = None
 
     def __getitem__(self, find_id: Any) -> DiagramConvertItem:
         """Retrieves the results for the file with (external) id"""
@@ -429,7 +442,7 @@ class DiagramConvertResults(ContextualizationJob):
         return DiagramConvertItem._load(found[0], cognite_client=self._cognite_client)
 
     @property
-    def items(self) -> Optional[List[DiagramConvertItem]]:
+    def items(self) -> list[DiagramConvertItem] | None:
         """returns a list of all results by file"""
         if JobStatus(self.status) is JobStatus.COMPLETED:
             self._items = [
@@ -445,19 +458,21 @@ class DiagramConvertResults(ContextualizationJob):
 class DiagramDetectItem(CogniteResource):
     def __init__(
         self,
-        file_id: int = None,
-        file_external_id: str = None,
-        annotations: list = None,
-        error_message: str = None,
-        cognite_client: CogniteClient = None,
-        page_range: Optional[Dict[str, int]] = None,
-    ):
+        file_id: int | None = None,
+        file_external_id: str | None = None,
+        annotations: list | None = None,
+        error_message: str | None = None,
+        cognite_client: CogniteClient | None = None,
+        page_range: dict[str, int] | None = None,
+        page_count: int | None = None,
+    ) -> None:
         self.file_id = file_id
         self.file_external_id = file_external_id
         self.annotations = annotations
         self.error_message = error_message
         self._cognite_client = cast("CogniteClient", cognite_client)
         self.page_range = page_range
+        self.page_count = page_count
 
     def to_pandas(self, camel_case: bool = False) -> pandas.DataFrame:  # type: ignore[override]
         """Convert the instance into a pandas DataFrame.
@@ -478,7 +493,7 @@ class DiagramDetectResults(ContextualizationJob):
 
     def __init__(self, **kwargs: Any) -> None:
         super().__init__(**kwargs)
-        self._items: Optional[List[DiagramDetectItem]] = None
+        self._items: list[DiagramDetectItem] | None = None
 
     def __getitem__(self, find_id: Any) -> DiagramDetectItem:
         """retrieves the results for the file with (external) id"""
@@ -494,7 +509,7 @@ class DiagramDetectResults(ContextualizationJob):
         return DiagramDetectItem._load(found[0], cognite_client=self._cognite_client)
 
     @property
-    def items(self) -> Optional[List[DiagramDetectItem]]:
+    def items(self) -> list[DiagramDetectItem] | None:
         """Returns a list of all results by file"""
         if JobStatus(self.status) is JobStatus.COMPLETED:
             self._items = [
@@ -503,11 +518,11 @@ class DiagramDetectResults(ContextualizationJob):
         return self._items
 
     @items.setter
-    def items(self, items: List[DiagramDetectItem]) -> None:
+    def items(self, items: list[DiagramDetectItem]) -> None:
         self._items = items
 
     @property
-    def errors(self) -> List[str]:
+    def errors(self) -> list[str]:
         """Returns a list of all error messages across files"""
         return [item["errorMessage"] for item in self.result["items"] if "errorMessage" in item]
 
@@ -529,21 +544,21 @@ class VisionFeature(str, Enum):
     PERSONAL_PROTECTIVE_EQUIPMENT_DETECTION = "PersonalProtectiveEquipmentDetection"
 
     @classmethod
-    def beta_features(cls) -> Set[VisionFeature]:
+    def beta_features(cls) -> set[VisionFeature]:
         """Returns a set of VisionFeature's that are currently in beta"""
         return {cls.INDUSTRIAL_OBJECT_DETECTION, cls.PERSONAL_PROTECTIVE_EQUIPMENT_DETECTION}
 
 
 @dataclass
 class VisionExtractPredictions(VisionResource):
-    text_predictions: Optional[List[TextRegion]] = None
-    asset_tag_predictions: Optional[List[AssetLink]] = None
-    industrial_object_predictions: Optional[List[ObjectDetection]] = None
-    people_predictions: Optional[List[ObjectDetection]] = None
-    personal_protective_equipment_predictions: Optional[List[ObjectDetection]] = None
+    text_predictions: list[TextRegion] | None = None
+    asset_tag_predictions: list[AssetLink] | None = None
+    industrial_object_predictions: list[ObjectDetection] | None = None
+    people_predictions: list[ObjectDetection] | None = None
+    personal_protective_equipment_predictions: list[ObjectDetection] | None = None
 
 
-VISION_FEATURE_MAP: Dict[str, Any] = {
+VISION_FEATURE_MAP: dict[str, Any] = {
     "text_predictions": TextRegion,
     "asset_tag_predictions": AssetLink,
     "industrial_object_predictions": ObjectDetection,
@@ -552,7 +567,7 @@ VISION_FEATURE_MAP: Dict[str, Any] = {
 }
 
 
-VISION_ANNOTATION_TYPE_MAP: Dict[str, str] = {
+VISION_ANNOTATION_TYPE_MAP: dict[str, str] = {
     "text_predictions": "images.TextRegion",
     "asset_tag_predictions": "images.AssetLink",
     "industrial_object_predictions": "images.ObjectDetection",
@@ -563,13 +578,13 @@ VISION_ANNOTATION_TYPE_MAP: Dict[str, str] = {
 
 @dataclass
 class ThresholdParameter:
-    threshold: Optional[float] = None
+    threshold: float | None = None
 
 
 @dataclass
 class AssetTagDetectionParameters(VisionResource, ThresholdParameter):
-    partial_match: Optional[bool] = None
-    asset_subtree_ids: Optional[List[int]] = None
+    partial_match: bool | None = None
+    asset_subtree_ids: list[int] | None = None
 
 
 @dataclass
@@ -597,7 +612,7 @@ class DetectJobBundle:
     _STATUS_PATH = "/context/diagram/detect/status"
     _WAIT_TIME = 2
 
-    def __init__(self, job_ids: List[int], cognite_client: CogniteClient = None):
+    def __init__(self, job_ids: list[int], cognite_client: CogniteClient | None = None) -> None:
         warnings.warn(
             "DetectJobBundle.result is calling a beta endpoint which is still in development. "
             "Breaking changes can happen in between patch versions."
@@ -607,11 +622,11 @@ class DetectJobBundle:
         if not job_ids:
             raise ValueError("You need to specify job_ids")
         self.job_ids = job_ids
-        self._remaining_job_ids: List[int] = []
+        self._remaining_job_ids: list[int] = []
 
-        self.jobs: List[Dict[str, Any]] = []
+        self.jobs: list[dict[str, Any]] = []
 
-        self._result: List[Dict[str, Any]] = []
+        self._result: list[dict[str, Any]] = []
 
     def __str__(self) -> str:
         return f"DetectJobBundle({self.job_ids=}, {self.jobs=}, {self._result=}, {self._remaining_job_ids=})"
@@ -625,12 +640,11 @@ class DetectJobBundle:
         if self._WAIT_TIME < 10:
             self._WAIT_TIME += 2
 
-    def wait_for_completion(self, timeout: int = None) -> None:
+    def wait_for_completion(self, timeout: int | None = None) -> None:
         """Waits for all jobs to complete, generally not needed to call as it is called by result.
 
         Args:
-            timeout (int): Time out after this many seconds. (None means wait indefinitely)
-            interval (int): Poll status every this many seconds.
+            timeout (int | None): Time out after this many seconds. (None means wait indefinitely)
         """
         start = time.time()
         self._remaining_job_ids = self.job_ids
@@ -653,11 +667,11 @@ class DetectJobBundle:
                 self._WAIT_TIME = 2
                 break
 
-    def fetch_results(self) -> List[Dict[str, Any]]:
+    def fetch_results(self) -> list[dict[str, Any]]:
         return [self._cognite_client.diagrams._get(f"{self._RESOURCE_PATH}{j}").json() for j in self.job_ids]
 
     @property
-    def result(self) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]]]:
+    def result(self) -> tuple[list[dict[str, Any]], list[dict[str, Any]]]:
         """Waits for the job to finish and returns the results."""
         if not self._result:
             self.wait_for_completion()
@@ -665,8 +679,8 @@ class DetectJobBundle:
             self._result = self.fetch_results()
         assert self._result is not None
         # Sort into succeeded and failed
-        failed: List[Dict[str, Any]] = []
-        succeeded: List[Dict[str, Any]] = []
+        failed: list[dict[str, Any]] = []
+        succeeded: list[dict[str, Any]] = []
         for job_result in self._result:
             for item in job_result["items"]:
                 if "errorMessage" in item:
@@ -678,11 +692,11 @@ class DetectJobBundle:
 
 @dataclass
 class FeatureParameters(VisionResource):
-    text_detection_parameters: Optional[TextDetectionParameters] = None
-    asset_tag_detection_parameters: Optional[AssetTagDetectionParameters] = None
-    people_detection_parameters: Optional[PeopleDetectionParameters] = None
-    industrial_object_detection_parameters: Optional[IndustrialObjectDetectionParameters] = None
-    personal_protective_equipment_detection_parameters: Optional[PersonalProtectiveEquipmentDetectionParameters] = None
+    text_detection_parameters: TextDetectionParameters | None = None
+    asset_tag_detection_parameters: AssetTagDetectionParameters | None = None
+    people_detection_parameters: PeopleDetectionParameters | None = None
+    industrial_object_detection_parameters: IndustrialObjectDetectionParameters | None = None
+    personal_protective_equipment_detection_parameters: PersonalProtectiveEquipmentDetectionParameters | None = None
 
 
 class VisionJob(ContextualizationJob):
@@ -701,15 +715,16 @@ class VisionJob(ContextualizationJob):
 
 
 class VisionExtractItem(CogniteResource):
+    """Data class for storing predictions for a single image file"""
+
     def __init__(
         self,
-        file_id: int = None,
-        predictions: Dict[str, Any] = None,
-        file_external_id: str = None,
-        error_message: str = None,
-        cognite_client: CogniteClient = None,
+        file_id: int | None = None,
+        predictions: dict[str, Any] | None = None,
+        file_external_id: str | None = None,
+        error_message: str | None = None,
+        cognite_client: CogniteClient | None = None,
     ) -> None:
-        """Data class for storing predictions for a single image file"""
         self.file_id = file_id
         self.file_external_id = file_external_id
         self.error_message = error_message
@@ -719,7 +734,7 @@ class VisionExtractItem(CogniteResource):
         self._cognite_client = cast("CogniteClient", cognite_client)
 
     @classmethod
-    def _load(cls, resource: Union[Dict, str], cognite_client: CogniteClient = None) -> VisionExtractItem:
+    def _load(cls, resource: dict | str, cognite_client: CogniteClient | None = None) -> VisionExtractItem:
         """Override CogniteResource._load so that we can convert the dicts returned by the API to data classes"""
         extracted_item = super()._load(resource, cognite_client=cognite_client)
         if isinstance(extracted_item.predictions, dict):
@@ -727,7 +742,7 @@ class VisionExtractItem(CogniteResource):
             extracted_item.predictions = cls._process_predictions_dict(extracted_item._predictions_dict)
         return extracted_item
 
-    def dump(self, camel_case: bool = False) -> Dict[str, Any]:
+    def dump(self, camel_case: bool = False) -> dict[str, Any]:
         item_dump = super().dump(camel_case=camel_case)
         # Replace the loaded VisionExtractPredictions with its corresponding dict representation
         if "predictions" in item_dump and isinstance(self._predictions_dict, Dict):
@@ -737,7 +752,7 @@ class VisionExtractItem(CogniteResource):
         return item_dump
 
     @classmethod
-    def _process_predictions_dict(cls, predictions_dict: Dict[str, Any]) -> VisionExtractPredictions:
+    def _process_predictions_dict(cls, predictions_dict: dict[str, Any]) -> VisionExtractPredictions:
         """Converts a (validated) predictions dict to a corresponding VisionExtractPredictions"""
         prediction_object = VisionExtractPredictions()
         snake_case_predictions_dict = cls._resource_to_snake_case(predictions_dict)
@@ -763,7 +778,7 @@ class VisionExtractJob(VisionJob):
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
-        self._items: Optional[List[VisionExtractItem]] = None
+        self._items: list[VisionExtractItem] | None = None
 
     def __getitem__(self, file_id: int) -> VisionExtractItem:
         """Retrieves the results for a file by id"""
@@ -773,7 +788,7 @@ class VisionExtractJob(VisionJob):
         return VisionExtractItem._load(found[0], cognite_client=self._cognite_client)
 
     @property
-    def items(self) -> Optional[List[VisionExtractItem]]:
+    def items(self) -> list[VisionExtractItem] | None:
         """Returns a list of all predictions by file"""
         if JobStatus(self.status) is JobStatus.COMPLETED:
             self._items = [
@@ -782,20 +797,20 @@ class VisionExtractJob(VisionJob):
         return self._items
 
     @items.setter
-    def items(self, items: List[VisionExtractItem]) -> None:
+    def items(self, items: list[VisionExtractItem]) -> None:
         self._items = items
 
     @property
-    def errors(self) -> List[str]:
+    def errors(self) -> list[str]:
         """Returns a list of all error messages across files"""
         return [item["errorMessage"] for item in self.result["items"] if "errorMessage" in item]
 
     def _predictions_to_annotations(
         self,
-        creating_user: Optional[str] = None,
-        creating_app: Optional[str] = None,
-        creating_app_version: Optional[str] = None,
-    ) -> List[Annotation]:
+        creating_user: str | None = None,
+        creating_app: str | None = None,
+        creating_app_version: str | None = None,
+    ) -> list[Annotation]:
         return [
             Annotation(
                 annotated_resource_id=item.file_id,
@@ -815,20 +830,20 @@ class VisionExtractJob(VisionJob):
 
     def save_predictions(
         self,
-        creating_user: Optional[str] = None,
-        creating_app: Optional[str] = None,
-        creating_app_version: Optional[str] = None,
-    ) -> Union[Annotation, AnnotationList]:
+        creating_user: str | None = None,
+        creating_app: str | None = None,
+        creating_app_version: str | None = None,
+    ) -> Annotation | AnnotationList:
         """
         Saves all predictions made by the feature extractors in CDF using the Annotations API.
-        See https://docs.cognite.com/api/v1/#tag/Annotations/operation/annotationsSuggest
+        See https://docs.cognite.com/api/v1/#operation/annotationsSuggest
 
         Args:
-            creating_app (str, optional): The name of the app from which this annotation was created. Defaults to 'cognite-sdk-python'.
-            creating_app_version (str, optional): The version of the app that created this annotation. Must be a valid semantic versioning (SemVer) string. Defaults to client version.
-            creating_user: (str, optional): A username, or email, or name. This is not checked nor enforced. If the value is None, it means the annotation was created by a service.
+            creating_user (str | None): (str, optional): A username, or email, or name. This is not checked nor enforced. If the value is None, it means the annotation was created by a service.
+            creating_app (str | None): The name of the app from which this annotation was created. Defaults to 'cognite-sdk-python'.
+            creating_app_version (str | None): The version of the app that created this annotation. Must be a valid semantic versioning (SemVer) string. Defaults to client version.
         Returns:
-            Union[Annotation, AnnotationList]: (suggested) annotation(s) stored in CDF.
+            Annotation | AnnotationList: (suggested) annotation(s) stored in CDF.
 
         """
         if JobStatus(self.status) is JobStatus.COMPLETED:
@@ -842,3 +857,16 @@ class VisionExtractJob(VisionJob):
         raise CogniteException(
             "Extract job is not completed. If the job is queued or running, wait for completion and try again"
         )
+
+
+class ResourceReference(CogniteResource):
+    def __init__(
+        self, id: int | None = None, external_id: str | None = None, cognite_client: None | CogniteClient = None
+    ) -> None:
+        self.id = id
+        self.external_id = external_id
+        self._cognite_client: CogniteClient = cast("CogniteClient", None)  # Read only
+
+
+class ResourceReferenceList(CogniteResourceList[ResourceReference]):
+    _RESOURCE = ResourceReference
