@@ -2,9 +2,10 @@ from __future__ import annotations
 
 import json
 import math
-from typing import TYPE_CHECKING, Any, Dict, Generator, List
+from typing import TYPE_CHECKING, Any, Iterator, List, Literal, Union, cast
 from typing import Sequence as SequenceType
-from typing import Tuple, Union, cast
+
+from typing_extensions import TypeAlias
 
 from cognite.client import utils
 from cognite.client.data_classes._base import (
@@ -16,7 +17,11 @@ from cognite.client.data_classes._base import (
     CognitePropertyClassUtil,
     CogniteResource,
     CogniteResourceList,
+    CogniteSort,
     CogniteUpdate,
+    EnumProperty,
+    IdTransformerMixin,
+    PropertySpec,
 )
 from cognite.client.data_classes.shared import TimestampRange
 from cognite.client.utils._identifier import Identifier
@@ -32,33 +37,33 @@ class Sequence(CogniteResource):
     """Information about the sequence stored in the database
 
     Args:
-        id (int): Unique cognite-provided identifier for the sequence
-        name (str): Name of the sequence
-        description (str): Description of the sequence
-        asset_id (int): Optional asset this sequence is associated with
-        external_id (str): The external ID provided by the client. Must be unique for the resource type.
-        metadata (Dict[str, Any]): Custom, application specific metadata. String key -> String value. Maximum length of key is 32 bytes, value 512 bytes, up to 16 key-value pairs.
-        columns (SequenceType[Dict[str, Any]]): List of column definitions
-        created_time (int): Time when this sequence was created in CDF in milliseconds since Jan 1, 1970.
-        last_updated_time (int): The last time this sequence was updated in CDF, in milliseconds since Jan 1, 1970.
-        data_set_id (int): Data set that this sequence belongs to
-        cognite_client (CogniteClient): The client to associate with this object.
+        id (int | None): Unique cognite-provided identifier for the sequence
+        name (str | None): Name of the sequence
+        description (str | None): Description of the sequence
+        asset_id (int | None): Optional asset this sequence is associated with
+        external_id (str | None): The external ID provided by the client. Must be unique for the resource type.
+        metadata (dict[str, Any] | None): Custom, application specific metadata. String key -> String value. Maximum length of key is 32 bytes, value 512 bytes, up to 16 key-value pairs.
+        columns (SequenceType[dict[str, Any]] | None): List of column definitions
+        created_time (int | None): Time when this sequence was created in CDF in milliseconds since Jan 1, 1970.
+        last_updated_time (int | None): The last time this sequence was updated in CDF, in milliseconds since Jan 1, 1970.
+        data_set_id (int | None): Data set that this sequence belongs to
+        cognite_client (CogniteClient | None): The client to associate with this object.
     """
 
     def __init__(
         self,
-        id: int = None,
-        name: str = None,
-        description: str = None,
-        asset_id: int = None,
-        external_id: str = None,
-        metadata: Dict[str, Any] = None,
-        columns: SequenceType[Dict[str, Any]] = None,
-        created_time: int = None,
-        last_updated_time: int = None,
-        data_set_id: int = None,
-        cognite_client: CogniteClient = None,
-    ):
+        id: int | None = None,
+        name: str | None = None,
+        description: str | None = None,
+        asset_id: int | None = None,
+        external_id: str | None = None,
+        metadata: dict[str, Any] | None = None,
+        columns: SequenceType[dict[str, Any]] | None = None,
+        created_time: int | None = None,
+        last_updated_time: int | None = None,
+        data_set_id: int | None = None,
+        cognite_client: CogniteClient | None = None,
+    ) -> None:
         self.id = id
         self.name = name
         self.description = description
@@ -71,31 +76,35 @@ class Sequence(CogniteResource):
         self.data_set_id = data_set_id
         self._cognite_client = cast("CogniteClient", cognite_client)
 
-    def rows(self, start: int, end: int) -> List[dict]:
+    def rows(self, start: int, end: int | None) -> SequenceData:
         """Retrieves rows from this sequence.
 
+        Args:
+            start (int): Row number to start from (inclusive).
+            end (int | None): Upper limit on the row number (exclusive). Set to None or -1 to get all rows until end of sequence.
+
         Returns:
-            List of sequence data.
+            SequenceData: List of sequence data.
         """
         identifier = Identifier.load(self.id, self.external_id).as_dict()
-        return self._cognite_client.sequences.data.retrieve(**identifier, start=start, end=end)
+        return cast(SequenceData, self._cognite_client.sequences.data.retrieve(**identifier, start=start, end=end))
 
     @property
-    def column_external_ids(self) -> List[str]:
+    def column_external_ids(self) -> list[str]:
         """Retrieves list of column external ids for the sequence, for use in e.g. data retrieve or insert methods
 
         Returns:
-            List of sequence column external ids
+            list[str]: List of sequence column external ids
         """
         assert self.columns is not None
         return [cast(str, c.get("externalId")) for c in self.columns]
 
     @property
-    def column_value_types(self) -> List[str]:
+    def column_value_types(self) -> list[str]:
         """Retrieves list of column value types
 
         Returns:
-            List of column value types
+            list[str]: List of column value types
         """
         assert self.columns is not None
         return [cast(str, c.get("valueType")) for c in self.columns]
@@ -105,29 +114,27 @@ class SequenceFilter(CogniteFilter):
     """No description.
 
     Args:
-        name (str): Return only sequences with this *exact* name.
-        external_id_prefix (str): Filter by this (case-sensitive) prefix for the external ID.
-        metadata (Dict[str, Any]): Filter the sequences by metadata fields and values (case-sensitive). Format is {"key1":"value1","key2":"value2"}.
-        asset_ids (SequenceType[int]): Return only sequences linked to one of the specified assets.
-        asset_subtree_ids (SequenceType[Dict[str, Any]]): Only include sequences that have a related asset in a subtree rooted at any of these assetIds (including the roots given). If the total size of the given subtrees exceeds 100,000 assets, an error will be returned.
-        created_time (Union[Dict[str, Any], TimestampRange]): Range between two timestamps.
-        last_updated_time (Union[Dict[str, Any], TimestampRange]): Range between two timestamps.
-        data_set_ids (SequenceType[Dict[str, Any]]): Only include sequences that belong to these datasets.
-        cognite_client (CogniteClient): The client to associate with this object.
+        name (str | None): Return only sequences with this *exact* name.
+        external_id_prefix (str | None): Filter by this (case-sensitive) prefix for the external ID.
+        metadata (dict[str, Any] | None): Filter the sequences by metadata fields and values (case-sensitive). Format is {"key1":"value1","key2":"value2"}.
+        asset_ids (SequenceType[int] | None): Return only sequences linked to one of the specified assets.
+        asset_subtree_ids (SequenceType[dict[str, Any]] | None): Only include sequences that have a related asset in a subtree rooted at any of these assetIds (including the roots given). If the total size of the given subtrees exceeds 100,000 assets, an error will be returned.
+        created_time (dict[str, Any] | TimestampRange | None): Range between two timestamps.
+        last_updated_time (dict[str, Any] | TimestampRange | None): Range between two timestamps.
+        data_set_ids (SequenceType[dict[str, Any]] | None): Only include sequences that belong to these datasets.
     """
 
     def __init__(
         self,
-        name: str = None,
-        external_id_prefix: str = None,
-        metadata: Dict[str, Any] = None,
-        asset_ids: SequenceType[int] = None,
-        asset_subtree_ids: SequenceType[Dict[str, Any]] = None,
-        created_time: Union[Dict[str, Any], TimestampRange] = None,
-        last_updated_time: Union[Dict[str, Any], TimestampRange] = None,
-        data_set_ids: SequenceType[Dict[str, Any]] = None,
-        cognite_client: CogniteClient = None,
-    ):
+        name: str | None = None,
+        external_id_prefix: str | None = None,
+        metadata: dict[str, Any] | None = None,
+        asset_ids: SequenceType[int] | None = None,
+        asset_subtree_ids: SequenceType[dict[str, Any]] | None = None,
+        created_time: dict[str, Any] | TimestampRange | None = None,
+        last_updated_time: dict[str, Any] | TimestampRange | None = None,
+        data_set_ids: SequenceType[dict[str, Any]] | None = None,
+    ) -> None:
         self.name = name
         self.external_id_prefix = external_id_prefix
         self.metadata = metadata
@@ -136,17 +143,6 @@ class SequenceFilter(CogniteFilter):
         self.created_time = created_time
         self.last_updated_time = last_updated_time
         self.data_set_ids = data_set_ids
-        self._cognite_client = cast("CogniteClient", cognite_client)
-
-    @classmethod
-    def _load(cls, resource: Union[Dict, str]) -> SequenceFilter:
-        instance = super()._load(resource)
-        if isinstance(resource, Dict):
-            if instance.created_time is not None:
-                instance.created_time = TimestampRange(**instance.created_time)
-            if instance.last_updated_time is not None:
-                instance.last_updated_time = TimestampRange(**instance.last_updated_time)
-        return instance
 
 
 class SequenceColumnUpdate(CogniteUpdate):
@@ -161,13 +157,13 @@ class SequenceColumnUpdate(CogniteUpdate):
             return self._set(value)
 
     class _ObjectSequenceColumnUpdate(CogniteObjectUpdate):
-        def set(self, value: Dict) -> SequenceColumnUpdate:
+        def set(self, value: dict) -> SequenceColumnUpdate:
             return self._set(value)
 
-        def add(self, value: Dict) -> SequenceColumnUpdate:
+        def add(self, value: dict) -> SequenceColumnUpdate:
             return self._add(value)
 
-        def remove(self, value: List) -> SequenceColumnUpdate:
+        def remove(self, value: list) -> SequenceColumnUpdate:
             return self._remove(value)
 
     @property
@@ -186,6 +182,15 @@ class SequenceColumnUpdate(CogniteUpdate):
     def metadata(self) -> _ObjectSequenceColumnUpdate:
         return SequenceColumnUpdate._ObjectSequenceColumnUpdate(self, "metadata")
 
+    @classmethod
+    def _get_update_properties(cls) -> list[PropertySpec]:
+        return [
+            PropertySpec("description"),
+            PropertySpec("external_id", is_nullable=False),
+            PropertySpec("name"),
+            PropertySpec("metadata", is_container=True),
+        ]
+
 
 class SequenceUpdate(CogniteUpdate):
     """No description.
@@ -200,34 +205,34 @@ class SequenceUpdate(CogniteUpdate):
             return self._set(value)
 
     class _ObjectSequenceUpdate(CogniteObjectUpdate):
-        def set(self, value: Dict) -> SequenceUpdate:
+        def set(self, value: dict) -> SequenceUpdate:
             return self._set(value)
 
-        def add(self, value: Dict) -> SequenceUpdate:
+        def add(self, value: dict) -> SequenceUpdate:
             return self._add(value)
 
-        def remove(self, value: List) -> SequenceUpdate:
+        def remove(self, value: list) -> SequenceUpdate:
             return self._remove(value)
 
     class _ListSequenceUpdate(CogniteListUpdate):
-        def set(self, value: List) -> SequenceUpdate:
+        def set(self, value: list) -> SequenceUpdate:
             return self._set(value)
 
-        def add(self, value: List) -> SequenceUpdate:
+        def add(self, value: list) -> SequenceUpdate:
             return self._add(value)
 
-        def remove(self, value: List) -> SequenceUpdate:
+        def remove(self, value: list) -> SequenceUpdate:
             return self._remove(value)
 
     class _LabelSequenceUpdate(CogniteLabelUpdate):
-        def add(self, value: List) -> SequenceUpdate:
+        def add(self, value: list) -> SequenceUpdate:
             return self._add(value)
 
-        def remove(self, value: List) -> SequenceUpdate:
+        def remove(self, value: list) -> SequenceUpdate:
             return self._remove(value)
 
     class _ColumnsSequenceUpdate(CogniteListUpdate):
-        def add(self, value: Union[Dict, List[Dict]]) -> SequenceUpdate:
+        def add(self, value: dict | list[dict]) -> SequenceUpdate:
             single_item = not isinstance(value, list)
             if single_item:
                 value_list = cast(List[str], [value])
@@ -236,7 +241,7 @@ class SequenceUpdate(CogniteUpdate):
 
             return self._add(value_list)
 
-        def remove(self, value: Union[str, List[str]]) -> SequenceUpdate:
+        def remove(self, value: str | list[str]) -> SequenceUpdate:
             single_item = not isinstance(value, list)
             if single_item:
                 value_list = cast(List[str], [value])
@@ -245,7 +250,7 @@ class SequenceUpdate(CogniteUpdate):
 
             return self._remove([{"externalId": id} for id in value_list])
 
-        def modify(self, value: List[SequenceColumnUpdate]) -> SequenceUpdate:
+        def modify(self, value: list[SequenceColumnUpdate]) -> SequenceUpdate:
             return self._modify([col.dump() for col in value])
 
     @property
@@ -276,22 +281,37 @@ class SequenceUpdate(CogniteUpdate):
     def columns(self) -> _ColumnsSequenceUpdate:
         return SequenceUpdate._ColumnsSequenceUpdate(self, "columns")
 
+    @classmethod
+    def _get_update_properties(cls) -> list[PropertySpec]:
+        return [
+            # External ID is nullable, but is used in the upsert logic and thus cannot be nulled out.
+            PropertySpec("external_id", is_nullable=False),
+            PropertySpec("name"),
+            PropertySpec("description"),
+            PropertySpec("asset_id"),
+            # Sequences do not support setting metadata to an empty array.
+            PropertySpec("metadata", is_container=True, is_nullable=False),
+            PropertySpec("data_set_id"),
+            # PropertySpec("columns", is_list=True),
+        ]
+
 
 class SequenceAggregate(dict):
     """No description.
 
     Args:
-        count (int): No description.
+        count (int | None): No description.
+        **kwargs (Any): No description.
     """
 
-    def __init__(self, count: int = None, **kwargs: Any):
+    def __init__(self, count: int | None = None, **kwargs: Any) -> None:
         self.count = count
         self.update(kwargs)
 
     count = CognitePropertyClassUtil.declare_property("count")
 
 
-class SequenceList(CogniteResourceList):
+class SequenceList(CogniteResourceList[Sequence], IdTransformerMixin):
     _RESOURCE = Sequence
 
 
@@ -299,23 +319,23 @@ class SequenceData(CogniteResource):
     """An object representing a list of rows from a sequence.
 
     Args:
-        id (int): Id of the sequence the data belong to
-        external_id (str): External id of the sequence the data belong to
-        rows (SequenceType[dict]): Combined row numbers and row data object from the API. If you pass this, row_numbers/values are ignored.
-        row_numbers (SequenceType[int]): The data row numbers.
-        values (SequenceType[SequenceType[ Union[int, str, float]]]): The data values, one row at a time.
-        columns: SequenceType[dict]: The column information, in the format returned by the API.
+        id (int | None): Id of the sequence the data belong to
+        external_id (str | None): External id of the sequence the data belong to
+        rows (SequenceType[dict] | None): Combined row numbers and row data object from the API. If you pass this, row_numbers/values are ignored.
+        row_numbers (SequenceType[int] | None): The data row numbers.
+        values (SequenceType[SequenceType[int | str | float]] | None): The data values, one row at a time.
+        columns (SequenceType[dict[str, Any]] | None): SequenceType[dict]: The column information, in the format returned by the API.
     """
 
     def __init__(
         self,
-        id: int = None,
-        external_id: str = None,
-        rows: SequenceType[dict] = None,
-        row_numbers: SequenceType[int] = None,
-        values: SequenceType[SequenceType[Union[int, str, float]]] = None,
-        columns: SequenceType[Dict[str, Any]] = None,
-    ):
+        id: int | None = None,
+        external_id: str | None = None,
+        rows: SequenceType[dict] | None = None,
+        row_numbers: SequenceType[int] | None = None,
+        values: SequenceType[SequenceType[int | str | float]] | None = None,
+        columns: SequenceType[dict[str, Any]] | None = None,
+    ) -> None:
         if rows:
             row_numbers = [r["rowNumber"] for r in rows]
             values = [r["values"] for r in rows]
@@ -333,27 +353,27 @@ class SequenceData(CogniteResource):
 
     def __eq__(self, other: Any) -> bool:
         return (
-            type(self) == type(other)
+            type(self) is type(other)
             and self.id == other.id
             and self.external_id == other.external_id
             and self.row_numbers == other.row_numbers
             and self.values == other.values
         )
 
-    def __getitem__(self, item: int) -> SequenceType[Union[int, str, float]]:
+    def __getitem__(self, item: int) -> SequenceType[int | str | float]:
         # slow, should be replaced by dict cache if it sees more than incidental use
         if isinstance(item, slice):
             raise TypeError("Slicing SequenceData not supported")
         return self.values[self.row_numbers.index(item)]
 
-    def get_column(self, external_id: str) -> List[Union[int, str, float]]:
+    def get_column(self, external_id: str) -> list[int | str | float]:
         """Get a column by external_id.
 
         Args:
             external_id (str): External id of the column.
 
         Returns:
-            List[Union[int, str, float]]: A list of values for that column in the sequence
+            list[int | str | float]: A list of values for that column in the sequence
         """
         try:
             ix = self.column_external_ids.index(external_id)
@@ -363,19 +383,19 @@ class SequenceData(CogniteResource):
             )
         return [r[ix] for r in self.values]
 
-    def items(self) -> Generator[Tuple[int, List[Union[int, str, float]]], None, None]:
+    def items(self) -> Iterator[tuple[int, list[int | str | float]]]:
         """Returns an iterator over tuples of (row number, values)."""
         for row, values in zip(self.row_numbers, self.values):
             yield row, list(values)
 
-    def dump(self, camel_case: bool = False) -> Dict[str, Any]:
+    def dump(self, camel_case: bool = False) -> dict[str, Any]:
         """Dump the sequence data into a json serializable Python data type.
 
         Args:
             camel_case (bool): Use camelCase for attribute names. Defaults to False.
 
         Returns:
-            Dict[str, Any]: A dictionary representing the instance.
+            dict[str, Any]: A dictionary representing the instance.
         """
         dumped = {
             "id": self.id,
@@ -412,34 +432,34 @@ class SequenceData(CogniteResource):
             for eid in self.column_external_ids
         ]
         # TODO: Optimization required (None/nan):
-        return pd.DataFrame(  # type: ignore
+        return pd.DataFrame(
             [[x if x is not None else math.nan for x in r] for r in self.values],
             index=self.row_numbers,
             columns=df_columns,
         )
 
     @property
-    def column_external_ids(self) -> List[str]:
+    def column_external_ids(self) -> list[str]:
         """Retrieves list of column external ids for the sequence, for use in e.g. data retrieve or insert methods.
 
         Returns:
-            List of sequence column external ids.
+            list[str]: List of sequence column external ids.
         """
         assert self.columns is not None
         return [cast(str, c.get("externalId")) for c in self.columns]
 
     @property
-    def column_value_types(self) -> List[str]:
+    def column_value_types(self) -> list[str]:
         """Retrieves list of column value types.
 
         Returns:
-            List of column value types
+            list[str]: List of column value types
         """
         assert self.columns is not None
         return [cast(str, c.get("valueType")) for c in self.columns]
 
 
-class SequenceDataList(CogniteResourceList):
+class SequenceDataList(CogniteResourceList[SequenceData]):
     _RESOURCE = SequenceData
 
     def __str__(self) -> str:
@@ -450,10 +470,54 @@ class SequenceDataList(CogniteResourceList):
 
         Args:
             column_names (str):  Which field to use as column header. Can use any combination of "externalId", "columnExternalId", "id" and other characters as a template.
-            include_aggregate_name (bool): Include aggregate in the column name
 
         Returns:
             pandas.DataFrame: The sequence data list as a pandas DataFrame.
         """
         pd = utils._auxiliary.local_import("pandas")
-        return pd.concat([seq_data.to_pandas(column_names=column_names) for seq_data in self.data], axis=1)  # type: ignore
+        return pd.concat([seq_data.to_pandas(column_names=column_names) for seq_data in self.data], axis=1)
+
+
+class SequenceProperty(EnumProperty):
+    description = "description"
+    external_id = "externalId"
+    name = "name"
+    asset_id = "assetId"
+    asset_root_id = "assetRootId"
+    created_time = "createdTime"
+    data_set_id = "dataSetId"
+    id = "id"
+    last_updated_time = "lastUpdatedTime"
+    access_categories = "accessCategories"
+    metadata = "metadata"
+
+    @staticmethod
+    def metadata_key(key: str) -> list[str]:
+        return ["metadata", key]
+
+
+class SortableSequenceProperty(EnumProperty):
+    asset_id = "assetId"
+    created_time = "createdTime"
+    data_set_id = "dataSetId"
+    description = "description"
+    external_id = "externalId"
+    last_updated_time = "lastUpdatedTime"
+    name = "name"
+
+    @staticmethod
+    def metadata_key(key: str) -> list[str]:
+        return ["metadata", key]
+
+
+SortableSequencePropertyLike: TypeAlias = Union[SortableSequenceProperty, str, List[str]]
+
+
+class SequenceSort(CogniteSort):
+    def __init__(
+        self,
+        property: SortableSequenceProperty,
+        order: Literal["asc", "desc"] = "asc",
+        nulls: Literal["auto", "first", "last"] = "auto",
+    ):
+        super().__init__(property, order, nulls)
