@@ -1,8 +1,9 @@
 from __future__ import annotations
 
-import warnings
-from typing import Any, Dict, Optional, Union
+from typing import Any
 
+from cognite.client.credentials import OAuthClientCredentials
+from cognite.client.data_classes.iam import ClientCredentials
 from cognite.client.utils._auxiliary import basic_obj_dump
 from cognite.client.utils._text import convert_all_keys_to_snake_case, iterable_to_case
 
@@ -11,10 +12,10 @@ class TransformationDestination:
     """TransformationDestination has static methods to define the target resource type of a transformation
 
     Args:
-        type (str): Used as data type identifier on transformation creation/retrieval.
+        type (str | None): Used as data type identifier on transformation creation/retrieval.
     """
 
-    def __init__(self, type: str = None):
+    def __init__(self, type: str | None = None) -> None:
         self.type = type
 
     def __hash__(self) -> int:
@@ -23,10 +24,10 @@ class TransformationDestination:
     def __eq__(self, other: Any) -> bool:
         return isinstance(other, type(self)) and hash(other) == hash(self)
 
-    def dump(self, camel_case: bool = False) -> Dict[str, Any]:
+    def dump(self, camel_case: bool = False) -> dict[str, Any]:
         ret = basic_obj_dump(self, camel_case)
 
-        needs_dump = set(iterable_to_case(("view", "edge_type"), camel_case))
+        needs_dump = set(iterable_to_case(("view", "edge_type", "data_model"), camel_case))
         for k in needs_dump.intersection(ret):
             if ret[k] is not None:
                 ret[k] = ret[k].dump(camel_case=camel_case)
@@ -96,7 +97,7 @@ class TransformationDestination:
             table (str): name of the target raw table
 
         Returns:
-            TransformationDestination pointing to the target table
+            RawTable: TransformationDestination pointing to the target table
         """
         return RawTable(database=database, table=table)
 
@@ -108,64 +109,53 @@ class TransformationDestination:
             external_id (str): Sequence external id.
 
         Returns:
-            TransformationDestination pointing to the target sequence rows
+            SequenceRows: TransformationDestination pointing to the target sequence rows
         """
         return SequenceRows(external_id=external_id)
 
     @staticmethod
-    def data_model_instances(
-        model_external_id: str = "", space_external_id: str = "", instance_space_external_id: str = ""
-    ) -> DataModelInstances:
-        """To be used when the transformation is meant to produce data model instances.
-            Flexible Data Models resource type is on `beta` version currently.
+    def nodes(view: ViewInfo | None = None, instance_space: str | None = None) -> Nodes:
+        """
 
         Args:
-            model_external_id (str): external_id of the flexible data model.
-            space_external_id (str): space external_id of the flexible data model.
-            instance_space_external_id (str): space external_id of the flexible data model instance.
+            view (ViewInfo | None): information of the view.
+            instance_space (str | None): space id of the instance.
         Returns:
-            TransformationDestination pointing to the target flexible data model.
+            Nodes: pointing to the target flexible data model.
         """
-        return DataModelInstances(
-            model_external_id=model_external_id,
-            space_external_id=space_external_id,
-            instance_space_external_id=instance_space_external_id,
-        )
+        return Nodes(view=view, instance_space=instance_space)
 
     @staticmethod
-    def instance_nodes(view: Optional[ViewInfo] = None, instance_space: Optional[str] = None) -> InstanceNodes:
-        """To be used when the transformation is meant to produce node's instances.
-            Flexible Data Models resource type is on `beta` version currently.
+    def edges(
+        view: ViewInfo | None = None,
+        instance_space: str | None = None,
+        edge_type: EdgeType | None = None,
+    ) -> Edges:
+        """
 
         Args:
-            view (ViewInfo): information of the view.
-            instance_space (str): space id of the instance.
+            view (ViewInfo | None): information of the view.
+            instance_space (str | None): space id of the instance.
+            edge_type (EdgeType | None): information about the type of the edge
         Returns:
-            InstanceNodes: pointing to the target flexible data model.
+            Edges: pointing to the target flexible data model.
         """
-        return InstanceNodes(view=view, instance_space=instance_space)
+        return Edges(view=view, instance_space=instance_space, edge_type=edge_type)
 
     @staticmethod
-    def instance_edges(
-        view: Optional[ViewInfo] = None,
-        instance_space: Optional[str] = None,
-        edge_type: Optional[EdgeType] = None,
-    ) -> InstanceEdges:
-        """To be used when the transformation is meant to produce edge's instances.
-            Flexible Data Models resource type is on `beta` version currently.
-
-        Args:
-            view (ViewInfo): information of the view.
-            instance_space (str): space id of the instance.
-            edge_type (EdgeType): information about the type of the edge
-        Returns:
-            InstanceEdges: pointing to the target flexible data model.
+    def instances(data_model: DataModelInfo | None = None, instance_space: str | None = None) -> Instances:
         """
-        return InstanceEdges(view=view, instance_space=instance_space, edge_type=edge_type)
+        Args:
+            data_model (DataModelInfo | None): information of the Data Model.
+            instance_space (str | None): space id of the instance.
+        Returns:
+            Instances: pointing to the target centric data model.
+        """
+        return Instances(data_model=data_model, instance_space=instance_space)
 
 
 class RawTable(TransformationDestination):
-    def __init__(self, database: str = None, table: str = None):
+    def __init__(self, database: str | None = None, table: str | None = None) -> None:
         super().__init__(type="raw")
         self.database = database
         self.table = table
@@ -175,7 +165,7 @@ class RawTable(TransformationDestination):
 
 
 class SequenceRows(TransformationDestination):
-    def __init__(self, external_id: str = None):
+    def __init__(self, external_id: str | None = None) -> None:
         super().__init__(type="sequence_rows")
         self.external_id = external_id
 
@@ -183,26 +173,8 @@ class SequenceRows(TransformationDestination):
         return hash((self.type, self.external_id))
 
 
-class DataModelInstances(TransformationDestination):
-    def __init__(
-        self, model_external_id: str = None, space_external_id: str = None, instance_space_external_id: str = None
-    ):
-        warnings.warn(
-            "Feature DataModelStorage is in beta and still in development. "
-            "Breaking changes can happen in between patch versions.",
-            stacklevel=2,
-        )
-        super().__init__(type="data_model_instances")
-        self.model_external_id = model_external_id
-        self.space_external_id = space_external_id
-        self.instance_space_external_id = instance_space_external_id
-
-    def __hash__(self) -> int:
-        return hash((self.type, self.model_external_id, self.space_external_id, self.instance_space_external_id))
-
-
 class ViewInfo:
-    def __init__(self, space: str, external_id: str, version: str):
+    def __init__(self, space: str, external_id: str, version: str) -> None:
         self.space = space
         self.external_id = external_id
         self.version = version
@@ -210,64 +182,73 @@ class ViewInfo:
     def __hash__(self) -> int:
         return hash((self.space, self.external_id, self.version))
 
-    def dump(self, camel_case: bool = False) -> Dict[str, Any]:
+    def dump(self, camel_case: bool = False) -> dict[str, Any]:
         return basic_obj_dump(self, camel_case)
 
 
 class EdgeType:
-    def __init__(self, space: str, external_id: str):
+    def __init__(self, space: str, external_id: str) -> None:
         self.space = space
         self.external_id = external_id
 
     def __hash__(self) -> int:
         return hash((self.space, self.external_id))
 
-    def dump(self, camel_case: bool = False) -> Dict[str, Any]:
+    def dump(self, camel_case: bool = False) -> dict[str, Any]:
         return basic_obj_dump(self, camel_case)
 
 
-class InstanceNodes(TransformationDestination):
+class DataModelInfo:
     def __init__(
         self,
-        view: Optional[ViewInfo] = None,
-        instance_space: Optional[str] = None,
-    ):
-        warnings.warn(
-            "Feature DataModelStorage is in beta and still in development. "
-            "Breaking changes can happen in between patch versions.",
-            stacklevel=2,
-        )
+        space: str,
+        external_id: str,
+        version: str,
+        destination_type: str,
+        destination_relationship_from_type: str | None = None,
+    ) -> None:
+        self.space = space
+        self.external_id = external_id
+        self.version = version
+        self.destination_type = destination_type
+        self.destination_relationship_from_type = destination_relationship_from_type
+
+    def dump(self, camel_case: bool = False) -> dict[str, Any]:
+        return basic_obj_dump(self, camel_case)
+
+
+class Nodes(TransformationDestination):
+    def __init__(
+        self,
+        view: ViewInfo | None = None,
+        instance_space: str | None = None,
+    ) -> None:
         super().__init__(type="nodes")
         self.view = view
         self.instance_space = instance_space
 
     @classmethod
-    def _load(cls, resource: Dict[str, Any]) -> InstanceNodes:
+    def _load(cls, resource: dict[str, Any]) -> Nodes:
         inst = cls(**resource)
         if isinstance(inst.view, dict):
             inst.view = ViewInfo(**convert_all_keys_to_snake_case(inst.view))
         return inst
 
 
-class InstanceEdges(TransformationDestination):
+class Edges(TransformationDestination):
     def __init__(
         self,
-        view: Optional[ViewInfo] = None,
-        instance_space: Optional[str] = None,
-        edge_type: Optional[EdgeType] = None,
-    ):
-        warnings.warn(
-            "Feature DataModelStorage is in beta and still in development. "
-            "Breaking changes can happen in between patch versions.",
-            stacklevel=2,
-        )
+        view: ViewInfo | None = None,
+        instance_space: str | None = None,
+        edge_type: EdgeType | None = None,
+    ) -> None:
         super().__init__(type="edges")
         self.view = view
         self.instance_space = instance_space
         self.edge_type = edge_type
 
     @classmethod
-    def _load(cls, resource: Dict[str, Any]) -> InstanceEdges:
+    def _load(cls, resource: dict[str, Any]) -> Edges:
         inst = cls(**resource)
         if isinstance(inst.view, dict):
             inst.view = ViewInfo(**convert_all_keys_to_snake_case(inst.view))
@@ -276,17 +257,34 @@ class InstanceEdges(TransformationDestination):
         return inst
 
 
+class Instances(TransformationDestination):
+    def __init__(
+        self,
+        data_model: DataModelInfo | None = None,
+        instance_space: str | None = None,
+    ) -> None:
+        super().__init__(type="instances")
+        self.data_model = data_model
+        self.instance_space = instance_space
+
+    @classmethod
+    def _load(cls, resource: dict[str, Any]) -> Instances:
+        inst = cls(**resource)
+        if isinstance(inst.data_model, dict):
+            inst.data_model = DataModelInfo(**convert_all_keys_to_snake_case(inst.data_model))
+        return inst
+
+
 class OidcCredentials:
     def __init__(
         self,
-        client_id: str = None,
-        client_secret: str = None,
-        scopes: str = None,
-        token_uri: str = None,
-        audience: str = None,
-        cdf_project_name: str = None,
-    ):
-
+        client_id: str,
+        client_secret: str,
+        scopes: str,
+        token_uri: str,
+        audience: str | None = None,
+        cdf_project_name: str | None = None,
+    ) -> None:
         self.client_id = client_id
         self.client_secret = client_secret
         self.scopes = scopes
@@ -294,14 +292,26 @@ class OidcCredentials:
         self.audience = audience
         self.cdf_project_name = cdf_project_name
 
-    def dump(self, camel_case: bool = False) -> Dict[str, Any]:
+    def as_credential_provider(self) -> OAuthClientCredentials:
+        return OAuthClientCredentials(
+            token_url=self.token_uri,
+            client_id=self.client_id,
+            client_secret=self.client_secret,
+            scopes=self.scopes.split(","),
+            audience=self.audience,
+        )
+
+    def as_client_credentials(self) -> ClientCredentials:
+        return ClientCredentials(client_id=self.client_id, client_secret=self.client_secret)
+
+    def dump(self, camel_case: bool = False) -> dict[str, Any]:
         """Dump the instance into a json serializable Python data type.
 
         Args:
             camel_case (bool): Use camelCase for attribute names. Defaults to False.
 
         Returns:
-            Dict[str, Any]: A dictionary representation of the instance.
+            dict[str, Any]: A dictionary representation of the instance.
         """
         return basic_obj_dump(self, camel_case)
 
@@ -312,19 +322,19 @@ class NonceCredentials:
         session_id: int,
         nonce: str,
         cdf_project_name: str,
-    ):
+    ) -> None:
         self.session_id = session_id
         self.nonce = nonce
         self.cdf_project_name = cdf_project_name
 
-    def dump(self, camel_case: bool = False) -> Dict[str, Any]:
+    def dump(self, camel_case: bool = False) -> dict[str, Any]:
         """Dump the instance into a json serializable Python data type.
 
         Args:
             camel_case (bool): Use camelCase for attribute names. Defaults to False.
 
         Returns:
-            Dict[str, Any]: A dictionary representation of the instance.
+            dict[str, Any]: A dictionary representation of the instance.
         """
         return basic_obj_dump(self, camel_case)
 
@@ -334,31 +344,33 @@ class TransformationBlockedInfo:
 
     Args:
         reason (str): Reason why the transformation is blocked.
-        created_time (Optional[int]): Timestamp when the transformation was blocked.
+        created_time (int): Timestamp when the transformation was blocked.
     """
 
-    def __init__(self, reason: str = None, created_time: Optional[int] = None):
+    def __init__(self, reason: str, created_time: int) -> None:
         self.reason = reason
         self.created_time = created_time
 
+    @classmethod
+    def _load(cls, resource: dict[str, Any]) -> TransformationBlockedInfo:
+        return cls(reason=resource["reason"], created_time=resource["createdTime"])
 
-def _load_destination_dct(
-    dct: Dict[str, Any]
-) -> Union[RawTable, DataModelInstances, InstanceNodes, InstanceEdges, SequenceRows, TransformationDestination]:
+
+def _load_destination_dct(dct: dict[str, Any]) -> RawTable | Nodes | Edges | SequenceRows | TransformationDestination:
     """Helper function to load destination from dictionary"""
     snake_dict = convert_all_keys_to_snake_case(dct)
     destination_type = snake_dict.pop("type")
     simple = {
         "raw": RawTable,
-        "data_model_instances": DataModelInstances,
         "sequence_rows": SequenceRows,
     }
     if destination_type in simple:
         return simple[destination_type](**snake_dict)
 
-    nested: Dict[str, Union[type[InstanceNodes], type[InstanceEdges]]] = {
-        "nodes": InstanceNodes,
-        "edges": InstanceEdges,
+    nested: dict[str, type[Nodes] | type[Edges] | type[Instances]] = {
+        "nodes": Nodes,
+        "edges": Edges,
+        "instances": Instances,
     }
     if destination_type in nested:
         return nested[destination_type]._load(snake_dict)
