@@ -3,7 +3,7 @@ from __future__ import annotations
 import asyncio
 import time
 from enum import Enum
-from typing import TYPE_CHECKING, Dict, Optional, Union, cast
+from typing import TYPE_CHECKING, Dict, cast
 
 from cognite.client.data_classes._base import CogniteFilter, CogniteResource, CogniteResourceList
 from cognite.client.data_classes.transformations.common import TransformationDestination, _load_destination_dct
@@ -23,32 +23,33 @@ class TransformationJobMetric(CogniteResource):
     """The transformation job metric resource allows following details of execution of a transformation run.
 
     Args:
-        timestamp (int): Time of the last metric update.
-        name (str): Name of the metric.
-        count (int): Value of the metric.
-        cognite_client (CogniteClient): The client to associate with this object.
+        id (int | None): No description.
+        timestamp (int | None): Time of the last metric update.
+        name (str | None): Name of the metric.
+        count (int | None): Value of the metric.
+        cognite_client (CogniteClient | None): The client to associate with this object.
     """
 
     def __init__(
         self,
-        id: int = None,
-        timestamp: int = None,
-        name: str = None,
-        count: int = None,
-        cognite_client: CogniteClient = None,
-    ):
+        id: int | None = None,
+        timestamp: int | None = None,
+        name: str | None = None,
+        count: int | None = None,
+        cognite_client: CogniteClient | None = None,
+    ) -> None:
         self.timestamp = timestamp
         self.name = name
         self.count = count
         self._cognite_client = cast("CogniteClient", cognite_client)
 
     @classmethod
-    def _load(cls, resource: Union[Dict, str], cognite_client: CogniteClient = None) -> TransformationJobMetric:
+    def _load(cls, resource: dict | str, cognite_client: CogniteClient | None = None) -> TransformationJobMetric:
         instance = super()._load(resource, cognite_client)
         return instance
 
 
-class TransformationJobMetricList(CogniteResourceList):
+class TransformationJobMetricList(CogniteResourceList[TransformationJobMetric]):
     _RESOURCE = TransformationJobMetric
 
 
@@ -56,45 +57,43 @@ class TransformationJob(CogniteResource):
     """The transformation job resource allows following the status of execution of a transformation run.
 
     Args:
-        id (int): A server-generated ID for the object.
-        status (TransformationJobStatus): Status of the job.
-        transformation_id (int): Server-generated ID of the transformation.
-        transformation_external_id (str): external ID of the transformation.
-        source_project (str): Name of the CDF project the data will be read from.
-        destination_project (str): Name of the CDF project the data will be written to.
-        destination_type (str): Target resource type of the transformation.
-        destination_database (str): Target database if the destination type is raw.
-        destination_table (str): Target table name if the destination type is RAW.
-        conflict_mode (str): What to do in case of id collisions: either "abort", "upsert", "update" or "delete".
-        query (str): Query of the transformation that is being executed.
-        error (str): Error message from the server.
+        id (int | None): A server-generated ID for the object.
+        status (TransformationJobStatus | None): Status of the job.
+        transformation_id (int | None): Server-generated ID of the transformation.
+        transformation_external_id (str | None): external ID of the transformation.
+        source_project (str | None): Name of the CDF project the data will be read from.
+        destination_project (str | None): Name of the CDF project the data will be written to.
+        destination (TransformationDestination | None): No description.
+        conflict_mode (str | None): What to do in case of id collisions: either "abort", "upsert", "update" or "delete".
+        query (str | None): Query of the transformation that is being executed.
+        error (str | None): Error message from the server.
         ignore_null_fields (bool): Indicates how null values are handled on updates: ignore or set null.
-        created_time (int): Time when the job was created.
-        started_time (int): Time when the job started running.
-        finished_time (int): Time when the job finished running.
-        last_seen_time (int): Time of the last status update from the job.
-        cognite_client (CogniteClient): The client to associate with this object.
+        created_time (int | None): Time when the job was created.
+        started_time (int | None): Time when the job started running.
+        finished_time (int | None): Time when the job finished running.
+        last_seen_time (int | None): Time of the last status update from the job.
+        cognite_client (CogniteClient | None): The client to associate with this object.
     """
 
     def __init__(
         self,
-        id: int = None,
-        status: TransformationJobStatus = None,
-        transformation_id: int = None,
-        transformation_external_id: str = None,
-        source_project: str = None,
-        destination_project: str = None,
-        destination: TransformationDestination = None,
-        conflict_mode: str = None,
-        query: str = None,
-        error: str = None,
+        id: int | None = None,
+        status: TransformationJobStatus | None = None,
+        transformation_id: int | None = None,
+        transformation_external_id: str | None = None,
+        source_project: str | None = None,
+        destination_project: str | None = None,
+        destination: TransformationDestination | None = None,
+        conflict_mode: str | None = None,
+        query: str | None = None,
+        error: str | None = None,
         ignore_null_fields: bool = False,
-        created_time: int = None,
-        started_time: int = None,
-        finished_time: int = None,
-        last_seen_time: int = None,
-        cognite_client: CogniteClient = None,
-    ):
+        created_time: int | None = None,
+        started_time: int | None = None,
+        finished_time: int | None = None,
+        last_seen_time: int | None = None,
+        cognite_client: CogniteClient | None = None,
+    ) -> None:
         self.id = id
         self.status = status
         self.transformation_id = transformation_id
@@ -114,7 +113,13 @@ class TransformationJob(CogniteResource):
 
     def update(self) -> None:
         """`Get updated job status.`"""
+        if self.id is None:
+            raise ValueError("Unable to update, TransformationJob is missing 'id'")
+
         updated = self._cognite_client.transformations.jobs.retrieve(id=self.id)
+        if updated is None:
+            raise RuntimeError("Unable to update the transformation. Has it been deleted?")
+
         self.status = updated.status
         self.error = updated.error
         self.started_time = updated.started_time
@@ -132,12 +137,12 @@ class TransformationJob(CogniteResource):
         assert self.id is not None
         return self._cognite_client.transformations.jobs.list_metrics(self.id)
 
-    def wait(self, polling_interval: float = 1, timeout: Optional[float] = None) -> TransformationJob:
+    def wait(self, polling_interval: float = 1, timeout: float | None = None) -> TransformationJob:
         """`Waits for the job to finish.`
 
         Args:
             polling_interval (float): time (s) to wait between job status updates, default is one second.
-            timeout (Optional[float]): maximum time (s) to wait, default is None (infinite time). Once the timeout is reached, it returns with the current status.
+            timeout (float | None): maximum time (s) to wait, default is None (infinite time). Once the timeout is reached, it returns with the current status.
 
         Returns:
             TransformationJob: self.
@@ -184,15 +189,15 @@ class TransformationJob(CogniteResource):
 
         return self
 
-    async def wait_async(self, polling_interval: float = 1, timeout: Optional[float] = None) -> TransformationJob:
+    async def wait_async(self, polling_interval: float = 1, timeout: float | None = None) -> TransformationJob:
         """Asyncio coroutine, waits for the job to finish asynchronously.
 
         Args:
             polling_interval (float): time (s) to wait between job status updates, default is one second.
-            timeout (Optional[float]): maximum time (s) to wait, default is None (infinite time). Once the timeout is reached, it returns with the current status.
+            timeout (float | None): maximum time (s) to wait, default is None (infinite time). Once the timeout is reached, it returns with the current status.
 
         Returns:
-            Awaitable[TransformationJob]: coroutine object that will finish when the job finishes and resolves to self.
+            TransformationJob: coroutine object that will finish when the job finishes and resolves to self.
 
         Examples:
 
@@ -246,7 +251,7 @@ class TransformationJob(CogniteResource):
         return self
 
     @classmethod
-    def _load(cls, resource: Union[Dict, str], cognite_client: CogniteClient = None) -> TransformationJob:
+    def _load(cls, resource: dict | str, cognite_client: CogniteClient | None = None) -> TransformationJob:
         instance = super()._load(resource, cognite_client)
         if isinstance(instance.destination, Dict):
             instance.destination = _load_destination_dct(instance.destination)
@@ -256,18 +261,18 @@ class TransformationJob(CogniteResource):
         return hash(self.id)
 
 
-class TransformationJobList(CogniteResourceList):
+class TransformationJobList(CogniteResourceList[TransformationJob]):
     _RESOURCE = TransformationJob
 
 
 class TransformationJobFilter(CogniteFilter):
-    """
+    """TransformationJobFilter
 
     Args:
-        transformation_id (Optional[int]):  Filter jobs by transformation internal numeric ID.
-        transformation_external_id (str): Filter jobs by transformation external ID.
+        transformation_id (int | None):  Filter jobs by transformation internal numeric ID.
+        transformation_external_id (str | None): Filter jobs by transformation external ID.
     """
 
-    def __init__(self, transformation_id: Optional[int] = None, transformation_external_id: str = None):
+    def __init__(self, transformation_id: int | None = None, transformation_external_id: str | None = None) -> None:
         self.transformation_id = transformation_id
         self.transformation_external_id = transformation_external_id
