@@ -7,9 +7,18 @@ from unittest.mock import MagicMock
 from cognite.client import CogniteClient
 from cognite.client._api.annotations import AnnotationsAPI
 from cognite.client._api.assets import AssetsAPI
+from cognite.client._api.data_modeling import DataModelingAPI
+from cognite.client._api.data_modeling.containers import ContainersAPI
+from cognite.client._api.data_modeling.data_models import DataModelsAPI
+from cognite.client._api.data_modeling.graphql import DataModelingGraphQLAPI
+from cognite.client._api.data_modeling.instances import InstancesAPI
+from cognite.client._api.data_modeling.spaces import SpacesAPI
+from cognite.client._api.data_modeling.views import ViewsAPI
 from cognite.client._api.data_sets import DataSetsAPI
 from cognite.client._api.datapoints import DatapointsAPI
+from cognite.client._api.datapoints_subscriptions import DatapointsSubscriptionAPI
 from cognite.client._api.diagrams import DiagramsAPI
+from cognite.client._api.documents import DocumentPreviewAPI, DocumentsAPI
 from cognite.client._api.entity_matching import EntityMatchingAPI
 from cognite.client._api.events import EventsAPI
 from cognite.client._api.extractionpipelines import (
@@ -48,7 +57,9 @@ from cognite.client._api.transformations import (
     TransformationSchedulesAPI,
     TransformationSchemaAPI,
 )
+from cognite.client._api.user_profiles import UserProfilesAPI
 from cognite.client._api.vision import VisionAPI
+from cognite.client._api.workflows import WorkflowAPI, WorkflowExecutionAPI, WorkflowTaskAPI, WorkflowVersionAPI
 
 
 class CogniteClientMock(MagicMock):
@@ -70,9 +81,20 @@ class CogniteClientMock(MagicMock):
         #   - Use `spec_set=MyNestedAPI` for all nested APIs
         self.annotations = MagicMock(spec_set=AnnotationsAPI)
         self.assets = MagicMock(spec_set=AssetsAPI)
+
+        self.data_modeling = MagicMock(spec=DataModelingAPI)
+        self.data_modeling.containers = MagicMock(spec_set=ContainersAPI)
+        self.data_modeling.data_models = MagicMock(spec_set=DataModelsAPI)
+        self.data_modeling.spaces = MagicMock(spec_set=SpacesAPI)
+        self.data_modeling.views = MagicMock(spec_set=ViewsAPI)
+        self.data_modeling.instances = MagicMock(spec_set=InstancesAPI)
+        self.data_modeling.graphql = MagicMock(spec_set=DataModelingGraphQLAPI)
+
         self.data_sets = MagicMock(spec_set=DataSetsAPI)
 
         self.diagrams = MagicMock(spec_set=DiagramsAPI)
+        self.documents = MagicMock(spec=DocumentsAPI)
+        self.documents.previews = MagicMock(spec_set=DocumentPreviewAPI)
         self.entity_matching = MagicMock(spec_set=EntityMatchingAPI)
         self.events = MagicMock(spec_set=EventsAPI)
 
@@ -92,6 +114,7 @@ class CogniteClientMock(MagicMock):
         self.iam.groups = MagicMock(spec_set=GroupsAPI)
         self.iam.security_categories = MagicMock(spec_set=SecurityCategoriesAPI)
         self.iam.sessions = MagicMock(spec_set=SessionsAPI)
+        self.iam.user_profiles = MagicMock(spec_set=UserProfilesAPI)
         self.iam.token = MagicMock(spec_set=TokenAPI)
 
         self.labels = MagicMock(spec_set=LabelsAPI)
@@ -121,6 +144,7 @@ class CogniteClientMock(MagicMock):
         self.time_series = MagicMock(spec=TimeSeriesAPI)
         self.time_series.data = MagicMock(spec=DatapointsAPI)
         self.time_series.data.synthetic = MagicMock(spec_set=SyntheticDatapointsAPI)
+        self.time_series.subscriptions = MagicMock(spec_set=DatapointsSubscriptionAPI)
 
         self.transformations = MagicMock(spec=TransformationsAPI)
         self.transformations.jobs = MagicMock(spec_set=TransformationJobsAPI)
@@ -130,6 +154,11 @@ class CogniteClientMock(MagicMock):
 
         self.vision = MagicMock(spec_set=VisionAPI)
 
+        self.workflows = MagicMock(spec=WorkflowAPI)
+        self.workflows.versions = MagicMock(spec_set=WorkflowVersionAPI)
+        self.workflows.executions = MagicMock(spec_set=WorkflowExecutionAPI)
+        self.workflows.tasks = MagicMock(spec_set=WorkflowTaskAPI)
+
 
 @contextmanager
 def monkeypatch_cognite_client() -> Iterator[CogniteClientMock]:
@@ -182,6 +211,6 @@ def monkeypatch_cognite_client() -> Iterator[CogniteClientMock]:
             >>>         assert "Something went wrong" == e.message
     """
     cognite_client_mock = CogniteClientMock()
-    CogniteClient.__new__ = lambda *args, **kwargs: cognite_client_mock  # type: ignore[assignment]
+    CogniteClient.__new__ = lambda *args, **kwargs: cognite_client_mock  # type: ignore[method-assign]
     yield cognite_client_mock
-    CogniteClient.__new__ = lambda cls, *args, **kwargs: object.__new__(cls)  # type: ignore[assignment]
+    CogniteClient.__new__ = lambda cls, *args, **kwargs: object.__new__(cls)  # type: ignore[method-assign]
