@@ -6,19 +6,22 @@ import re
 import sys
 import time
 from abc import ABC, abstractmethod
+from contextlib import suppress
 from datetime import datetime, timedelta, timezone
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Union, cast, overload
+from typing import TYPE_CHECKING, Any, cast, overload
 
 from cognite.client.exceptions import CogniteImportError
 from cognite.client.utils._auxiliary import local_import
 
 if TYPE_CHECKING:
+    from datetime import tzinfo
+
     import pandas
 
     if sys.version_info >= (3, 9):
-        from zoneinfo import ZoneInfo
+        from zoneinfo import ZoneInfo, ZoneInfoNotFoundError
     else:
-        from backports.zoneinfo import ZoneInfo
+        from backports.zoneinfo import ZoneInfo, ZoneInfoNotFoundError
 
 
 UNIT_IN_MS_WITHOUT_WEEK = {"s": 1000, "m": 60000, "h": 3600000, "d": 86400000}
@@ -30,13 +33,13 @@ MIN_TIMESTAMP_MS = -2208988800000  # 1900-01-01 00:00:00.000
 MAX_TIMESTAMP_MS = 4102444799999  # 2099-12-31 23:59:59.999
 
 
-def import_zoneinfo() -> ZoneInfo:
+def import_zoneinfo() -> type[ZoneInfo]:
     try:
         if sys.version_info >= (3, 9):
             from zoneinfo import ZoneInfo
         else:
             from backports.zoneinfo import ZoneInfo
-        return ZoneInfo  # type: ignore [return-value]
+        return ZoneInfo
 
     except ImportError as e:
         raise CogniteImportError(
@@ -46,27 +49,42 @@ def import_zoneinfo() -> ZoneInfo:
         ) from e
 
 
+def _import_zoneinfo_not_found_error() -> type[ZoneInfoNotFoundError]:
+    if sys.version_info >= (3, 9):
+        from zoneinfo import ZoneInfoNotFoundError
+    else:
+        from backports.zoneinfo import ZoneInfoNotFoundError
+    return ZoneInfoNotFoundError
+
+
 def get_utc_zoneinfo() -> ZoneInfo:
-    return import_zoneinfo()("UTC")  # type: ignore [operator]
+    return import_zoneinfo()("UTC")
 
 
 def datetime_to_ms(dt: datetime) -> int:
-    """Converts datetime object to milliseconds since epoch.
+    """Converts a datetime object to milliseconds since epoch.
 
     Args:
         dt (datetime): Naive or aware datetime object. Naive datetimes are interpreted as local time.
 
     Returns:
-        ms: Milliseconds since epoch (negative for time prior to 1970-01-01)
+        int: Milliseconds since epoch (negative for time prior to 1970-01-01)
     """
-    return int(1000 * dt.timestamp())
+    try:
+        return int(1000 * dt.timestamp())
+    except OSError as e:
+        # OSError is raised if dt.timestamp() is called before 1970-01-01 on Windows for naive datetime.
+        raise ValueError(
+            "Failed to convert datetime to epoch. This likely because you are using a naive datetime. "
+            "Try using a timezone aware datetime instead."
+        ) from e
 
 
-def ms_to_datetime(ms: Union[int, float]) -> datetime:
+def ms_to_datetime(ms: int | float) -> datetime:
     """Converts valid Cognite timestamps, i.e. milliseconds since epoch, to datetime object.
 
     Args:
-        ms (Union[int, float]): Milliseconds since epoch.
+        ms (int | float): Milliseconds since epoch.
 
     Raises:
         ValueError: On invalid Cognite timestamps.
@@ -81,7 +99,7 @@ def ms_to_datetime(ms: Union[int, float]) -> datetime:
     return datetime(1970, 1, 1, tzinfo=timezone.utc) + timedelta(milliseconds=ms)
 
 
-def time_string_to_ms(pattern: str, string: str, unit_in_ms: Dict[str, int]) -> Optional[int]:
+def time_string_to_ms(pattern: str, string: str, unit_in_ms: dict[str, int]) -> int | None:
     pattern = pattern.format("|".join(unit_in_ms))
     if res := re.fullmatch(pattern, string):
         magnitude = int(res[1])
@@ -118,11 +136,11 @@ def time_ago_to_ms(time_ago_string: str) -> int:
     return ms
 
 
-def timestamp_to_ms(timestamp: Union[int, float, str, datetime]) -> int:
+def timestamp_to_ms(timestamp: int | float | str | datetime) -> int:
     """Returns the ms representation of some timestamp given by milliseconds, time-ago format or datetime object
 
     Args:
-        timestamp (Union[int, float, str, datetime]): Convert this timestamp to ms.
+        timestamp (int | float | str | datetime): Convert this timestamp to ms.
 
     Returns:
         int: Milliseconds since epoch representation of timestamp
@@ -156,7 +174,7 @@ TIME_ATTRIBUTES = {
 }
 
 
-def _convert_time_attributes_in_dict(item: Dict) -> Dict:
+def _convert_time_attributes_in_dict(item: dict) -> dict:
     new_item = {}
     for k, v in item.items():
         if k in TIME_ATTRIBUTES:
@@ -169,16 +187,16 @@ def _convert_time_attributes_in_dict(item: Dict) -> Dict:
 
 
 @overload
-def convert_time_attributes_to_datetime(item: Dict) -> Dict:
+def convert_time_attributes_to_datetime(item: dict) -> dict:
     ...
 
 
 @overload
-def convert_time_attributes_to_datetime(item: List[Dict]) -> List[Dict]:
+def convert_time_attributes_to_datetime(item: list[dict]) -> list[dict]:
     ...
 
 
-def convert_time_attributes_to_datetime(item: Union[Dict, List[Dict]]) -> Union[Dict, List[Dict]]:
+def convert_time_attributes_to_datetime(item: dict | list[dict]) -> dict | list[dict]:
     if isinstance(item, dict):
         return _convert_time_attributes_in_dict(item)
     if isinstance(item, list):
@@ -186,7 +204,7 @@ def convert_time_attributes_to_datetime(item: Union[Dict, List[Dict]]) -> Union[
     raise TypeError("item must be dict or list of dicts")
 
 
-def align_start_and_end_for_granularity(start: int, end: int, granularity: str) -> Tuple[int, int]:
+def align_start_and_end_for_granularity(start: int, end: int, granularity: str) -> tuple[int, int]:
     # Note the API always aligns `start` with 1s, 1m, 1h or 1d (even when given e.g. 73h)
     if remainder := start % granularity_unit_to_ms(granularity):
         # Floor `start` when not exactly at boundary
@@ -199,7 +217,9 @@ def align_start_and_end_for_granularity(start: int, end: int, granularity: str)
 
 
 class DateTimeAligner(ABC):
-    _zeros_upto_hour = dict(hour=0, minute=0, second=0, microsecond=0)
+    @staticmethod
+    def normalize(date: datetime) -> datetime:
+        return date.replace(hour=0, minute=0, second=0, microsecond=0)
 
     @classmethod
     @abstractmethod
@@ -225,11 +245,11 @@ class DateTimeAligner(ABC):
 class DayAligner(DateTimeAligner):
     @classmethod
     def floor(cls, date: datetime) -> datetime:
-        return date.replace(**cls._zeros_upto_hour)  # type: ignore [arg-type]
+        return cls.normalize(date)
 
     @classmethod
     def ceil(cls, date: datetime) -> datetime:
-        return date.replace(**cls._zeros_upto_hour) + timedelta(days=1)  # type: ignore [arg-type]
+        return cls.normalize(date) + timedelta(days=1)
 
     @classmethod
     def units_between(cls, start: datetime, end: datetime) -> int:
@@ -248,7 +268,7 @@ class WeekAligner(DateTimeAligner):
         >>> WeekAligner.ceil(datetime(2023, 4, 9 ))
         datetime.datetime(2023, 4, 10, 0, 0)
         """
-        date = date.replace(**cls._zeros_upto_hour)  # type: ignore [arg-type]
+        date = cls.normalize(date)
         if (weekday := date.weekday()) != 0:
             return date + timedelta(days=7 - weekday)
         return date
@@ -260,7 +280,7 @@ class WeekAligner(DateTimeAligner):
         >>> WeekAligner.floor(datetime(2023, 4, 9))
         datetime.datetime(2023, 4, 3, 0, 0)
         """
-        date = date.replace(**cls._zeros_upto_hour)  # type: ignore [arg-type]
+        date = cls.normalize(date)
         if (weekday := date.weekday()) != 0:
             return date - timedelta(days=weekday)
         return date
@@ -292,11 +312,11 @@ class MonthAligner(DateTimeAligner):
         if date == datetime(year=date.year, month=date.month, day=1, tzinfo=date.tzinfo):
             return date
         extra, month = divmod(date.month + 1, 12)
-        return date.replace(year=date.year + extra, month=month, day=1, **cls._zeros_upto_hour)  # type: ignore [arg-type]
+        return cls.normalize(date.replace(year=date.year + extra, month=month, day=1))
 
     @classmethod
     def floor(cls, date: datetime) -> datetime:
-        return date.replace(day=1, **cls._zeros_upto_hour)  # type: ignore [arg-type]
+        return cls.normalize(date.replace(day=1))
 
     @classmethod
     def units_between(cls, start: datetime, end: datetime) -> int:
@@ -325,7 +345,7 @@ class QuarterAligner(DateTimeAligner):
             return date
         month = 3 * ((date.month - 1) // 3 + 1) + 1
         add_years, month = divmod(month, 12)
-        return date.replace(year=date.year + add_years, month=month, day=1, **cls._zeros_upto_hour)  # type: ignore [arg-type]
+        return cls.normalize(date.replace(year=date.year + add_years, month=month, day=1))
 
     @classmethod
     def floor(cls, date: datetime) -> datetime:
@@ -339,7 +359,7 @@ class QuarterAligner(DateTimeAligner):
         datetime.datetime(2023, 10, 1, 0, 0)
         """
         month = 3 * ((date.month - 1) // 3) + 1
-        return date.replace(month=month, day=1, **cls._zeros_upto_hour)  # type: ignore [arg-type]
+        return cls.normalize(date.replace(month=month, day=1))
 
     @classmethod
     def units_between(cls, start: datetime, end: datetime) -> int:
@@ -356,11 +376,11 @@ class YearAligner(DateTimeAligner):
     def ceil(cls, date: datetime) -> datetime:
         if date == datetime(date.year, 1, 1, tzinfo=date.tzinfo):
             return date
-        return date.replace(year=date.year + 1, month=1, day=1, **cls._zeros_upto_hour)  # type: ignore [arg-type]
+        return cls.normalize(date.replace(year=date.year + 1, month=1, day=1))
 
     @classmethod
     def floor(cls, date: datetime) -> datetime:
-        return date.replace(month=1, day=1, **cls._zeros_upto_hour)  # type: ignore [arg-type]
+        return cls.normalize(date.replace(month=1, day=1))
 
     @classmethod
     def units_between(cls, start: datetime, end: datetime) -> int:
@@ -377,12 +397,12 @@ def align_large_granularity(start: datetime, end: datetime, granularity: str) ->
     This is done to get consistent behavior with the Cognite Datapoints API.
 
     Args:
-        start: Start time
-        end: End time
-        granularity: The large granularity, day|week|month|quarter|year.
+        start (datetime): Start time
+        end (datetime): End time
+        granularity (str): The large granularity, day|week|month|quarter|year.
 
     Returns:
-        start and end aligned with granularity
+        tuple[datetime, datetime]: start and end aligned with granularity
     """
     multiplier, unit = get_granularity_multiplier_and_unit(granularity)
     # Can be replaced by a single dispatch pattern, but kept more explicit for readability.
@@ -404,7 +424,7 @@ def align_large_granularity(start: datetime, end: datetime, granularity: str) ->
     return start, end
 
 
-def split_time_range(start: int, end: int, n_splits: int, granularity_in_ms: int) -> List[int]:
+def split_time_range(start: int, end: int, n_splits: int, granularity_in_ms: int) -> list[int]:
     if n_splits < 1:
         raise ValueError(f"Cannot split into less than 1 piece, got {n_splits=}")
     tot_ms = end - start
@@ -484,30 +504,28 @@ def _to_fixed_utc_intervals_fixed_unit_length(
     utc = get_utc_zoneinfo()
 
     freq = multiplier * GRANULARITY_IN_HOURS[unit]
-    index = pandas_date_range_tz(start, end, f"{freq}H")
-    expected_freq = pd.Timedelta(hours=freq)
-    next_mask = (index - index.to_series().shift(1)) != expected_freq
-    last_mask = (index.to_series().shift(-1) - index) != expected_freq
-    index = index[last_mask | next_mask]
+    index = pandas_date_range_tz(start, end, to_pandas_freq(f"{multiplier}{unit}", start))
+    utc_offsets = pd.Series([t.utcoffset() for t in index], index=index)
+    transition_raw = index[(utc_offsets != utc_offsets.shift(-1)) | (utc_offsets != utc_offsets.shift(1))]
 
     hour, zero = pd.Timedelta(hours=1), pd.Timedelta(0)
     transitions = []
-    for start, end in zip(index[:-1], index[1:]):
-        if start.dst() == end.dst():
+    for t_start, t_end in zip(transition_raw[:-1], transition_raw[1:]):
+        if t_start.dst() == t_end.dst():
             dst_adjustment = 0
-        elif start.dst() == hour and end.dst() == zero:
-            # Fall, going away from summer.
+        elif t_start.dst() == hour and t_end.dst() == zero:
+            # Fall, going away from summer (above the equator).
             dst_adjustment = 1
-        elif start.dst() == zero and end.dst() == hour:
-            # Spring, going to summer.
+        elif t_start.dst() == zero and t_end.dst() == hour:
+            # Spring, going to summer (above the equator).
             dst_adjustment = -1
         else:
-            raise ValueError(f"Invalid dst, {start} and {end}")
+            raise ValueError(f"Invalid dst, {t_start} and {t_end}")
 
         transitions.append(
             {
-                "start": start.to_pydatetime().astimezone(utc),  # type: ignore
-                "end": end.to_pydatetime().astimezone(utc),  # type: ignore
+                "start": t_start.to_pydatetime().astimezone(utc),
+                "end": t_end.to_pydatetime().astimezone(utc),
                 "granularity": f"{freq+dst_adjustment}h",
             }
         )
@@ -518,27 +536,72 @@ def pandas_date_range_tz(start: datetime, end: datetime, freq: str, inclusive: s
     """
     Pandas date_range struggles with time zone aware datetimes.
     This function overcomes that limitation.
+
+    Assumes that start and end have the same timezone.
     """
-    pd = local_import("pandas")
-    # Pandas seems to have issues with ZoneInfo object, so removing the timezone and adding it back.
-    return pd.date_range(start.replace(tzinfo=None), end.replace(tzinfo=None), freq=freq, inclusive=inclusive).tz_localize(start.tzinfo.key)  # type: ignore [union-attr]
+    pd = cast(Any, local_import("pandas"))
+    # There is a bug in date_range which makes it fail to handle ambiguous timestamps when you use time zone aware
+    # datetimes. This is a workaround by passing the time zone as an argument to the function.
+    # In addition, pandas struggle with ZoneInfo objects, so we convert them to string so that pandas can use its own
+    # tzdata implementation.
+
+    # An ambiguous timestamp is for example 1916-10-01 00:00:00 Europe/Oslo, as this corresponds to two different points in time,
+    # 1916-10-01 00:00:00+02:00 and 1916-10-01 00:00:00+01:00; before and after the DST transition.
+    # (Back in 1916 they did not consider the needs of software engineers in 2023 :P).
+    # Setting ambiguous=True will make pandas ignore the ambiguity and use the DST timestamp. This is what we want;
+    # for a user requesting monthly aggregates, we don't want to miss the first hour of the month.
+    return pd.date_range(
+        start.replace(tzinfo=None),
+        end.replace(tzinfo=None),
+        tz=str(start.tzinfo),
+        freq=freq,
+        inclusive=inclusive,
+        nonexistent="shift_forward",
+        ambiguous=True,
+    )
+
+
+def _timezones_are_equal(start_tz: tzinfo, end_tz: tzinfo) -> bool:
+    """There are unfortunately several ways to pass/represent the same timezone (without it being a user error).
+    For example pandas uses 'pytz' under the hood -except- for UTC, then it uses the built-in `datetime.timezone.utc`
+    -except- when given something concrete like pytz.UTC or ZoneInfo(...).
+
+    To make sure we don't raise something silly like 'UTC != UTC', we convert both to ZoneInfo for comparison
+    via str(). This is safe as all return the lookup key (for the IANA time zone database).
+
+    Note:
+        We do not consider timezones with different keys, but equal fixed offsets from UTC to be equal. An example
+        would be Zulu Time (which is +00:00 ahead of UTC) and UTC.
+    """
+    if start_tz is end_tz:
+        return True
+    ZoneInfo, ZoneInfoNotFoundError = import_zoneinfo(), _import_zoneinfo_not_found_error()
+    with suppress(ValueError, ZoneInfoNotFoundError):
+        # ValueError is raised for non-conforming keys (ZoneInfoNotFoundError is self-explanatory)
+        if ZoneInfo(str(start_tz)) is ZoneInfo(str(end_tz)):
+            return True
+    return False
 
 
 def validate_timezone(start: datetime, end: datetime) -> ZoneInfo:
-    ZoneInfo = import_zoneinfo()
-
-    if missing := [name for name, timestamp in zip(("start", "end"), (start, end)) if not timestamp.tzinfo]:
+    if (start_tz := start.tzinfo) is None or (end_tz := end.tzinfo) is None:
+        missing = [name for name, timestamp in zip(("start", "end"), (start, end)) if not timestamp.tzinfo]
         names = " and ".join(missing)
         end_sentence = " do not have timezones." if len(missing) >= 2 else " does not have a timezone."
-        raise ValueError(f"All times must be time zone aware, {names}{end_sentence}")
+        raise ValueError(f"All times must be timezone aware, {names}{end_sentence}")
 
-    if not isinstance(start.tzinfo, ZoneInfo) or not isinstance(end.tzinfo, ZoneInfo):  # type: ignore [arg-type]
-        raise ValueError("Only ZoneInfo implementation of tzinfo is supported")
+    if not _timezones_are_equal(start_tz, end_tz):
+        raise ValueError(f"'start' and 'end' represent different timezones: '{start_tz}' and '{end_tz}'.")
 
-    if start.tzinfo is not end.tzinfo:
-        raise ValueError(f"start and end have different timezones, {start.tzinfo.key!r} and {end.tzinfo.key!r}.")  # type: ignore [union-attr]
+    ZoneInfo = import_zoneinfo()
+    if isinstance(start_tz, ZoneInfo):
+        return start_tz
 
-    return start.tzinfo  # type: ignore [return-value]
+    pd = cast(Any, local_import("pandas"))
+    if isinstance(start, pd.Timestamp):
+        return ZoneInfo(str(start_tz))
+
+    raise ValueError("Only tz-aware pandas.Timestamp and datetime (must be using ZoneInfo) are supported.")
 
 
 def to_pandas_freq(granularity: str, start: datetime) -> str:
@@ -586,12 +649,11 @@ def in_timedelta(granularity: str, ceil: bool = True) -> timedelta:
     Converts the granularity to a timedelta.
 
     Args:
-        granularity: The granularity.
-        ceil: In the case the unit is month, quarter or year. Ceil = True will use 31, 92, 366 days for these
-              timespans, and if ceil is false 28, 91, 365
+        granularity (str): The granularity.
+        ceil (bool): In the case the unit is month, quarter or year. Ceil = True will use 31, 92, 366 days for these timespans, and if ceil is false 28, 91, 365
 
     Returns:
-        A timespan for the granularity
+        timedelta: A timespan for the granularity
     """
     multiplier, unit = get_granularity_multiplier_and_unit(granularity, standardize=True)
     if unit in {"w", "d", "h", "m", "s"}:
